Package: limpca
Type: Package
Title: An R package for the linear modeling of high-dimensional
        designed data based on ASCA/APCA family of methods
Version: 1.8.0
Authors@R: c(person("Bernadette", "Govaerts", role = c("aut", "ths"),
                     email = "bernadette.govaerts@uclouvain.be"),
              person("Sebastien","Franceschini", role = "ctb",
                    email="sfranceschini@uliege.be"),
              person("Robin","van Oirbeek", role = "ctb",
                    email="robin.vanoirbeek@gmail.com"),
              person("Michel","Thiel", role = "aut",
                    email="michel.thiel@uclouvain.be"),
              person("Pascal","de Tullio", role = "dtc",
                    email="pdetullio@uliege.be"),
              person("Manon","Martin", role = c("aut", "cre"),
                    email="manon.martin@uclouvain.be",
                    comment = c(ORCID = "0000-0003-4800-0942")),
              person("Nadia", "Benaiche", role = "ctb",
                     email = "nadia.benaiche@student.uclouvain.be"))
Description: This package has for objectives to provide a method to
        make Linear Models for high-dimensional designed data. limpca
        applies a GLM (General Linear Model) version of ASCA and APCA
        to analyse multivariate sample profiles generated by an
        experimental design. ASCA/APCA provide powerful visualization
        tools for multivariate structures in the space of each effect
        of the statistical model linked to the experimental design and
        contrarily to MANOVA, it can deal with mutlivariate datasets
        having more variables than observations. This method can handle
        unbalanced design.
License: Artistic-2.0
Encoding: UTF-8
LazyData: FALSE
VignetteBuilder: knitr
Imports: ggplot2, stringr, plyr, ggrepel, reshape2, grDevices,
        graphics, doParallel, parallel, dplyr, tibble, tidyr, ggsci,
        tidyverse, methods, stats, SummarizedExperiment, S4Vectors
Suggests: BiocStyle, pander, rmarkdown, car, gridExtra, knitr, testthat
        (>= 3.0.0)
biocViews: StatisticalMethod, PrincipalComponent, Regression,
        Visualization, ExperimentalDesign, MultipleComparison,
        GeneExpression, Metabolomics
RoxygenNote: 7.3.3
Roxygen: list(markdown=TRUE)
BugReports: https://github.com/ManonMartin/limpca/issues
URL: https://github.com/ManonMartin/limpca,
        https://manonmartin.github.io/limpca/
Config/testthat/edition: 3
Config/pak/sysreqs: cmake libfontconfig1-dev libfreetype6-dev
        libfribidi-dev make libharfbuzz-dev libicu-dev libjpeg-dev
        libpng-dev libtiff-dev libuv1-dev libwebp-dev libxml2-dev
        libssl-dev libx11-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:03:08 UTC
RemoteUrl: https://github.com/bioc/limpca
RemoteRef: RELEASE_3_23
RemoteSha: 6ef673cca64c39757e06f412054fa883e70cac52
NeedsCompilation: no
Packaged: 2026-04-29 23:10:22 UTC; root
Author: Bernadette Govaerts [aut, ths],
  Sebastien Franceschini [ctb],
  Robin van Oirbeek [ctb],
  Michel Thiel [aut],
  Pascal de Tullio [dtc],
  Manon Martin [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4800-0942>),
  Nadia Benaiche [ctb]
Maintainer: Manon Martin <manon.martin@uclouvain.be>
Depends: R (>= 3.5.0)
Built: R 4.6.0; ; 2026-04-30 00:21:03 UTC; unix

# Generated by roxygen2: do not edit by hand

export(data2LmpDataList)
export(lmpBootstrapTests)
export(lmpContributions)
export(lmpEffectMatrices)
export(lmpEffectPlot)
export(lmpLoading1dPlot)
export(lmpLoading2dPlot)
export(lmpModelMatrix)
export(lmpPcaEffects)
export(lmpScorePlot)
export(lmpScoreScatterPlotM)
export(lmpScreePlot)
export(pcaBySvd)
export(pcaLoading1dPlot)
export(pcaLoading2dPlot)
export(pcaScorePlot)
export(pcaScreePlot)
export(plotDesign)
export(plotLine)
export(plotMeans)
export(plotScatter)
export(plotScatterM)
import(SummarizedExperiment)
import(doParallel)
import(dplyr)
import(ggplot2)
import(ggrepel)
import(ggsci)
import(grDevices)
import(graphics)
import(parallel)
import(reshape2)
import(stringr)
import(tibble)
import(tidyr)
import(tidyverse)
importFrom(S4Vectors,metadata)
importFrom(methods,hasArg)
importFrom(methods,is)
importFrom(plyr,aaply)
importFrom(plyr,ddply)
importFrom(plyr,is.formula)
importFrom(plyr,laply)
importFrom(plyr,llply)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,model.matrix)

limpca
================

<!-- badges: start -->

[![R-CMD-check](https://github.com/ManonMartin/limpca/actions/workflows/check-standard.yaml/badge.svg)](https://github.com/ManonMartin/limpca/actions/workflows/check-standard.yaml)
<!-- badges: end -->

The web page of the package can be accessed here:
<https://manonmartin.github.io/limpca/>

`limpca` applies a GLM (General Linear Model) version of ASCA and APCA
to analyse multivariate sample profiles generated by an experimental
design. ASCA/APCA provide powerful visualization tools for multivariate
structures in the space of each effect of the statistical model linked
to the experimental design and contrarily to MANOVA, it can deal with
mutlivariate datasets having more variables than observations. `limpca`
presents different advantages compared to other software in this field:
(1) it is able to treat any balanced or unbalanced experimental design
for fixed categorical factors, (2) it offers optimized methods to
calculate effect importance and test their significance, (3) it allows
the user to represent data and ASCA/APCA results with various and rich
ggplot2-based graphical outputs that are highly customizable and (4) the
package is open to future extensions to more sophisticated statistical
models.

## Installation

Installation from Bioconductor:

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
    
BiocManager::install("limpca")
```


## Additional information

For any enquiry, you can open an
[issue](https://github.com/ManonMartin/limpca/issues) on Github or send
an email to the package authors: <bernadette.govaerts@uclouvain.be> ;
<michel.thiel@uclouvain.be> or <manon.martin@uclouvain.be>.

## ----setup,include=FALSE------------------------------------------------------
# Install package if necessary (to be used before running the vignette)
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("limpca")

library(ggplot2)
library(limpca)
if (!requireNamespace("gridExtra", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(gridExtra)

if (!requireNamespace("pander", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(pander)
if (!requireNamespace("car", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(car)


knitr::opts_chunk$set(
    message = FALSE, warning = TRUE,
    comment = NA, crop = NULL,
    width = 60, dpi = 50,
    fig.width = 5,
    fig.height = 3.5, dev = "png"
)

## ----eval=FALSE---------------------------------------------------------------
# library(car)
# library(pander)
# library(gridExtra)
# library(ggplot2)

## ----Install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("limpca")

## ----Load, eval=FALSE---------------------------------------------------------
# library("limpca")

## ----Data Importation---------------------------------------------------------
data("trout")
# print number of and response names
cat("\n Nb of Responses :  ", ncol(trout$outcomes), "\n ")
cat("\nResponses :\n", colnames(trout$outcomes), "\n ")
# Order responses by alphabetic order
trout$outcomes <- trout$outcomes[, order(dimnames(trout$outcomes)[[2]])]
cat("\n Ordered responses :\n  ", colnames(trout$outcomes), "\n ")
# print factor names
cat("\nDesign factors :  ", colnames(trout$design), "\n ")
# plot the design with plotDesign function
limpca::plotDesign(
    design = trout$design, x = "Treatment",
    y = "Day", cols = "Exposure",
    title = "Initial design of the trout dataset"
)

## ----Exploratory PCA----------------------------------------------------------
resPCA <- limpca::pcaBySvd(trout$outcomes)
limpca::pcaScreePlot(resPCA, nPC = 8)
limpca::pcaScorePlot(
    resPcaBySvd = resPCA, axes = c(1, 2),
    title = "Score plot of original data ",
    design = trout$design, color = "Aquarium",
    points_labs_rn = TRUE
)

## ----Transformation-----------------------------------------------------------
# Log Transformation
trout_log <- trout
trout_log$outcomes <- as.matrix(log10(trout$outcomes))

# new PCA
resPCA1 <- limpca::pcaBySvd(trout_log$outcomes)
limpca::pcaScreePlot(resPCA1, nPC = 8)
limpca::pcaScorePlot(
    resPcaBySvd = resPCA1, axes = c(1, 2),
    title = "Score plot of Log10 data ",
    design = trout_log$design, color = "Aquarium",
    drawShapes = "polygon", points_labs_rn = TRUE
)

## ----Remove Outliers----------------------------------------------------------
# Remove outliers and create new dataset
trout_clean <- trout_log
outliers <- match(
    c("D72EPA2.1", "D28EPA2.2"),
    rownames(trout_log$outcomes)
)
trout_clean$outcomes <- trout_log$outcomes[-outliers, ]
trout_clean$design <- trout_log$design[-outliers, ]

## ----new PCA------------------------------------------------------------------
# PCA
resPCA2 <- limpca::pcaBySvd(trout_clean$outcomes)
limpca::pcaScreePlot(resPCA2, nPC = 8)
# Score plot Components 1 and 2
limpca::pcaScorePlot(
    resPcaBySvd = resPCA2, axes = c(1, 2),
    title = "Score plot of Log10 data without outliers (PC 1&2)",
    design = trout_clean$design, color = "Aquarium",
    drawShapes = "polygon",
    points_labs_rn = FALSE
)
# Score plot Components 3 and 4
limpca::pcaScorePlot(
    resPcaBySvd = resPCA2, axes = c(3, 4),
    title = "Score plot of Log10 data without outliers (PC 3&4)",
    design = trout_clean$design, color = "Aquarium",
    drawShapes = "polygon", points_labs_rn = FALSE
)

## ----Mean aggregation---------------------------------------------------------
# Mean aggregation
mean_outcomes <- matrix(0, nrow = 24, ncol = 15)
mean_design <- matrix(0, nrow = 24, ncol = 3)
y <- list(
    trout_clean$design[["Day"]],
    trout_clean$design[["Treatment"]],
    trout_clean$design[["Exposure"]]
)
for (i in 1:15) {
    mean_outcomes[, i] <- aggregate(trout_clean$outcomes[, i], by = y, mean)[, 4]
}
mean_design <- aggregate(trout_clean$outcomes[, 1], by = y, mean)[, c(1:3)]

# Set row and col names
colnames(mean_outcomes) <- colnames(trout_clean$outcomes)
colnames(mean_design) <- colnames(trout_clean$design)[1:3]
trout_mean_names <- apply(mean_design, 1, paste, collapse = "")
rownames(mean_outcomes) <- trout_mean_names
rownames(mean_design) <- trout_mean_names

## ----Centering and Scaling----------------------------------------------------
# Outcomes centering and Scaling
mean_outcomes <- scale(mean_outcomes, center = TRUE, scale = TRUE)
# New data object creation
trout_mean <- list(
    "outcomes" = mean_outcomes,
    "design" = mean_design,
    "formula" = trout$formula
)
# Clean objects
rm(
    resPCA, resPCA1, resPCA2, y, mean_design, mean_outcomes,
    trout_mean_names
)

## ----Design-------------------------------------------------------------------
pander(head(trout_mean$design))
limpca::plotDesign(
    design = trout_mean$design,
    title = "Design of mean aggregated trout dataset"
)

## ----Lineplot-----------------------------------------------------------------
limpca::plotLine(trout_mean$outcomes,
    rows = c(1, 24),
    xaxis_type = "character", type = "s"
) +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))

## ----PCA----------------------------------------------------------------------
resPCA_mean <- limpca::pcaBySvd(trout_mean$outcomes)
pcaScreePlot(resPCA_mean, nPC = 6)

## -----------------------------------------------------------------------------
limpca::pcaScorePlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA score plot by Exposure and Day",
    design = trout_mean$design,
    shape = "Exposure", color = "Day",
    points_labs_rn = FALSE
)

limpca::pcaScorePlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA scores plot by Treatment and Day",
    design = trout_mean$design,
    shape = "Day", color = "Treatment",
    points_labs_rn = FALSE
)

limpca::pcaScorePlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA scores plot by Exposure and Treatment",
    design = trout_mean$design,
    shape = "Treatment", color = "Exposure",
    points_labs_rn = FALSE
)

## -----------------------------------------------------------------------------
limpca::pcaLoading1dPlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA loadings plot trout", xlab = " ",
    ylab = "Expression", xaxis_type = "character", type = "s"
) +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))

## -----------------------------------------------------------------------------
limpca::pcaLoading2dPlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA loadings plot trout", addRownames = TRUE
)

## ----Scatterplot Matrix,fig.width=10,fig.height=10----------------------------
limpca::plotScatterM(
    Y = trout_mean$outcomes, cols = c(1:15),
    design = trout_mean$design,
    varname.colorup = "Day",
    vec.colorup = c("CadetBlue4", "pink", "orange"),
    varname.colordown = "Day",
    vec.colordown = c("CadetBlue4", "pink", "orange"),
    varname.pchup = "Treatment",
    varname.pchdown = "Exposure"
)

## ----ModelMatrix--------------------------------------------------------------
resLmpModelMatrix <- limpca::lmpModelMatrix(trout_mean)
pander::pander(head(resLmpModelMatrix$modelMatrix))

## ----EffectMatrices-----------------------------------------------------------
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resLmpEffectMatrices$varPercentagesPlot

## ----Bootstrap----------------------------------------------------------------
resLmpBootstrapTests <- lmpBootstrapTests(
    resLmpEffectMatrices = resLmpEffectMatrices,
    nboot = 1000
)

# Print p-values
pander::pander(t(resLmpBootstrapTests$resultsTable))

## ----ASCA PCA-----------------------------------------------------------------
resASCA <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "ASCA",
    combineEffects = list(c(
        "Day", "Treatment",
        "Day:Treatment"
    ))
)

## ----ASCA Contrib-------------------------------------------------------------
resLmpContributions <- lmpContributions(resASCA)

## ----ASCA ContribP------------------------------------------------------------
pander::pander(resLmpContributions$totalContribTable)
pander::pander(resLmpContributions$effectTable)
pander::pander(resLmpContributions$contribTable)
pander::pander(resLmpContributions$combinedEffectTable)

## Visualize the more important contributions
resLmpContributions$plotContrib

## ----ASCA Day-----------------------------------------------------------------
A <- lmpScorePlot(resASCA,
    effectNames = "Day",
    color = "Day", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Day",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)

## ----ASCA Treatment-----------------------------------------------------------
A <- lmpScorePlot(resASCA,
    effectNames = "Treatment",
    color = "Treatment", shape = "Treatment"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Treatment",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)

## ----ASCA DayTreatment--------------------------------------------------------
A <- lmpScorePlot(resASCA,
    effectNames = "Day:Treatment",
    color = "Treatment", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Day:Treatment",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)

## ----ASCA Combined effect-----------------------------------------------------
A <- lmpScorePlot(resASCA,
    effectNames = "Day+Treatment+Day:Treatment",
    color = "Treatment", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Day+Treatment+Day:Treatment",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)

## ----ASCA Residuals-----------------------------------------------------------
A <- lmpScorePlot(resASCA,
    effectNames = "Residuals",
    color = "Treatment", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Residuals",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)

## ----ASCA Effects-------------------------------------------------------------
A <- lmpEffectPlot(resASCA,
    effectName = "Day:Treatment",
    x = "Day", z = "Treatment", axes = c(1, 2)
)
A$PC1 <- A$PC1 + ggtitle("PC1: Day:Treatment effect alone")
A$PC2 <- A$PC2 + ggtitle("PC2: Day:Treatment effect alone")
grid.arrange(A$PC1, A$PC2, ncol = 2)

A <- lmpEffectPlot(resASCA,
    effectName = "Day+Treatment+Day:Treatment",
    x = "Day", z = "Treatment", axes = c(1, 2)
)
A$PC1 <- A$PC1 + ggtitle("PC1: Combined D+T+D:T effects")
A$PC2 <- A$PC2 + ggtitle("PC2: Combined D+T+D:T effects")
grid.arrange(A$PC1, A$PC2, ncol = 2)

## ----APCA PCA-----------------------------------------------------------------
resAPCA <- lmpPcaEffects(
    resLmpEffectMatrices =
        resLmpEffectMatrices, method = "APCA"
)

## ----APCA Scores--------------------------------------------------------------
# Day Effect
lmpScorePlot(resAPCA,
    effectNames = "Day",
    color = "Day", shape = "Day", drawShapes = "ellipse"
)

# Treatment Effect
lmpScorePlot(resAPCA,
    effectNames = "Treatment",
    color = "Treatment", shape = "Treatment", drawShapes = "ellipse"
)


# Exposure Effect
lmpScorePlot(resAPCA,
    effectNames = "Exposure",
    color = "Exposure", shape = "Exposure", drawShapes = "ellipse"
)

# Day:Treatment Effect
lmpScorePlot(resAPCA,
    effectNames = "Day:Treatment",
    color = "Treatment", shape = "Day", drawShapes = "polygon"
)

## ----Anova--------------------------------------------------------------------
# Creation of a matrix to store the p-values
m <- ncol(trout_mean$outcomes)
mat_pval <- matrix(nrow = m, ncol = 6)
dimnames(mat_pval) <- list(
    dimnames(trout_mean$outcomes)[[2]],
    c(
        "Day", "Treatment", "Exposure", "Day:Treatment",
        "Day:Exposure", "Treatment:Exposure"
    )
)

# Parallel ANOVA modeling
for (i in 1:m) {
    data <- cbind(y = trout_mean$outcomes[, i], trout_mean$design)
    Modl <- lm(y ~ Day + Treatment + Exposure + Day:Treatment + Day:Exposure + Treatment:Exposure,
        contrasts = list(Day = contr.sum, Treatment = contr.sum, Exposure = contr.sum),
        data = data
    )
    tabanova <- Anova(Modl, type = 3)
    mat_pval[i, ] <- tabanova[2:7, 4]
}

# FDR p-values correction
for (i in 1:6) mat_pval[, i] <- p.adjust(mat_pval[, i], method = "BH")

## -----------------------------------------------------------------------------
pander(mat_pval)
heatmap(-log10(mat_pval),
    Rowv = NA, Colv = "Rowv",
    cexCol = 0.8, scale = "none", main = "Heatmap of -log10(q-values)"
)

## ----eval=TRUE----------------------------------------------------------------
Effects <- c("Day", "Treatment", "Day:Treatment")

for (i in 1:3) {
    Pval_log <- -log10(mat_pval[, Effects[i]])
    resA <- resASCA[[Effects[i]]]
    PC12Load <- as.vector(sqrt(resA$loadings[, 1:2]^2 %*% resA$singvar[1:2]^2))
    matres <- cbind(PC12Load, Pval_log)
    A[[i]] <- plotScatter(
        Y = matres, xy = c(1, 2),
        points_labs = rownames(matres),
        xlab = "PC1&2 ASCA loadings", ylab = "-log10(p-values)",
        title = paste(Effects[i], "effect")
    )
}

A[[1]]
A[[2]]
A[[3]]

## -----------------------------------------------------------------------------
sessionInfo()

## ----Setup,include=FALSE------------------------------------------------------
# require(knitr)
knitr::opts_chunk$set(
    message = FALSE, warning = TRUE, comment = NA,
    crop = NULL,
    tidy.opts = list(width.cutoff = 60),
    tidy = TRUE, dpi = 50, dev = "png",
    fig.width = 5, fig.height = 3.5
)
rm(list = ls())

if (!requireNamespace("pander", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(pander)

library(ggplot2)
if (!requireNamespace("gridExtra", quietly = TRUE)) {
    stop("Install 'gridExtra' to knit this vignette")
}
library(gridExtra)

if (!requireNamespace("SummarizedExperiment", quietly = TRUE)) {
    stop("Install 'SummarizedExperiment' to knit this vignette")
}
library(SummarizedExperiment)

## ----eval=FALSE---------------------------------------------------------------
# 
# library(pander)
# library(gridExtra)
# library(ggplot2)
# library(SummarizedExperiment)

## ----Install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("limpca")

## ----Load, results=FALSE, message=FALSE---------------------------------------
library("limpca")

## ----Data_Importation---------------------------------------------------------
data("UCH")

str(UCH)

## ----message = TRUE-----------------------------------------------------------
UCH2 <- data2LmpDataList(
   outcomes = UCH$outcomes,
   design = UCH$design, 
   formula = UCH$formula
 )

## ----message = TRUE-----------------------------------------------------------
se <- SummarizedExperiment(
   assays = list(
     counts = t(UCH$outcomes)), colData = UCH$design,
   metadata = list(formula = UCH$formula)
 )

UCH3 <- data2LmpDataList(se, assay_name = "counts")

## ----Design_visualization-----------------------------------------------------
pander(head(UCH$design))
plotDesign(
    design = UCH$design, x = "Hippurate",
    y = "Citrate", rows = "Time",
    title = "Design of the UCH dataset"
)

## ----Spectrum_visualization---------------------------------------------------
p1 <- plotLine(
    Y = UCH$outcomes,
    title = "H-NMR spectrum",
    rows = c(3),
    xlab = "ppm",
    ylab = "Intensity"
)

cit_peaks <- annotate("rect",
    xmin = c(2.509), xmax = c(2.709),
    ymin = -Inf, ymax = Inf, alpha = 0.2,
    fill = c("tomato")
)

hip_peaks <- annotate("rect",
    xmin = c(7.458, 3.881), xmax = c(7.935, 4.041),
    ymin = -Inf, ymax = Inf, alpha = 0.2,
    fill = c("yellowgreen")
)

p1 <- plotLine(
    Y = UCH$outcomes,
    title = "H-NMR spectrum",
    rows = c(3),
    xlab = "ppm",
    ylab = "Intensity"
)

p1 + cit_peaks + hip_peaks

## -----------------------------------------------------------------------------
# xy corresponds to citrate (453) and hippurate peaks (369)
plotScatter(
    Y = UCH$outcomes,
    xy = c(453, 369),
    design = UCH$design,
    color = "Hippurate",
    shape = "Citrate"
)
# Or refering to the variables names (ppm values here)
plotScatter(
    Y = UCH$outcomes,
    xy = c("2.6092056", "3.9811536"),
    design = UCH$design,
    color = "Hippurate",
    shape = "Citrate"
)

## ----fig.width=6,fig.height=7-------------------------------------------------
plotScatterM(
    Y = UCH$outcomes, cols = c(133, 145, 150, 369, 453),
    design = UCH$design, varname.colorup = "Hippurate",
    varname.colordown = "Citrate"
)

## -----------------------------------------------------------------------------
plotMeans(
    Y = UCH$outcomes,
    design = UCH$design,
    cols = c(453),
    x = c("Citrate"),
    w = c("Hippurate"),
    z = c("Time"),
    ylab = "Intensity",
    title = c("Mean reponse for main Citrate peak")
)

## ----PCA----------------------------------------------------------------------
ResPCA <- pcaBySvd(UCH$outcomes)
pcaScreePlot(ResPCA, nPC = 6)

## ----Scores-------------------------------------------------------------------
pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA scores plot: PC1 and PC2",
    design = UCH$design,
    color = "Hippurate", shape = "Citrate",
    points_labs_rn = FALSE
)

pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA scores plot: PC1 and PC2",
    design = UCH$design,
    color = "Time", shape = "Hippurate",
    points_labs_rn = FALSE
)

pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(3, 4),
    title = "PCA scores plot: PC3 and PC4",
    design = UCH$design,
    color = "Time", shape = "Citrate",
    points_labs_rn = FALSE
)

## ----Loadings-----------------------------------------------------------------
p2 <- pcaLoading1dPlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA loadings plot UCH", xlab = "ppm",
    ylab = "Intensity"
)

p2 + hip_peaks + cit_peaks

## ----Formula------------------------------------------------------------------
UCH$formula <- "outcomes ~ Hippurate + Citrate + Time + Hippurate:Citrate +
                  Time:Hippurate + Time:Citrate + Hippurate:Citrate:Time"

## ----ModelMatrix--------------------------------------------------------------
resLmpModelMatrix <- lmpModelMatrix(UCH)
pander::pander(head(resLmpModelMatrix$modelMatrix))

## ----EffectMatrices-----------------------------------------------------------
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)

## -----------------------------------------------------------------------------
pander(resLmpEffectMatrices$variationPercentages)
resLmpEffectMatrices$varPercentagesPlot

## ----Bootstrap----------------------------------------------------------------
resLmpBootstrapTests <-
    lmpBootstrapTests(
        resLmpEffectMatrices = resLmpEffectMatrices,
        nboot = 100
    )

# Print P-values
pander::pander(t(resLmpBootstrapTests$resultsTable))

## ----ASCA_PCA-----------------------------------------------------------------
resASCA <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "ASCA",
    combineEffects = list(c(
        "Hippurate", "Time",
        "Hippurate:Time"
    ))
)

## ----ASCA_Contrib-------------------------------------------------------------
resLmpContributions <- lmpContributions(resASCA)

## -----------------------------------------------------------------------------
pander::pander(resLmpContributions$totalContribTable)

## -----------------------------------------------------------------------------
pander::pander(resLmpContributions$effectTable)

## -----------------------------------------------------------------------------
pander::pander(resLmpContributions$combinedEffectTable)

## -----------------------------------------------------------------------------
pander::pander(resLmpContributions$contribTable)

## -----------------------------------------------------------------------------
pander("Ordered contributions per effect:")
resLmpContributions$plotTotal

pander("For each PC of the different effects:")
resLmpContributions$plotContrib

## ----include=TRUE-------------------------------------------------------------
all_loadings_pl <- lmpLoading1dPlot(resASCA,
    effectNames = c(
        "Hippurate", "Citrate", "Time",
        "Hippurate:Time",
        "Hippurate+Time+Hippurate:Time",
        "Residuals"
    ),
    axes = 1, xlab = "ppm"
)

## ----ASCA_ScoresXY------------------------------------------------------------
# Hippurate
hip_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate"
)

hip_loadings_pl <- all_loadings_pl$Hippurate + hip_peaks

grid.arrange(hip_scores_pl, hip_loadings_pl, ncol = 2)

# Citrate
cit_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Citrate",
    color = "Citrate", shape = "Citrate"
)
cit_loadings_pl <- all_loadings_pl$Citrate + cit_peaks

grid.arrange(cit_scores_pl, cit_loadings_pl, ncol = 2)

# Time
tim_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Time", color = "Time",
    shape = "Time"
)

time_peaks <- annotate("rect",
    xmin = c(5.955364), xmax = c(6.155364),
    ymin = -Inf, ymax = Inf, alpha = 0.2,
    fill = c("royalblue")
)

tim_loadings_pl <- all_loadings_pl$Time + time_peaks

grid.arrange(tim_scores_pl, tim_loadings_pl, ncol = 2)

## -----------------------------------------------------------------------------
# Hippurate:Time
hiptim_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time"
)
hiptim_loadings_pl <- all_loadings_pl$`Hippurate:Time` +
    time_peaks +
    hip_peaks

grid.arrange(hiptim_scores_pl, hiptim_loadings_pl, ncol = 2)

## -----------------------------------------------------------------------------
# Hippurate+Time+Hippurate:Time
hiptimInter_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Hippurate+Time+Hippurate:Time",
    color = "Hippurate", shape = "Time"
)

hiptimInter_loadings_pl <- all_loadings_pl$`Hippurate:Time` +
    time_peaks + hip_peaks

grid.arrange(hiptimInter_scores_pl, hiptimInter_loadings_pl, ncol = 2)

## ----Plot_Residuals-----------------------------------------------------------
resid_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Residuals",
    color = "Day", shape = "Day",
    drawShapes = "segment"
)


resid_loadings_pl <- all_loadings_pl$Residuals

grid.arrange(resid_scores_pl, resid_loadings_pl, ncol = 2)

## ----ASCA_ScoresMatrix, fig.height=7------------------------------------------
lmpScoreScatterPlotM(resASCA,
    PCdim = c(1, 1, 1, 1, 1, 1, 1, 2),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "ASCA scores scatterplot matrix"
)

## ----ASCA_loadings------------------------------------------------------------
lmpLoading2dPlot(
    resLmpPcaEffects = resASCA,
    effectNames = c("Hippurate"),
    axes = c(1, 2),
    addRownames = TRUE, pl_n = 10
)

# adding manually labels to points for the Hippurate peaks
labels <- substr(colnames(UCH$outcomes), 1, 4)
labels[-c(369, 132, 150, 133, 149, 144, 145, 368, 151)] <- ""

lmpLoading2dPlot(
    resLmpPcaEffects = resASCA,
    effectNames = c("Hippurate"),
    axes = c(1, 2), points_labs = labels
)

## -----------------------------------------------------------------------------
lmpEffectPlot(resASCA, effectName = "Hippurate", x = "Hippurate")

## -----------------------------------------------------------------------------
lmpEffectPlot(resASCA,
    effectName = "Hippurate:Time",
    x = "Hippurate", z = "Time"
)
lmpEffectPlot(resASCA,
    effectName = "Hippurate:Time",
    x = "Time", z = "Hippurate"
)

## ----ASCA_effects-------------------------------------------------------------
lmpEffectPlot(resASCA,
    effectName = "Hippurate+Time+Hippurate:Time",
    x = "Hippurate", z = "Time"
)
lmpEffectPlot(resASCA,
    effectName = "Hippurate+Time+Hippurate:Time",
    axes = c(2), x = "Time", z = "Hippurate"
)

## ----APCA_PCA-----------------------------------------------------------------
resAPCA <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "APCA"
)

## ----APCA_ScoresXY------------------------------------------------------------
# Hippurate main effect
lmpScorePlot(resAPCA,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate", drawShapes = "ellipse"
)

# Citrate main effect
lmpScorePlot(resAPCA,
    effectNames = "Citrate",
    color = "Citrate", shape = "Citrate", drawShapes = "ellipse"
)

# Time main effect
lmpScorePlot(resAPCA,
    effectNames = "Time",
    color = "Time", shape = "Time", drawShapes = "ellipse"
)
lmpScorePlot(resAPCA,
    effectNames = "Time",
    color = "Time", shape = "Time", drawShapes = "polygon"
)
lmpScorePlot(resAPCA,
    effectNames = "Time",
    color = "Time", shape = "Time", drawShapes = "segment"
)

# Interaction term
lmpScorePlot(resAPCA,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time", drawShapes = "segment"
)
lmpScorePlot(resAPCA,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time", drawShapes = "polygon"
)

## ----APCA_ScoresMatrix--------------------------------------------------------
lmpScoreScatterPlotM(resAPCA,
    effectNames = c(
        "Hippurate", "Citrate", "Time",
        "Hippurate:Time"
    ),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "APCA scores scatterplot matrix"
)

## ----APCA_loadings------------------------------------------------------------
lmpLoading1dPlot(resAPCA, effectNames = c(
    "Hippurate", "Citrate",
    "Time", "Hippurate:Time"
), axes = 1)

## ----ASCAE_PCA----------------------------------------------------------------
resASCAE <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "ASCA-E"
)

## ----ASCAE_ScoresXY-----------------------------------------------------------
lmpScorePlot(resASCAE,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate"
)
lmpScorePlot(resASCAE,
    effectNames = "Citrate",
    color = "Citrate", shape = "Citrate"
)
lmpScorePlot(resASCAE,
    effectNames = "Time",
    color = "Time", shape = "Time"
)
lmpScorePlot(resASCAE,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time"
)

## ----ASCAE_ScoresMatrix-------------------------------------------------------
lmpScoreScatterPlotM(resASCAE,
    effectNames = c(
        "Hippurate", "Citrate", "Time",
        "Hippurate:Time"
    ),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "ASCA-E scores scatterplot matrix"
)

## -----------------------------------------------------------------------------
sessionInfo()

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    out.width = "70%",
    fig.align = "center",
    eval = TRUE
)
rm(list = ls())
library("limpca")

if (!requireNamespace("SummarizedExperiment", quietly = TRUE)) {
    stop("Install 'SummarizedExperiment' to knit this vignette")
}
library(SummarizedExperiment)

## ----Install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("limpca")

## ----Load, results=FALSE, message=FALSE---------------------------------------
library("limpca")

## -----------------------------------------------------------------------------
data("UCH")
str(UCH)

## ----message = TRUE-----------------------------------------------------------
UCH2 <- data2LmpDataList(
   outcomes = UCH$outcomes,
   design = UCH$design, 
   formula = UCH$formula
 )

## ----message = TRUE-----------------------------------------------------------
se <- SummarizedExperiment(
   assays = list(
     counts = t(UCH$outcomes)), colData = UCH$design,
   metadata = list(formula = UCH$formula)
 )

UCH3 <- data2LmpDataList(se, assay_name = "counts")

## ----dataVisu-----------------------------------------------------------------
# design
plotDesign(
    design = UCH$design, x = "Hippurate",
    y = "Citrate", rows = "Time",
    title = "Design of the UCH dataset"
)

# row 3 of outcomes
plotLine(
    Y = UCH$outcomes,
    title = "H-NMR spectrum",
    rows = c(3),
    xlab = "ppm",
    ylab = "Intensity"
)

## ----PCA----------------------------------------------------------------------
ResPCA <- pcaBySvd(UCH$outcomes)
pcaScreePlot(ResPCA, nPC = 6)
pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA scores plot: PC1 and PC2",
    design = UCH$design,
    color = "Hippurate", shape = "Citrate",
    points_labs_rn = FALSE
)

## ----modelEst-----------------------------------------------------------------
# Model matrix generation
resMM <- lmpModelMatrix(UCH)

# Model estimation and effect matrices decomposition
resEM <- lmpEffectMatrices(resMM)

## ----effectImpSign------------------------------------------------------------
# Effects importance
resEM$varPercentagesPlot

# Bootstrap tests
resBT <- lmpBootstrapTests(resLmpEffectMatrices = resEM, nboot = 100)
resBT$resultsTable

## ----ASCA---------------------------------------------------------------------
# ASCA decomposition
resASCA <- lmpPcaEffects(resLmpEffectMatrices = resEM, method = "ASCA")

# Scores Plot for the hippurate
lmpScorePlot(resASCA,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate"
)

# Loadings Plot for the hippurate
lmpLoading1dPlot(resASCA,
    effectNames = c("Hippurate"),
    axes = 1, xlab = "ppm"
)

# Scores ScatterPlot matrix
lmpScoreScatterPlotM(resASCA,
    PCdim = c(1, 1, 1, 1, 1, 1, 1, 2),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "ASCA scores scatterplot matrix"
)

## -----------------------------------------------------------------------------
sessionInfo()

#' @export lmpPcaEffects
#' @title PCA on the effect matrices
#'
#' @description
#' Performs a PCA on each of the effect matrices from the outputs of \code{\link{lmpEffectMatrices}}. It has an option to choose the method applied: ASCA, APCA or ASCA-E. Combined effects (i.e. linear combinations of original effect matrices) can also be created and decomposed by PCA.
#'
#' @param resLmpEffectMatrices A resLmpEffectMatrices list resulting of \code{\link{lmpEffectMatrices}}.
#' @param method The method used to compute the PCA. One of \code{c("ASCA","APCA","ASCA-E")}.
#' @param combineEffects If not \code{NULL}, a list of vectors containing the names of the effects to be combined.
#' @param verbose If \code{TRUE}, will display a message with the duration of execution.
#'
#' @return A list with first,the PCA results from \code{\link{pcaBySvd}} for each effect matrix. Those results contain :
#'  \describe{
#'   \item{\code{scores}}{Scores from the PCA for each principal component.}
#'   \item{\code{loadings}}{Loadings from the PCA for each principal component.}
#'   \item{\code{eigval}}{Eigenvalues of each principal component.}
#'   \item{\code{singvar}}{Singular values of each principal component.}
#'   \item{\code{var}}{Explained variances of each principal component.}
#'   \item{\code{cumvar}}{Cumulated explained variances of each principal component.}
#'   \item{\code{original.dataset}}{Original dataset.}
#'  }
#'
#'  There are also others outputs :
#'  \describe{
#'  \item{\code{lmpDataList}}{The initial object: a list of outcomes, design and formula.}
#'  \item{\code{effectsNamesUnique}}{A character vector with the \emph{F+1} names of the model terms, each repeated once.}
#'  \item{\code{method}}{The dimension reduction method used: \code{c("ASCA","APCA","ASCA-E")}.}
#'  \item{\code{type3SS}}{A vector with the type III SS for each model term.}
#'  \item{\code{variationPercentages}}{A vector with the percentage of variance explained by each model term.}
#'  }
#'
#' @details
#'  The function allows 3 different methods :
#'
#'   \describe{
#'   \item{ASCA}{PCA is applied directly on each pure effect matrix \eqn{\hat{\mathbf{M}}_f, f=1...F}.}
#'   \item{ASCA-E}{PCA is applied on each pure effect matrix but then the augmented effect matrix is projected in the space of the ASCA components.}
#'   \item{APCA}{PCA is applied on each augmented effect matrix : \eqn{\hat{\mathbf{M}}_f+\hat{\mathbf{E}}}.}
#'  }
#'
#' @examples
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resLmpPcaEffects <- lmpPcaEffects(resLmpEffectMatrices, method = "ASCA-E")
#'
#' @references Thiel M.,Feraud B. and Govaerts B. (2017) ASCA+ and APCA+: Extensions of ASCA and APCA
#' in the analysis of unbalanced multifactorial designs. \emph{Journal of Chemometrics}. 31:e2895.
#' \url{https://doi.org/10.1002/cem.2895}
#'


lmpPcaEffects <- function(resLmpEffectMatrices, method = c("ASCA", "APCA", "ASCA-E"),
                          combineEffects = NULL, verbose = FALSE) {
  # Checking the resLmpEffectMatrices list

  checkname <- c(
    "lmpDataList", "modelMatrix", "modelMatrixByEffect",
    "effectsNamesUnique",
    "effectsNamesAll", "effectMatrices",
    "predictedvalues", "residuals", "parameters",
    "type3SS", "variationPercentages", "varPercentagesPlot"
  )


  if (!is.list(resLmpEffectMatrices)) {
    stop("Argument resLmpEffectMatrices is not a list")
  }
  if (length(resLmpEffectMatrices) != 12) {
    stop("List does not contain 12 arguments")
  }
  if (!all(names(resLmpEffectMatrices) == checkname)) {
    stop("Argument is not a resLmpEffectMatrices object")
  }
  if (length(resLmpEffectMatrices$effectMatrices) !=
    length(resLmpEffectMatrices$effectsNamesUnique)) {
    stop("Number of effect matrices different from the number of effects")
  }
  # if(!method %in% c("ASCA","APCA","ASCA-E")){stop("Method must be one of: ASCA, ASCA-E, APCA")}
  method <- match.arg(method)
  checkArg(combineEffects, c("list"), can.be.null = TRUE)

  # Attributing name

  lmpDataList <- resLmpEffectMatrices$lmpDataList
  effectsNamesUnique <- effectsNamesUniqueCombined <-
    resLmpEffectMatrices$effectsNamesUnique

  # Combining effects

  if (!is.null(combineEffects)) {
    combineMatrices <- function(combineVectors) {
      matList <- list()
      combinedEffects <- list()
      matList <- lapply(combineVectors, function(x) {
        resLmpEffectMatrices$effectMatrices[[x]]
      })
      combinedEffects <- Reduce("+", matList)
    }

    combinedMatrices <- lapply(combineEffects, combineMatrices)
    names(combinedMatrices) <- lapply(combineEffects,
      FUN = function(x) {
        paste0(x,
          collapse = "+"
        )
      }
    )

    resLmpEffectMatrices$effectMatrices <- c(
      resLmpEffectMatrices$effectMatrices,
      combinedMatrices
    )

    effectsNamesUniqueCombined <- c(
      effectsNamesUnique,
      names(combinedMatrices)
    )
  }


  # Construction of the list of pure effect matrix

  EffectMatGLM <- resLmpEffectMatrices$effectMatrices[-1] # minus intercept
  res <- vector(mode = "list")
  res[[1]] <- resLmpEffectMatrices$residuals
  EffectMatGLM <- c(EffectMatGLM, Residuals = res) # plus residuals

  if (!is.null(combineEffects)) {
    p <- length(effectsNamesUnique) + length(combineEffects)
  } else {
    p <- length(effectsNamesUnique) # Number of parameters
  }


  # Defining the matrix for the PCA depending on the method

  if (method == "ASCA") {
    if (verbose) {
      message("ASCA method used : PCA on the pure effect matrices")
    }
  } else if (method == "APCA") {
    if (verbose) {
      message("APCA method used : PCA on the augmented effect matrices")
    }

    # Compute the augmented effect matrices

    EffectMatGLM <- resLmpEffectMatrices$effectMatrices[-1] # effectMatrices minus intercept
    EffectMatGLM <- lapply(EffectMatGLM, function(x) {
      x +
        resLmpEffectMatrices$residuals
    })
    EffectMatGLM <- c(EffectMatGLM, Residuals = res) # plus residuals
  } else if (method == "ASCA-E") {
    if (verbose) {
      message("ASCA-E method used : PCA on the pure effect matrices
            but scores are updated")
    }
  } else {
    stop("The method argument is not one of those : ASCA, APCA, ASCA-E")
  }


  # Run the PCA on the different effects

  resLmpPcaEffects <- vector(mode = "list")

  for (i in seq_len(p)) {
    resLmpPcaEffects[[i]] <- pcaBySvd(EffectMatGLM[[i]])
  }

  # Updating the score for ASCA-E method

  if (method == "ASCA-E") {
    for (i in seq_len(p)) {
      resLmpPcaEffects[[i]]$scores[, seq_len(5)] <- (EffectMatGLM[[i]] +
        EffectMatGLM[[p]]) %*% resLmpPcaEffects[[i]]$loadings[, seq_len(5)]
    }
  }

  names(resLmpPcaEffects) <- names(EffectMatGLM)

  resLmpPcaEffects2 <- list(
    lmpDataList = lmpDataList,
    effectsNamesUnique = effectsNamesUnique,
    effectsNamesUniqueCombined = effectsNamesUniqueCombined
  )

  resLmpPcaEffects <- c(resLmpPcaEffects, resLmpPcaEffects2,
    method = method,
    type3SS = list(resLmpEffectMatrices$type3SS),
    variationPercentages = list(resLmpEffectMatrices$variationPercentages),
    combineEffects = list(combineEffects)
  )

  return(resLmpPcaEffects)
}

#' @export lmpModelMatrix
#' @title Creates the model matrix `X`
#'
#' @description
#' Creates the model matrix `X` from the design matrix and the model formula.
#'
#' @param lmpDataList A list containing the outcomes, the experimental design and the formula.
#'
#' @return A list with the 5 following named elements :
#' \describe{
#'    \item{\code{lmpDataList}}{The initial object: a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}
#'    \item{\code{modelMatrix}}{A \emph{nxK} model matrix specifically encoded for the ASCA-GLM method.}
#'    \item{\code{modelMatrixByEffect}}{A list of \emph{p} model matrices for each model effect.}
#'    \item{\code{effectsNamesUnique}}{A character vector with the \emph{p} names of the model effects, each repeated once.}
#'    \item{\code{effectsNamesAll}}{A character vector with the \emph{K} names of the model effects ordered and repeated as the column names of the model matrix.}
#' }
#'
#' @details
#' In typical ASCA-GLM (ASCA+) analysis, the effects of the GLM model must first be used to transform the design matrix to a model matrix where the design factors encoded usign \emph{sum coding} commonly used in industrial experimental design.
#' Suppose the design matrix is \emph{nxk} with n observations and \emph{k} factors. After the transformation, the model matrix
#' will be of size \emph{nxp}. For a fator with \emph{a} levels, the \emph{sum coding} creates \emph{a-1} columns in the model matrix with 0 and 1 for the \emph{a-1} first levels and -1 for the last one.
#' \emph{p} is the total number parameter for each response (outcome) in the ASCA model.
#' More information is available in the article (\emph{Thiel et al}, 2017)
#' Note that at the moment, only factors can be used as explanatory variables.
#'
#' @seealso \code{\link{model.matrix}}
#'
#' @examples
#'
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#'
#' head(resLmpModelMatrix$modelMatrix)
#'
#' @references Thiel M.,Feraud B. and Govaerts B. (2017) \emph{ASCA+ and APCA+: Extensions of ASCA and APCA
#' in the analysis of unbalanced multifactorial designs}, Journal of Chemometrics
#'
#'
#' @import grDevices
#' @importFrom stats as.formula model.matrix


lmpModelMatrix <- function(lmpDataList) {
  lmpDataList <- data2LmpDataList(
    outcomes = lmpDataList$outcomes,
    design = lmpDataList$design,
    formula = lmpDataList$formula, verbose = FALSE
  )
  lmpDataListCheck(lmpDataList = lmpDataList)

  formula <- stats::as.formula(lmpDataList$formula)
  formulaDesignMatrix <- formula

  design <- lmpDataList$design

  # Checking if all variables are factors

  varNames <- all.vars(formula)

  if (!is.null(varNames)) {
    testClass <- mapply(is.factor, design[, varNames])
    if (!all(testClass)) {
      stop(
        "at least one design variable used in the formula is not a factor ",
        "(", paste(varNames[!testClass], collapse = ", "), ")"
      )
    }
  }

  # Checking which variables are factors
  factorsDesign <- names(Filter(is.factor, design))
  varNamesFactors <- intersect(factorsDesign, varNames)



  # Creating model matrix If factors are present, a list is created to specify
  # which variables are considered as factors in model.matrix
  if (length(varNamesFactors) != 0) {
    contrasts.arg.Values <- list()
    length(contrasts.arg.Values) <- length(varNamesFactors)
    names(contrasts.arg.Values) <- varNamesFactors
    for (iList in seq_along(contrasts.arg.Values)) {
      contrasts.arg.Values[[iList]] <- "contr.sum"
    }
    modelMatrix <- (stats::model.matrix(formulaDesignMatrix,
      contrasts.arg = contrasts.arg.Values,
      data = design
    ))
  }

  # If factors are not present (Currently not the case)
  if (length(varNamesFactors) == 0) {
    modelMatrix <- (stats::model.matrix(formulaDesignMatrix,
      data = design
    ))
  }

  # Creating a list containing model matrices by effect

  # Finding all unique variables
  dummyVarNames <- colnames(modelMatrix)
  presencePolynomialEffects <- stringr::str_detect(
    dummyVarNames,
    "\\^[0-9]"
  ) # Detect exponent
  effectsNamesAll <- character(length = length(dummyVarNames))
  effectsNamesAll[presencePolynomialEffects] <- dummyVarNames[presencePolynomialEffects]
  effectsNamesAll[!presencePolynomialEffects] <- gsub(
    "[0-9]", "",
    dummyVarNames[!presencePolynomialEffects]
  )
  effectsNamesAll[effectsNamesAll == "(Intercept)"] <- "Intercept"
  effectsNamesUnique <- unique(effectsNamesAll)
  nEffect <- length(effectsNamesUnique)

  # Creating empty model matrices by effect
  modelMatrixByEffect <- list()
  length(modelMatrixByEffect) <- nEffect
  names(modelMatrixByEffect) <- effectsNamesUnique

  # Filling model matrices by effect
  for (iEffect in seq_len(nEffect)) {
    selection <- which(effectsNamesAll == effectsNamesUnique[iEffect])
    selectionComplement <- which(effectsNamesAll != effectsNamesUnique[iEffect])
    # Model matrices by effect
    modelMatrixByEffect[[iEffect]] <- as.matrix(modelMatrix[, selection])
  }

  resLmpModelMatrix <- list(
    lmpDataList = lmpDataList,
    modelMatrix = modelMatrix,
    modelMatrixByEffect = modelMatrixByEffect,
    effectsNamesUnique = effectsNamesUnique,
    effectsNamesAll = effectsNamesAll
  )

  return(resLmpModelMatrix)
}

set.seed(1)
design <- data.frame(
    factA = factor(c(rep(1, 5), rep(2, 5))),
    factB = factor(rep(c(1, 2), 5))
)
outcomes <- matrix(rnorm(30), ncol = 3)
rownames(outcomes) = c(1:nrow(outcomes))
formula <- "~ factA+factB"
lmpDL <- list(design = design, outcomes = outcomes, formula = formula)

resLMM <- lmpModelMatrix(lmpDL)

test_that("lmpModelMatrix runs correctly", {
    expect_type(resLMM, "list")
    expect_length(resLMM, 5)
    expect_equal(resLMM$modelMatrix[, "factA1"],
        c(1, 1, 1, 1, 1, -1, -1, -1, -1, -1),
        ignore_attr = "names"
    )
    expect_equal(
        as.vector(resLMM$modelMatrixByEffect$factA),
        c(1, 1, 1, 1, 1, -1, -1, -1, -1, -1)
    )
    expect_equal(resLMM$effectsNamesUnique, c("Intercept", "factA", "factB"))
    expect_equal(resLMM$effectsNamesAll, c("Intercept", "factA", "factB"))
})

#' @importFrom plyr is.formula

# Check of the entry arguments
checkArg <- function(arg, checks, can.be.null = FALSE) {
  check.list <- list(
    bool = c(is.logical, "a boolean"),
    int = c(
      function(x) {
        identical(
          as.numeric(x),
          round(as.numeric(x))
        )
      },
      "an integer"
    ),
    num = c(is.numeric, "a numeric"),
    str = c(is.character, "a character string"),
    pos = c(function(x) {
      min(x) > 0
    }, "positive"),
    pos0 = c(function(x) {
      min(x) >= 0
    }, "positive or zero"),
    formula = c(plyr::is.formula, "a formula"),
    data.frame = c(is.data.frame, "a dataframe"),
    matrix = c(is.matrix, "a matrix"),
    list = c(is.list, "a list"),
    length1 = c(function(x) {
      length(x) == 1
    }, "of length 1")
  )

  if (is.null(arg)) {
    if (!can.be.null) {
      stop(deparse(substitute(arg)), " is null.")
    }
  } else {
    for (cc in checks) {
      if (!check.list[[cc]][[1]](arg)) {
        stop(
          deparse(substitute(arg)), " is not ",
          check.list[[cc]][[2]], "."
        )
      }
    }
  }
}

#' @export lmpBootstrapTests
#' @title Tests the significance of model effects by bootstrap.
#'
#' @description
#' Tests the significance of the effects from the model using bootstrap. This function is based on the outputs of \code{\link{lmpEffectMatrices}}. Tests on combined effects are also provided.
#'
#' @param resLmpEffectMatrices A list of 12 from \code{\link{lmpEffectMatrices}}.
#' @param nboot An integer with the number of bootstrap sample to be drawn.
#' @param nCores The number of cores to use for parallel execution.
#' @param verbose If \code{TRUE}, will display a message with the duration of execution.
#'
#' @return A list with the following elements:
#'  \describe{
#'    \item{\code{f.obs}}{A vector of size F (number of effects in the model) with the F statistics for each model term calculated on the initial data.}
#'    \item{\code{f.boot}}{ b × F matrix with the F statistics calculated on the bootstrap samples.}
#'    \item{\code{p.values}}{A vector of size F with the p-value for each model effect.}
#'    \item{\code{resultsTable}}{A 2 × F matrix with the p-value and the percentage of variance for each model effect.}
#'  }
#'
#' @examples
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix = resLmpModelMatrix)
#'
#' res <- lmpBootstrapTests(
#'   resLmpEffectMatrices = resLmpEffectMatrices,
#'   nboot = 10, nCores = 2, verbose = TRUE
#' )
#'
#' @references
#' Thiel M.,Feraud B. and Govaerts B. (2017) \emph{ASCA+ and APCA+: Extensions of ASCA and APCA
#' in the analysis of unbalanced multifactorial designs}, Journal of Chemometrics
#'
#' Thiel, M., Benaiche, N., Martin, M., Franceschini, S., Van Oirbeek, R., & Govaerts, B. (2023)
#' \emph{limpca: an R package for the linear modeling of high dimensional designed data based on
#' ASCA/APCA family of methods}, Journal of Chemometrics
#' @import doParallel
#' @import parallel
#' @importFrom plyr laply llply

lmpBootstrapTests <- function(resLmpEffectMatrices, nboot = 100, nCores = 2, verbose = FALSE) {
  # Checking the resLmpEffectMatrices list

  checkname <- c(
    "lmpDataList", "modelMatrix", "modelMatrixByEffect",
    "effectsNamesUnique",
    "effectsNamesAll", "effectMatrices",
    "predictedvalues", "residuals", "parameters",
    "type3SS", "variationPercentages", "varPercentagesPlot"
  )


  if (!is.list(resLmpEffectMatrices)) {
    stop("Argument resLmpEffectMatrices is not a list")
  }
  if (length(resLmpEffectMatrices) != 12) {
    stop("List does not contain 12 arguments")
  }
  if (!all(names(resLmpEffectMatrices) == checkname)) {
    stop("Argument is not a resLmpEffectMatrices object")
  }
  if (length(resLmpEffectMatrices$effectMatrices) !=
    length(resLmpEffectMatrices$effectsNamesUnique)) {
    stop("Number of effect matrices differs from the number of effects")
  }

  # check if SS = TRUE

  if (all(is.na(resLmpEffectMatrices$type3SS)) &
    all(is.na(resLmpEffectMatrices$variationPercentages))) {
    stop("lmpBootstrapTests can't be performed if
         resLmpEffectMatrices doesn't include the effect percentage variations (SS=FALSE)")
  }

  # Attributing names
  start_time <- Sys.time()
  lmpDataList <- resLmpEffectMatrices$lmpDataList
  formula_complete <- resLmpEffectMatrices$lmpDataList$formula
  outcomes <- resLmpEffectMatrices$lmpDataList$outcomes
  modelMatrix <- resLmpEffectMatrices$modelMatrix
  modelMatrixByEffect <- resLmpEffectMatrices$modelMatrixByEffect
  effectsNamesAll <- resLmpEffectMatrices$effectsNamesAll
  effectsNamesUnique <- resLmpEffectMatrices$effectsNamesUnique
  nEffect <- length(effectsNamesUnique)
  SS_complete <- resLmpEffectMatrices$type3SS
  SSE_complete <- resLmpEffectMatrices$type3SS[which(names(SS_complete) == "Residuals")]
  nObs <- nrow(outcomes)
  nParam <- length(effectsNamesAll)

  # Recreate resLmpModelMatrix

  resLmpModelMatrix <- resLmpEffectMatrices[seq_len(6)]

  # Parallel computing

  doParallel::registerDoParallel(cores = nCores)

  #### Estimating the partial model for each effect ####

  listResultPartial <- list()
  Fobs <- list()
  Pobs <- list()

  partial_mod_fun <- function(iEffect) {
    selection_tmp <- which(effectsNamesAll == effectsNamesUnique[iEffect])
    selectionall <- which(effectsNamesAll == effectsNamesUnique[iEffect])
    selectionComplement_tmp <- which(effectsNamesUnique != effectsNamesUnique[iEffect])
    selectionComplementall <- which(effectsNamesAll != effectsNamesUnique[iEffect])

    # Model matrices Partial

    modelMatrixPartial <- modelMatrixByEffect[[selectionComplement_tmp[1]]]
    listModelMatrixByEffectPartial_temp <- list()
    listModelMatrixByEffectPartial_temp[[1]] <- modelMatrixByEffect[[selectionComplement_tmp[1]]]

    for (i in seq(2, length(selectionComplement_tmp))) {
      # Create Model Matrix for the partial model
      modelMatrixPartial <- cbind(
        modelMatrixPartial,
        modelMatrixByEffect[[selectionComplement_tmp[i]]]
      )

      # Create listModelMatrixByEffectPartial
      listModelMatrixByEffectPartial_temp[[i]] <- modelMatrixByEffect[[selectionComplement_tmp[i]]]
    }

    colnames(modelMatrixPartial) <- colnames(modelMatrix[, selectionComplementall])

    # Be careful modelMatrixByEffect with 1 parameters have no colnames

    # Create effectsNamesAll

    effectsNamesUniquePartial <- effectsNamesUnique[selectionComplement_tmp]
    effectsNamesAllPartial <- effectsNamesAll[selectionComplementall]
    names(listModelMatrixByEffectPartial_temp) <- effectsNamesUniquePartial

    # Create the partial formula

    temp <- gsub(":", "*", effectsNamesUniquePartial)
    temp <- paste(temp, collapse = "+")
    temp <- paste0("outcomes~", temp)
    formula_temp <- as.formula(temp)
    lmpDataList$formula <- as.formula(temp)

    # Create pseudo ResLMModelMatrix

    Pseudo_resLmpModelMatrix <- list(
      lmpDataList = lmpDataList,
      modelMatrix = modelMatrixPartial,
      modelMatrixByEffect = listModelMatrixByEffectPartial_temp,
      effectsNamesUnique = effectsNamesUniquePartial,
      effectsNamesAll = effectsNamesAllPartial
    )

    # Compute the partial models

    listResultPartial <- lmpEffectMatrices(Pseudo_resLmpModelMatrix,
      SS = TRUE
    )

    # Compute Fobs

    Fobs <- (SS_complete[iEffect] / length(selection_tmp)) /
      (SSE_complete / (nObs - nParam))

    return(list(listResultPartial = listResultPartial, Fobs = Fobs))
  }

  res_partial_mod_fun <- plyr::llply(seq_len(nEffect), partial_mod_fun,
    .parallel = TRUE
  )


  listResultPartial <- lapply(
    res_partial_mod_fun,
    function(x) x[["listResultPartial"]]
  )
  Fobs <- lapply(
    res_partial_mod_fun[seq(2, nEffect)],
    function(x) x[["Fobs"]]
  )

  # Formating the output
  names(listResultPartial) <- effectsNamesUnique
  Fobs <- unlist(Fobs)
  names(Fobs) <- effectsNamesUnique[seq(2, nEffect)]

  #### Bootstrap #####

  ###  useful functions for ComputeFboot()

  # Compute the Sum of Squares Type 3
  # function based on LMSSv2()
  LMSSv2_bis <- function(Res, listcontrast) {
    computeSS_bis <- function(Xmat, L, coef) {
      if (is.vector(L)) {
        L <- t(L)
      }
      LB <- L %*% coef
      BL <- t(LB)
      mat <- BL %*% solve(L %*% solve(t(Xmat) %*% Xmat) %*%
        t(L)) %*% LB
      SS <- sum(diag(mat))
      return(SS)
    }

    L <- listcontrast

    Y_withoutIntercept <- Res$outcomes - Res$Intercept
    denom <- norm(x = data.matrix(Y_withoutIntercept), "F")^2

    result <- vapply(L, function(x) {
      computeSS_bis(
        Xmat = Res$modelMatrix, L = x,
        Res$parameters
      )
    }, FUN.VALUE = 0, USE.NAMES = TRUE)

    result <- c(result, ((norm(x = Res$residuals, "F")^2) / denom) * 100)
    # result = c(result,norm(x=Res$residuals,"F")^2)

    names(result) <- c(Res$effectsNamesUnique, "Residuals")

    LMSS <- list(SS = result)
    return(LMSS)
  }

  # Compute Fboot from Sum of Squares Type 3
  Fboot_fun <- function(i, result_boot, effect_names, npar, nObs) {
    nume <- result_boot[[i]]$SS[which(names(result_boot[[i]]$SS) == effect_names[i])] /
      npar[[effect_names[i]]]
    denom <- result_boot[[i]]$SS[which(names(result_boot[[i]]$SS) == "Residuals")] /
      (nObs - nParam)
    Fboot <- nume / denom
    return(Fboot)
  }

  ### ComputeFboot() function to compute the F statistic for every effect

  ComputeFboot <- function(E_sample, listResultPartial,
                           resLmpModelMatrix) {
    effectsNamesAll <- resLmpModelMatrix$effectsNamesAll
    effectsNamesUnique <- resLmpModelMatrix$effectsNamesUnique
    nEffect <- length(effectsNamesUnique)

    # prepare  Res_list input argument for LMSSv2_bis --------------

    # Y_boot_list (simulated outcomes with partial models) for all the effects

    E_boot_list <- plyr::llply(listResultPartial[seq(2, nEffect)],
      function(x) x$residuals[E_sample, ],
      .parallel = FALSE
    )

    Y_boot_list <- plyr::llply(seq_along(E_boot_list),
      function(i) {
        listResultPartial[seq(2, nEffect)][[i]]$predictedvalues +
          E_boot_list[[i]]
      },
      .parallel = FALSE
    )

    names(Y_boot_list) <- names(E_boot_list)

    # # Find the number of parameters for each effect
    # npar <- plyr::llply(resLmpModelMatrix$modelMatrixByEffect, ncol,
    #                     .parallel = FALSE)

    # Estimate (X'X)-1X'

    X <- resLmpModelMatrix$modelMatrix
    XtX_1Xt <- solve(t(X) %*% X) %*% t(X)

    # Compute the parameters

    parameters_list <- plyr::llply(Y_boot_list, function(y) XtX_1Xt %*% y,
      .parallel = FALSE
    )


    # effectMatrices_list and intercept_list

    selection <- lapply(
      effectsNamesUnique,
      function(x) which(effectsNamesAll == x)
    )
    names(selection) <- effectsNamesUnique

    effectMatrices_list <- plyr::llply(parameters_list, function(y) {
      lapply(selection, function(x) {
        as.matrix(modelMatrix[, x]) %*% y[x, ]
      })
    },
    .parallel = FALSE
    )

    Intercept_list <- plyr::llply(effectMatrices_list,
      function(x) x[["Intercept"]],
      .parallel = FALSE
    )


    # residuals

    residuals_list <- plyr::llply(seq_along(Y_boot_list),
      function(i) {
        Y_boot_list[[i]] -
          Reduce("+", effectMatrices_list[[i]])
      },
      .parallel = FALSE
    )

    names(residuals_list) <- names(Y_boot_list)


    # prepare  Res_list input arg for LMSSv2_bis

    Res <- list(
      outcomes = Y_boot_list,
      residuals = residuals_list,
      Intercept = Intercept_list,
      modelMatrix = resLmpModelMatrix$modelMatrix,
      parameters = parameters_list
    )

    Res_list <- plyr::llply(seq_along(Y_boot_list),
      function(x) {
        list(
          outcomes = Res$outcomes[[x]],
          residuals = Res$residuals[[x]],
          Intercept = Res$Intercept[[x]],
          modelMatrix = Res$modelMatrix,
          parameters = Res$parameters[[x]],
          effectsNamesUnique =
            effectsNamesUnique
        )
      },
      .parallel = FALSE
    )

    names(Res_list) <- names(Y_boot_list)

    # List of contrasts

    listcontrast <- contrastSS(resLmpModelMatrix)


    ### Compute the Sum of Squares Type 3 -----------

    result_boot <- lapply(
      Res_list,
      function(x) {
        LMSSv2_bis(
          Res = x,
          listcontrast = listcontrast
        )
      }
    )

    effect_names <- names(result_boot)

    ### Compute Fboot from Sum of Squares Type 3 --------------

    # Find the number of parameters for each effect
    npar <- plyr::llply(resLmpModelMatrix$modelMatrixByEffect, ncol,
      .parallel = FALSE
    )

    Fboot <- plyr::laply(seq_along(effect_names),
      function(x) {
        Fboot_fun(
          x, result_boot,
          effect_names, npar, nObs
        )
      },
      .parallel = FALSE
    )

    return(Fboot)
  }


  # sample the observations for bootstrap

  E_sample <- lapply(seq_len(nboot), function(x) {
    sample(seq_len(nObs),
      nObs,
      replace = TRUE
    )
  })

  # compute Fboot for simulated data
  Fboot <- plyr::laply(E_sample,
    function(x) {
      ComputeFboot(
        E_sample = x,
        listResultPartial = listResultPartial,
        resLmpModelMatrix = resLmpModelMatrix
      )
    },
    .parallel = TRUE
  )


  ###### Compute the boostrapped pvalue ######
  result <- vector()
  matrix_temp <- rbind(Fobs, Fboot)

  ComputePval <- function(Effect, Fobs) {
    result <- 1 - sum(Effect[1] > Effect[seq(2, (nboot + 1))]) / nboot
    return(result)
  }

  # Outputs generation
  result <- apply(X = matrix_temp, FUN = ComputePval, MARGIN = 2)
  result <- signif(result, digits = log10(nboot))
  colnames(Fboot) <- names(Fobs)

  result <- replace(result, result == 0, paste0(
    "< ",
    format(1 / nboot,
      digits = 1,
      scientific = FALSE
    )
  ))

  resultsTable_temp <- rbind(
    result,
    round(resLmpEffectMatrices$variationPercentages[seq_along(Fobs)], 2)
  )
  resultsTable <- cbind(resultsTable_temp, c(
    "-",
    round(resLmpEffectMatrices$variationPercentages[["Residuals"]], 2)
  ))
  rownames(resultsTable) <- c("Bootstrap p-values", "% of variance (T III)")
  colnames(resultsTable) <- c(
    resLmpEffectMatrices$effectsNamesUnique[-1],
    "Residuals"
  )

  resultsTable <- t(resultsTable)
  resultsTable <- as.data.frame(resultsTable[, c(2, 1)])
  resultsTable[, 1] <- as.numeric(resultsTable[, 1])

  resLmpBootstrapTests <- list(
    f.obs = Fobs, f.boot = Fboot,
    p.values = result,
    resultsTable = resultsTable
  )

  doParallel::stopImplicitCluster
  if (verbose) {
    message(Sys.time() - start_time)
  }

  return(resLmpBootstrapTests)
}

set.seed(1)
design <- data.frame(
    factA = factor(c(rep(1, 5), rep(2, 5))),
    factB = factor(rep(c(1, 2), 5))
)
outcomes <- matrix(rnorm(30), ncol = 3)
formula <- "~ factA+factB"
lmpDL <- list(design = design, outcomes = outcomes, formula = formula)
mM <- model.matrix(as.formula(lmpDL$formula), lmpDL$design)
mMBE <- list(
    Intercept = mM[, "(Intercept)", drop = FALSE],
    factA = mM[, "factA2", drop = FALSE],
    factB = mM[, "factB2", drop = FALSE]
)
eNU <- eNA <- c("Intercept", "factA", "factB")
eM <- list(
    Intercept = matrix(c(
        rep(0.7, 10),
        rep(0.9, 10), rep(0.5, 10)
    ), ncol = 3),
    factA = matrix(rnorm(30), ncol = 3),
    factB = matrix(rnorm(30), ncol = 3)
)
err <- matrix(rnorm(30, sd = 0.2), ncol = 3)
pv <- outcomes + err
residuals <- outcomes - err
parameters <- matrix(rnorm(9),
    nrow = 3, ncol = 3,
    dimnames = list(c("(Intercept)", "factA2", "factB2"), NULL)
)
type3SS <- c(Intercept = 100, factA = 68, factB = 50, Residuals = 13)
vP <- c(factA = 68, factB = 10, Residuals = 22)
vPP <- NULL

resLmpEffectMatrices <- list(
    lmpDataList = lmpDL,
    modelMatrix = mM,
    modelMatrixByEffect = mMBE,
    effectsNamesUnique = eNU,
    effectsNamesAll = eNA,
    effectMatrices = eM,
    predictedvalues = pv,
    residuals = residuals,
    parameters = parameters,
    type3SS = type3SS,
    variationPercentages = vP,
    varPercentagesPlot = vPP
)
set.seed(2)
res_BT <- lmpBootstrapTests(resLmpEffectMatrices = resLmpEffectMatrices, nboot = 2)

test_that("lmpBootstrapTests runs correctly", {
    expect_type(res_BT, "list")
    expect_length(res_BT, 4)
    expect_equal(res_BT$f.obs, c(factA = 36.615385, factB = 26.923077))
})

#' @export plotScatterM
#' @title Scatter plot matrix
#'
#' @description
#' Produces a scatter plot matrix between the selected columns of the outcomes matrix \eqn{Y} choosing specific colors and symbols for up to four factors from the design on the upper and lower diagonals.
#'
#' @param Y \eqn{n \times m} matrix with \eqn{n} observations and \eqn{m} variables. Can be `NULL` if `lmpDataList` is defined.
#' @param design A \eqn{n \times k} "freely encoded" experimental design data.frame. Can be `NULL` if `lmpDataList` is defined.
#' @param lmpDataList If not `NULL`, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.
#' @param cols A vector with either the column names of the \eqn{Y} matrix to plot (character) or the column index positions.
#' @param labelVector Labels to display on the diagonal. If `NULL`, the column names are deduced from `cols`.
#' @param title Title of the graph.
#' @param varname.colorup A character string with the name of the variable used to color the upper triangle.
#' @param varname.colordown A character string with the name of the variable used to color the lower triangle.
#' @param varname.pchup A character string with the name of the variable used to mark points for the upper triangle.
#' @param varname.pchdown A character string with the name of the variable used to mark points for the lower triangle.
#' @param vec.colorup A color vector (character or numeric) with a length equal to the number of levels of `varname.colorup`.
#' @param vec.colordown A color vector (character or numeric) with a length equal to the number of levels of `varname.colordown`.
#' @param vec.pchup A symbol vector (character or numeric) with a length equal to the number of levels of `varname.pchup`.
#' @param vec.pchdown A symbol vector (character or numeric) with a length equal to the number of levels of `varname.pchdown`.
#'
#' @return A matrix of scatter plots.
#'
#' @details
#' Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
#' The same rule applies for \code{design} or \code{lmpDataList}.
#'
#' @examples
#'
#' data("UCH")
#'
#' # basic usage
#' plotScatterM(
#'   Y = UCH$outcomes, design = UCH$design, cols = c(1:4)
#' )
#'
#' # equivalent to:
#' plotScatterM(
#'   lmpDataList = UCH, cols = c(1:4)
#' )
#'
#' # with optionnal arguments
#' plotScatterM(
#'   Y = UCH$outcomes, design = UCH$design, cols = c(1:4),
#'   varname.colorup = "Hippurate", varname.colordown = "Citrate",
#'   varname.pchup = "Time", varname.pchdown = "Day",
#'   vec.colorup = c(2, 4, 1),
#'   vec.colordown = c("orange", "purple", "green"),
#'   vec.pchup = c(1, 2), vec.pchdown = c("a", "b")
#' )
#'
#' @import graphics grDevices
#' @import ggsci


plotScatterM <- function(Y = NULL, design = NULL, lmpDataList = NULL, cols,
                         labelVector = NULL, title = "Scatterplot matrix",
                         varname.colorup = NULL, varname.colordown = NULL,
                         varname.pchup = NULL, varname.pchdown = NULL,
                         vec.colorup = NULL, vec.colordown = NULL,
                         vec.pchup = NULL, vec.pchdown = NULL) {
  # Check arguments
  checkArg(Y, "matrix", can.be.null = TRUE)
  checkArg(design, "data.frame", can.be.null = TRUE)
  checkArg(labelVector, "str", can.be.null = TRUE)
  checkArg(title, c("str", "length1"), can.be.null = TRUE)
  checkArg(varname.colorup, c("str", "length1"), can.be.null = TRUE)
  checkArg(varname.colordown, c("str", "length1"), can.be.null = TRUE)
  checkArg(varname.pchup, c("str", "length1"), can.be.null = TRUE)
  checkArg(varname.pchdown, c("str", "length1"), can.be.null = TRUE)


  # define Y
  if (is.null(Y) & is.null(lmpDataList)) {
    stop("both Y and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(Y) & !is.null(lmpDataList)) {
    message("both Y and lmpDataList are non-NULL, Y will be used to perform PCA")
  } else if (is.null(Y) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    Y <- lmpDataList$outcomes
  }

  # define design
  if (is.null(design) & is.null(lmpDataList)) {
    stop("both design and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(design) & !is.null(lmpDataList)) {
    message("both design and lmpDataList are non-NULL, design will be used to perform plotScatterM")
  } else if (is.null(design) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    design <- lmpDataList$design
  }


  if (is.numeric(cols)) {
    checkArg(cols, c("pos", "int"), can.be.null = FALSE)
  }

  if (!is.numeric(cols) & !is.character(cols)) {
    stop("cols is neither numeric or character")
  }

  if (!is.null(varname.colorup)) {
    if (!(varname.colorup %in% colnames(design))) {
      stop(
        "varname.colorup (", varname.colorup,
        ") is not matching column names of design: ",
        paste(colnames(design), collapse = ", ")
      )
    }
  }

  if (!is.null(varname.colordown)) {
    if (!(varname.colordown %in% colnames(design))) {
      stop(
        "varname.colordown (", varname.colordown,
        ") is not matching column names of design: ",
        paste(colnames(design), collapse = ", ")
      )
    }
  }

  if (!is.null(varname.pchup)) {
    if (!(varname.pchup %in% colnames(design))) {
      stop(
        "varname.pchup (", varname.pchup,
        ") is not matching column names of design: ",
        paste(colnames(design), collapse = ", ")
      )
    }
  }

  if (!is.null(varname.pchdown)) {
    if (!(varname.pchdown %in% colnames(design))) {
      stop(
        "varname.pchdown (", varname.pchdown,
        ") is not matching column names of design: ",
        paste(colnames(design), collapse = ", ")
      )
    }
  }

  if (!is.null(varname.colorup) & !is.null(vec.colorup)) {
    if (length(levels(design[[varname.colorup]])) != length(vec.colorup)) {
      stop("vec.colorup must have the same length as varname.colorup")
    }
  }

  if (!is.null(varname.colordown) & !is.null(vec.colordown)) {
    if (length(levels(design[[varname.colordown]])) != length(vec.colordown)) {
      stop("vec.colordown must have the same length as varname.colordown")
    }
  }

  if (!is.null(varname.pchup) & !is.null(vec.pchup)) {
    if (length(levels(design[[varname.pchup]])) != length(vec.pchup)) {
      stop("vec.pchup must have the same length as varname.pchup")
    }
  }

  if (!is.null(varname.pchdown) & !is.null(vec.pchdown)) {
    if (length(levels(design[[varname.pchdown]])) != length(vec.pchdown)) {
      stop("vec.pchdown must have the same length as varname.pchdown")
    }
  }

  vars <- c(varname.colorup,varname.colordown,varname.pchup,varname.pchdown)
  if (!is.null(vars)) {
    testClass <- mapply(is.factor, design[, vars])
    if (!all(testClass)) {
      warning("at least one variable used as color or pch is not a factor and will be converted as such")
    }
  }


  if (is.null(labelVector)) {
    labelVector <- colnames(Y[, cols])
  }

  if (length(labelVector) != length(cols)) {
    stop("labelVector must have the same length as cols")
  }


  # Determine the graphic parameters ===============
  plot_type_colored <- TRUE
  plot_type_pch <- TRUE

  # varnames
  if (is.null(varname.colordown) & is.null(varname.colorup)) {
    if (!is.null(varname.pchup) & !is.null(varname.pchdown)) {
      varname.colordown <- varname.pchdown
      varname.colorup <- varname.pchup
    } else if (!is.null(varname.pchup)) {
      varname.colordown <- varname.pchup
      varname.colorup <- varname.pchup
    } else if (!is.null(varname.pchdown)) {
      varname.colordown <- varname.pchdown
      varname.colorup <- varname.pchdown
    }
  }

  if (is.null(varname.pchdown) & is.null(varname.pchup)) {
    if (!is.null(varname.colorup) & !is.null(varname.colordown)) {
      varname.pchdown <- varname.colordown
      varname.pchup <- varname.colorup
    } else if (!is.null(varname.colorup)) {
      varname.pchdown <- varname.colorup
      varname.pchup <- varname.colorup
    } else if (!is.null(varname.colordown)) {
      varname.pchdown <- varname.colordown
      varname.pchup <- varname.colordown
    }
  }


  if (is.null(varname.colorup)) {
    if (is.null(varname.colordown)) {
      plot_type_colored <- FALSE
    } else {
      varname.colorup <- varname.colordown
      if (is.null(vec.colordown)) {
        vec.colorup <- ggsci::pal_d3("category20")(length(levels(design[, which(names(design) == varname.colorup)])))
      } else {
        vec.colorup <- vec.colordown
      }
    }
  } else {
    if (is.null(varname.colordown)) {
      varname.colordown <- varname.colorup
    }
    if (is.null(vec.colorup)) { # Default
      if (is.null(vec.colordown)) {
        vec.colorup <- ggsci::pal_d3("category20")(length(levels(design[, which(names(design) == varname.colorup)])))
      } else if (varname.colordown == varname.colorup) {
        vec.colorup <- vec.colordown
      }
    }
  }


  if (is.null(varname.colordown)) {
    if (is.null(varname.colorup)) {
      plot_type_colored <- FALSE
    } else { # Copied
      varname.colordown <- varname.colorup
      if (is.null(vec.colorup)) {
        vec.colordown <- ggsci::pal_igv("default")(length(levels(design[, which(names(design) == varname.colordown)])))
      } else {
        vec.colordown <- vec.colorup
      }
    }
  } else {
    if (is.null(varname.colorup)) {
      varname.colorup <- varname.colordown
    }
    if (is.null(vec.colordown)) { # Default
      # if(is.null(vec.colorup)){
      #   vec.colordown = grDevices::topo.colors(length(levels(design[,which(names(design)==varname.colordown)])),alpha=1)
      # }else
      if (varname.colorup == varname.colordown) {
        vec.colordown <- vec.colorup
      } else {
        vec.colordown <- ggsci::pal_igv("default")(length(levels(design[, which(names(design) == varname.colordown)])))
      }
    }
  }

  if (is.null(varname.pchdown)) {
    if (is.null(varname.pchup)) {
      plot_type_pch <- FALSE
    } else {
      varname.pchdown <- varname.pchup
      if (is.null(vec.pchup)) {
        # vec.pchdown = c(seq_along(levels(design[,which(names(design)==varname.pchdown)])))
        pchs <- c(4, 16, 2, 1, 3, 0, 8, 6, 7, 5, seq(9, 15), seq(17, 25))
        vec.pchdown <- pchs[seq_along(levels(design[, which(names(design) == varname.pchdown)]))]
      } else {
        vec.pchdown <- vec.pchup
      }
    }
  } else {
    if (is.null(vec.pchdown)) {
      if (is.null(vec.pchup)) {
        # vec.pchdown = c(seq_along(levels(design[,which(names(design)==varname.pchdown)])))
        pchs <- c(4, 16, 2, 1, 3, 0, 8, 6, 7, 5, seq(9, 15), seq(17, 25))
        vec.pchdown <- pchs[seq_along(levels(design[, which(names(design) == varname.pchdown)]))]
      } else if (varname.pchdown == varname.pchup) {
        vec.pchdown <- vec.pchup
      } else {
        vec.pchdown <- seq_along(levels(design[, which(names(design) == varname.pchdown)]))
      }
    }
  }

  if (is.null(varname.pchup)) {
    if (is.null(varname.pchdown)) {
      plot_type_pch <- FALSE
    } else {
      varname.pchup <- varname.pchdown
      if (is.null(vec.pchdown)) {
        vec.pchup <- vec.pchdown
      } else {
        vec.pchup <- seq(20, (20 + (length(levels(design[, which(names(design) == varname.pchup)])))))
      }
    }
  } else {
    if (is.null(vec.pchup)) {
      # if(is.null(vec.pchdown)){
      #   vec.pchup = c(seq_along(levels(design[,which(names(design)==varname.pchup)])))
      # }else
      if (varname.pchdown == varname.pchup) {
        vec.pchup <- vec.pchdown
      } else {
        vec.pchup <- seq(20, (20 +
          (length(levels(design[, which(names(design) == varname.pchup)])))))
      }
    }
  }

  # Creation of the upper panel
  panelup <- function(x, y) {
    iEffect.color <- design[, which(names(design) == varname.colorup)]
    iEffect.pch <- design[, which(names(design) == varname.pchup)]

    var.color.level <- levels(iEffect.color)
    var.pch.level <- levels(iEffect.pch)

    colorvector <- vector()
    pchvector <- vector()

    for (i in seq_along(var.color.level)) {
      colorvector[iEffect.color == var.color.level[i]] <- vec.colorup[i]
    }
    for (i in seq_along(var.pch.level)) {
      pchvector[iEffect.pch == var.pch.level[i]] <- vec.pchup[i]
    }

    graphics::points(x, y, col = colorvector, pch = pchvector)
  }

  # Creation of the lower panel
  paneldown <- function(x, y) {
    iEffect.color <- design[, which(names(design) == varname.colordown)]
    iEffect.pch <- design[, which(names(design) == varname.pchdown)]

    var.color.level <- levels(iEffect.color)
    var.pch.level <- levels(iEffect.pch)

    colorvector <- vector()
    pchvector <- vector()

    for (i in seq_along(var.color.level)) {
      colorvector[iEffect.color == var.color.level[i]] <- vec.colordown[i]
    }
    for (i in seq_along(var.pch.level)) {
      pchvector[iEffect.pch == var.pch.level[i]] <- vec.pchdown[i]
    }

    graphics::points(x, y, col = colorvector, pch = pchvector)
  }

  # Creation Legend

  Legend <- function() {
    # Dividing the space for each legend

    totallegend <- length(c(vec.pchup, vec.pchdown, vec.colordown, vec.colorup))
    spacebyline <- 0.85 / totallegend

    spacecolup <- c(0.05 + length(vec.pchup) * spacebyline +
      length(vec.pchdown) * spacebyline +
      length(vec.colordown) * spacebyline, 0.9)
    spacepchup <- c(0.05 + length(vec.pchup) * spacebyline +
      length(vec.pchdown) * spacebyline, 0.05 +
      length(vec.pchup) * spacebyline +
      length(vec.pchdown) * spacebyline +
      length(vec.colordown) * spacebyline)
    spacepchdown <- c(0.05, 0.05 + length(vec.pchup) * spacebyline)
    spacecoldown <- c(0.05 + length(vec.pchup) * spacebyline, 0.05 +
      length(vec.pchup) * spacebyline +
      length(vec.pchdown) * spacebyline)

    # Plotting legend

    graphics::legend(
      x = c(0.93, 1), y = spacecolup,
      title = varname.colorup,
      legend = levels(design[, which(names(design) == varname.colorup)]),
      bty = "n",
      col = vec.colorup,
      pch = 15,
      inset = c(0.03, 0.1),
      title.adj = 0,
      cex = 0.7
    )

    graphics::legend(
      x = c(0.93, 1), y = spacepchup,
      title = varname.pchup,
      legend = levels(design[, which(names(design) == varname.pchup)]),
      bty = "n",
      pch = vec.pchup,
      inset = c(0.03, 0.6),
      cex = 0.7,
      title.adj = 0,
      xjust = 0,
      adj = 0
    )

    graphics::legend(
      x = c(0.93, 1), y = spacecoldown,
      title = varname.colordown,
      legend = levels(design[, which(names(design) == varname.colordown)]),
      bty = "n",
      col = vec.colordown,
      pch = 15,
      inset = c(0.03, 0.3),
      title.adj = 0,
      cex = 0.7
    )

    graphics::legend(
      x = c(0.93, 1), y = spacepchdown,
      title = varname.pchdown,
      legend = levels(design[, which(names(design) == varname.pchdown)]),
      bty = "n",
      pch = vec.pchdown,
      inset = c(0.03, 0.9),
      cex = 0.7,
      title.adj = 0,
      xjust = 0,
      adj = 0
    )
  }


  # Building of the pairs plot

  if (!plot_type_colored & !plot_type_pch) {
    graphics::pairs(Y[, cols],
      gap = 0.3,
      labels = labelVector
    )
    graphics::title(main = title, font.main = 1, adj = 0, line = 3)
  } else if (plot_type_colored & plot_type_pch) {
    graphics::pairs(Y[, cols],
      upper.panel = panelup,
      lower.panel = paneldown,
      gap = 0.3,
      oma = c(3, 3, 5, 10),
      labels = labelVector
    )
    graphics::title(main = title, font.main = 1, adj = 0, line = 3)
    graphics::par(xpd = TRUE)
    Legend()
    graphics::par(xpd = FALSE)
  }
}

contrastSS <- function(resLmpModelMatrix) {
  # Attributing name
  design <- resLmpModelMatrix$LmpDataList$design
  modelMatrixByEffect <- resLmpModelMatrix$modelMatrixByEffect
  effectsNamesUnique <- resLmpModelMatrix$effectsNamesUnique
  effectsNamesAll <- resLmpModelMatrix$effectsNamesAll
  neffect <- length(effectsNamesUnique)
  nparam <- length(effectsNamesAll)

  # Creating empy list
  L <- list()

  # Filling the list for every effect
  starting_col <- 1
  Mat <- diag(nparam)

  for (iEffect in seq_len(neffect)) {
    selection <- which(names(modelMatrixByEffect) == effectsNamesUnique[iEffect])
    MatEffect <- modelMatrixByEffect[[selection]]
    npi <- ncol(MatEffect)

    L_iEffect <- Mat[starting_col:(starting_col + npi - 1), ]
    L[[iEffect]] <- L_iEffect

    starting_col <- starting_col + npi
  }
  names(L) <- effectsNamesUnique
  return(L)
}

set.seed(1)
design <- data.frame(
    factA = factor(c(rep(1, 5), rep(2, 5))),
    factB = factor(rep(c(1, 2), 5))
)
outcomes <- matrix(rnorm(30), ncol = 3)
formula <- "~ factA+factB"
lmpDL <- list(design = design, outcomes = outcomes, formula = formula)
mM <- model.matrix(as.formula(lmpDL$formula), lmpDL$design)
mMBE <- list(
    Intercept = mM[, "(Intercept)", drop = FALSE],
    factA = mM[, "factA2", drop = FALSE],
    factB = mM[, "factB2", drop = FALSE]
)
eNU <- eNA <- c("Intercept", "factA", "factB")

inArg <- list(
    lmpDataList = lmpDL, modelMatrix = mM,
    modelMatrixByEffect = mMBE,
    effectsNamesUnique = eNU,
    effectsNamesAll = eNA
)

res_contrastSS <- contrastSS(inArg)

test_that("contrastSS runs correctly", {
    expect_equal(
        res_contrastSS,
        list(Intercept = c(1, 0, 0), factA = c(0, 1, 0), factB = c(0, 0, 1))
    )
})

#' @export lmpScoreScatterPlotM
#' @title Scatterplot matrix of effect matrices scores
#'
#' @description
#' Plots the scores of all model effects simultaneously in a scatterplot matrix.
#' By default, the first PC only is kept for each model effect and, as a wrapper of \code{\link{plotScatterM}}, the choice of symbols and colors to distinguish factor levels allows an enriched visualization of the factors’ effect on the responses.
#'
#' @param resLmpPcaEffects A list corresponding to the output value of \code{\link{lmpPcaEffects}}.
#' @param effectNames A character vector with the name of the effects to plot.
#' @param PCdim A numeric vector with the same length than effectNames and indicating the number of component to plot.
#' @param modelAbbrev A logical whether to abbreviate the interaction terms or not.
#' @param ... Additional arguments to be passed to \code{\link{plotScatterM}}.
#'
#' @return A matrix of graphs
#'
#' @details
#' `lmpScoreScatterPlotM` is a wrapper of \code{\link{plotScatterM}}.
#'
#' @examples
#'
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' ResLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resLmpPcaEffects <- lmpPcaEffects(ResLmpEffectMatrices, method = "ASCA-E")
#'
#' lmpScoreScatterPlotM(resLmpPcaEffects,
#'   varname.colorup = "Citrate",
#'   varname.pchup = "Hippurate",
#'   varname.pchdown = "Day",
#'   varname.colordown = "Time"
#' )
#'
#' # advanced setting
#' lmpScoreScatterPlotM(resLmpPcaEffects,
#'   modelAbbrev = FALSE,
#'   effectNames = c("Citrate", "Hippurate", "Hippurate:Citrate"),
#'   PCdim = c(2, 2, 2),
#'   varname.colorup = "Citrate",
#'   vec.colorup = c("red", "blue", "green"),
#'   varname.pchup = "Hippurate",
#'   vec.pchup = c(1, 2, 3),
#'   varname.pchdown = "Day",
#'   vec.pchdown = c(4, 5),
#'   varname.colordown = "Time",
#'   vec.colordown = c("brown", "grey")
#' )
#'
#' @import graphics grDevices

lmpScoreScatterPlotM <- function(resLmpPcaEffects,
                                 effectNames = NULL,
                                 PCdim = NULL,
                                 modelAbbrev = FALSE, ...) {
  # Define allEffect in regard to effectNames
  if (is.null(effectNames)) {
    allEffect <- TRUE
  } else {
    allEffect <- FALSE
  }

  # Checking arguments ===============
  checkArg(resLmpPcaEffects, "list", can.be.null = FALSE)
  checkArg(PCdim, "num", can.be.null = TRUE)
  checkArg(effectNames, "str", can.be.null = TRUE)
  checkArg(modelAbbrev, "bool", can.be.null = FALSE)

  # Checking resLmpPcaEffects object and match with effectNames
  if (!identical(
    names(resLmpPcaEffects[seq((length(resLmpPcaEffects) - 7), length(resLmpPcaEffects))]),
    c(
      "Residuals", "lmpDataList", "effectsNamesUnique",
      "effectsNamesUniqueCombined",
      "method", "type3SS", "variationPercentages",
      "combineEffects"
    )
  )) {
    stop("resLmpPcaEffects is not an output value of lmpPcaEffects")
  }

  if (allEffect == FALSE) {
    for (i in seq_along(effectNames)) {
      if (!effectNames[i] %in% names(resLmpPcaEffects)) {
        "One of the elements from effectNames is not in resLmpPcaEffects"
      }
    }
  }

  # Check the number of PC to use or create it with first PC only
  if (is.null(PCdim)) {
    classicalPC <- TRUE
    if (allEffect == TRUE) {
      effectsNamesUnique <- resLmpPcaEffects$effectsNamesUniqueCombined
      effectsNamesUnique <- effectsNamesUnique[effectsNamesUnique != "Intercept"]
      PCdim <- rep(1, (length(effectsNamesUnique) + 1)) # +1 for the residuals
      effectsNamesUniqueRes <- c(effectsNamesUnique, "Residuals")
    } else {
      PCdim <- rep(1, length(effectNames))
    }
  } else {
    classicalPC <- FALSE
    if (allEffect == TRUE) {
      effectsNamesUnique <- resLmpPcaEffects$effectsNamesUniqueCombined
      effectsNamesUnique <- effectsNamesUnique[effectsNamesUnique != "Intercept"]
      effectsNamesUniqueRes <- c(effectsNamesUnique, "Residuals")
    }
  }

  # Checking equivalent length
  if (allEffect == FALSE) {
    if (length(PCdim) != length(effectNames)) {
      stop("length(PCdim) differs from length(effectNames)")
    }
  }


  # Create the matrix for the "pairs" function ===============
  k <- sum(PCdim) # Length of the diagonal of the plot
  n <- length(resLmpPcaEffects[[1]]$scores[, 1]) # Find number of observations

  var <- c(rep(1, k))

  coomatrix <- matrix(data = NA, nrow = n, ncol = k)
  colnames(coomatrix) <- seq_len(k)

  if (allEffect == TRUE) {
    if (classicalPC) { # Only PC1 to all effects
      for (i in seq_along(effectsNamesUniqueRes)) {
        coomatrix[, i] <- resLmpPcaEffects[[effectsNamesUniqueRes[i]]]$scores[, PCdim[i]]
        colnames(coomatrix)[i] <- paste(effectsNamesUniqueRes[i], "PC1")
        var[i] <- (resLmpPcaEffects$variationPercentages[effectsNamesUniqueRes[i]] *
          resLmpPcaEffects[[effectsNamesUniqueRes[i]]]$var[1]) / 100
      }
    } else { # All effects but more than 1 PC
      l <- 1
      for (j in seq_along(PCdim)) { # Some effects must be printed on PC2 or more

        if (PCdim[j] == 1) {
          coomatrix[, l] <- resLmpPcaEffects[[effectsNamesUniqueRes[j]]]$scores[, PCdim[j]]
          colnames(coomatrix)[l] <- paste(effectsNamesUniqueRes[j], "PC1")
          var[l] <- (resLmpPcaEffects$variationPercentages[effectsNamesUniqueRes[j]] *
            resLmpPcaEffects[[effectsNamesUniqueRes[j]]]$var[1]) / 100
          l <- l + 1
        } else {
          for (m in seq_len(PCdim[j])) { # Get the others PC
            coomatrix[, l] <- resLmpPcaEffects[[effectsNamesUniqueRes[j]]]$scores[, m]
            colnames(coomatrix)[l] <- paste(
              effectsNamesUniqueRes[j],
              colnames(resLmpPcaEffects[[j]]$scores)[m]
            )
            var[l] <- (resLmpPcaEffects$variationPercentages[effectsNamesUniqueRes[j]] *
              resLmpPcaEffects[[effectsNamesUniqueRes[j]]]$var[m]) / 100
            l <- l + 1
          }
        }
      }
    }
  } else { # Not all effect and on one or more PC

    l <- 1
    for (i in seq_along(effectNames)) {
      iEffect_temp <- which(names(resLmpPcaEffects) == effectNames[i])
      iEffect <- resLmpPcaEffects[[iEffect_temp]]

      for (j in seq_len(PCdim[i])) {
        coomatrix[, l] <- iEffect$scores[, j]
        colnames(coomatrix)[l] <- paste(
          names(resLmpPcaEffects)[iEffect_temp],
          colnames(iEffect$scores)[j]
        )
        var[l] <- (resLmpPcaEffects$variationPercentages[iEffect_temp] *
          resLmpPcaEffects[[iEffect_temp]]$var[j]) / 100
        l <- l + 1
      }
    }
  }

  # Creation of the labels
  labelvector <- vector()

  for (i in seq_len(k)) {
    div_name <- strsplit(x = colnames(coomatrix)[i], split = " ")
    div_name <- div_name[[1]]
    effect_name <- div_name
    if (modelAbbrev == TRUE) {
      effect_name <- ModelAbbrev(div_name)
    } # Abbrev interaction term
    labelvector[i] <- paste(
      effect_name[1], "\n",
      div_name[2], "\n", round(var[i], 2), "%"
    )
  }



  # Plotting the graph

  plotScatterM(
    Y = coomatrix,
    cols = seq_len(ncol(coomatrix)),
    design = resLmpPcaEffects$lmpDataList$design,
    labelVector = labelvector, ...
  )
}

lmpSS <- function(resLmpEffectMatrices, listcontrast) {
  computeSS <- function(Xmat, L, coef) {
    if (is.vector(L)) {
      L <- t(L)
    }
    LB <- L %*% coef
    BL <- t(LB)
    mat <- BL %*% solve(L %*% solve(t(Xmat) %*% Xmat) %*% t(L)) %*% LB
    SS <- sum(diag(mat))
    return(SS)
  }

  L <- listcontrast

  result <- vector()
  var_percentage <- vector()

  Y_withoutIntercept <- resLmpEffectMatrices$lmpDataList$outcomes -
    resLmpEffectMatrices$effectMatrices$Intercept
  denom <- norm(x = data.matrix(Y_withoutIntercept), "F")^2

  for (i in seq_along(L)) {
    result[i] <- computeSS(
      Xmat = resLmpEffectMatrices$modelMatrix, L = L[[i]],
      coef = resLmpEffectMatrices$parameters
    )
    var_percentage[i] <- (result[i] / denom) * 100
  }
  # all effects plus residuals
  result <- c(result, ((norm(
    x = resLmpEffectMatrices$residuals,
    "F"
  )^2) / denom) * 100)
  names(result) <- c(names(L), "Residuals")

  var_percentage <- c(var_percentage, ((norm(
    x = resLmpEffectMatrices$residuals,
    "F"
  )^2) / denom) * 100)
  names(var_percentage) <- c(names(L), "Residuals")

  var_percentage <- var_percentage[names(var_percentage) != "Intercept"]

  # names(result) = c(resLmpEffectMatrices$effectsNamesUnique,"Residuals")
  # names(var_percentage) = c(names(listcontrast)[names(listcontrast) != "Intercept"],"Residuals")

  LMSS <- list(type3SS = result, variationPercentages = var_percentage)
  return(LMSS)
}

#' @export plotMeans
#' @title Means plot
#'
#' @description
#' For a given response variable, draws a plot of the response means by levels of up to three categorical factors from the design. When the design is balanced, it allows to visualize main effects or interactions for the response of interest. For unbalanced designs, this plot must be used with caution.
#'
#' @param Y A numerical matrix containing the columns to be drawn. Can be `NULL` if `lmpDataList` is defined.
#' @param design A \eqn{n \times k} "freely encoded" experimental design data.frame. Can be `NULL` if `lmpDataList` is defined.
#' @param lmpDataList If not `NULL`, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.
#' @param cols A vector with either the column name(s) of the \eqn{Y} matrix to plot (character) or the column index position(s) (integer).
#' @param x A character string giving the `design` factor whose levels will form the x-axis.
#' @param z A character string giving the `design` factor whose levels will form the traces.
#' @param w A character string giving the `design` factor whose levels will be used for the facet.
#' @param title Plot title.
#' @param xlab If not \code{NULL}, the label for the x-axis.
#' @param ylab If not \code{NULL}, the label for the y-axis.
#' @param color If not \code{NULL}, the color of the points and the line.
#' @param shape If not \code{NULL}, the points shape.
#' @param linetype If not \code{NULL}, the line type.
#' @param size Points size.
#' @param hline If not \code{NULL}, draws (a) horizontal line(s).
#' @param theme The `ggplot2` theme, see `?ggtheme` for more info.
#'
#' @return A list of `ggplot2` means plot(s).
#'
#' @details
#' Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
#' The same rule applies for \code{design} or \code{lmpDataList}.
#'
#' @examples
#'
#' data("UCH")
#' # 1 factor
#' plotMeans(
#'   Y = UCH$outcomes, design = UCH$design, cols = "4.0628702",
#'   x = "Hippurate", color = "blue"
#' )
#'
#' # equivalent to:
#' plotMeans(
#'   lmpDataList = UCH, cols = "4.0628702",
#'   x = "Hippurate", color = "blue"
#' )
#'
#' # 2 factors
#' plotMeans(
#'   Y = UCH$outcomes, design = UCH$design, cols = c(364, 365),
#'   x = "Hippurate", z = "Time", shape = c(15, 1)
#' )
#'
#' # 3 factors
#' plotMeans(
#'   Y = UCH$outcomes, design = UCH$design, cols = c(364, 365),
#'   x = "Hippurate", z = "Time", w = "Citrate", linetype = c(3, 3)
#' )
#'
#' @import ggplot2
#' @importFrom stats aggregate
#' @import reshape2

plotMeans <- function(Y = NULL, design = NULL, lmpDataList = NULL,
                      cols = NULL, x, z = NULL, w = NULL,
                      title = NULL, xlab = NULL, ylab = NULL,
                      color = NULL, shape = NULL, linetype = NULL, size = 2,
                      hline = NULL, theme = theme_bw()) {
  # checks =========================

  checkArg(Y, "matrix", can.be.null = TRUE)
  checkArg(design, "data.frame", can.be.null = TRUE)
  if (is.numeric(cols)) {
    checkArg(cols, c("pos", "int"), can.be.null = TRUE)
  }
  if (!is.numeric(cols) & !is.character(cols) & !is.null(cols)) {
    stop("cols is neither numeric or character")
  }
  checkArg(x, c("str", "length1"), can.be.null = FALSE)
  checkArg(z, c("str", "length1"), can.be.null = TRUE)
  checkArg(w, c("str", "length1"), can.be.null = TRUE)
  checkArg(title, c("str"), can.be.null = TRUE)
  checkArg(xlab, c("str", "length1"), can.be.null = TRUE)
  checkArg(ylab, c("str"), can.be.null = TRUE)
  checkArg(color, c("str"), can.be.null = TRUE)
  checkArg(shape, c("num", "pos"), can.be.null = TRUE)
  checkArg(linetype, c("num", "pos"), can.be.null = TRUE)
  checkArg(size, c("num", "pos", "length1"), can.be.null = FALSE)
  checkArg(hline, "num", can.be.null = TRUE)

  if (is.null(cols)) {
    if (ncol(Y) < 10) {
      cols <- c(seq_len(ncol(Y)))
    } else {
      cols <- seq_len(10)
    }
  }

  # define Y
  if (is.null(Y) & is.null(lmpDataList)) {
    stop("both Y and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(Y) & !is.null(lmpDataList)) {
    message("both Y and lmpDataList are non-NULL, Y will be used to perform PCA")
  } else if (is.null(Y) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    Y <- lmpDataList$outcomes
  }

  # define design
  if (is.null(design) & is.null(lmpDataList)) {
    stop("both design and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(design) & !is.null(lmpDataList)) {
    message("both design and lmpDataList are non-NULL, design will be used to perform plotMeans")
  } else if (is.null(design) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    design <- lmpDataList$design
  }


  if (!x %in% colnames(design)) {
    stop(x, " is not a column of the design")
  }
  if (!is.null(z) && !z %in% colnames(design)) {
    stop(z, " is not a column of the design")
  }
  if (!is.null(w) && !w %in% colnames(design)) {
    stop(w, " is not a column of the design")
  }

  if (is.null(z) & !is.null(w)) {
    stop("w must be NULL if z is NULL")
  }


  if (is.null(z)) {
    if (!is.null(color) & length(color) != 1) {
      stop("color must be of length 1")
    }
    if (!is.null(shape) & length(shape) != 1) {
      stop("shape must be of length 1")
    }
    if (!is.null(linetype) & length(linetype) != 1) {
      stop("linetype must be of length 1")
    }
  } else if (!is.null(z)) {
    zlevels <- length(levels(factor(design[, z])))
    if (!is.null(color) & length(color) != zlevels) {
      stop("color must be the same length as the number of levels of z")
    }
    if (!is.null(shape) & length(shape) != zlevels) {
      stop("shape must be the same length as the number of levels of z")
    }
    if (!is.null(linetype) & length(linetype) != zlevels) {
      stop("linetype must be the same length as the number of levels of z")
    }
  }

  # prepare the arguments  ==============================

  if (is.character(cols)) {
    cols <- grep(cols, colnames(Y))
  }

  if (max(cols) > ncol(Y)) {
    stop(
      "Columns (", paste(cols, collapse = ","),
      ") is beyond the number of columns of Y (",
      ncol(Y), ")"
    )
  }

  dataplot <- stats::aggregate(Y[, cols],
    by = design[c(x, z, w)], mean
  )
  names(dataplot) <- c(x, z, w, colnames(Y)[cols])

  if (is.null(title)) {
    if (is.null(z)) {
      title <- paste0("Mean response as a function of ", x)
    } else if (!is.null(z) & is.null(w)) {
      title <- paste0(
        "Mean response as a function of ",
        x, " and ", z
      )
    } else if (!is.null(w)) {
      title <- paste0(
        "Mean response as a function of ",
        x, ", ", z, " and ", w
      )
    }
  }
  title <- rep(title, times = length(cols))

  if (is.null(xlab)) {
    xlab <- x
  }

  if (is.null(ylab)) {
    ylab <- rep(colnames(Y)[cols], times = length(cols))
  } else {
    ylab <- rep(ylab, times = length(cols))
  }

  # Means Plot  ==============================

  if (is.null(z)) {
    buildFig <- function(columns) {
      # color
      if (is.null(color)) {
        color <- "#E97D72"
      }

      if (is.null(shape)) {
        shape <- 16
      }

      if (is.null(linetype)) {
        linetype <- 1
      }

      # ggplot
      columnsNames <- colnames(Y)[columns]
      dataplot1 <- reshape2::melt(dataplot,
        id.vars = c(x),
        measure.vars = columnsNames,
        variable.name = "Variables"
      )

      fig <- ggplot2::ggplot(
        data = dataplot1,
        ggplot2::aes(
          x = dataplot1[, x],
          y = dataplot1[, "value"],
          group = 1
        )
      ) +
        ggplot2::geom_point(col = color, shape = shape, size = size) +
        ggplot2::geom_line(col = color, linetype = linetype)
    }

    fig <- lapply(cols, FUN = buildFig)
  } else if (!is.null(z) && is.null(w)) { # z is not null
    buildFig <- function(columns) {
      columnsNames <- colnames(Y)[columns]

      dataplot1 <- reshape2::melt(dataplot,
        id.vars = c(x, z),
        measure.vars = columnsNames,
        variable.name = "Variables"
      )

      fig <- ggplot2::ggplot(
        data = dataplot1,
        ggplot2::aes(
          x = dataplot1[, x],
          y = dataplot1[, "value"],
          group = dataplot1[, z],
          col = dataplot1[, z],
          linetype = dataplot1[, z],
          shape = dataplot1[, z]
        )
      ) +
        ggplot2::geom_point(size = size) +
        ggplot2::geom_line() +
        ggplot2::labs(
          col = names(dataplot1)[2],
          linetype = names(dataplot1)[2],
          shape = names(dataplot1)[2]
        )
    }

    fig <- lapply(cols, FUN = buildFig)
  } else if (!is.null(z) && !is.null(w)) { # z and w are not null
    buildFig <- function(columns) {
      columnsNames <- colnames(Y)[columns]

      dataplot1 <- reshape2::melt(dataplot,
        id.vars = c(x, z, w),
        measure.vars = columnsNames,
        variable.name = "Variables"
      )

      fig <- ggplot2::ggplot(
        data = dataplot1,
        ggplot2::aes(
          x = dataplot1[, x],
          y = dataplot1[, "value"],
          group = dataplot1[, z],
          col = dataplot1[, z],
          linetype = dataplot1[, z],
          shape = dataplot1[, z]
        )
      ) +
        ggplot2::geom_point(size = size) +
        ggplot2::geom_line() +
        ggplot2::labs(
          col = names(dataplot1)[2],
          linetype = names(dataplot1)[2],
          shape = names(dataplot1)[2]
        ) +
        ggplot2::facet_wrap(w, labeller = label_both)
    }

    fig <- lapply(cols, FUN = buildFig)
  }

  # color
  if (!is.null(color)) {
    fig <- lapply(seq_along(cols), function(x) {
      fig[[x]] +
        ggplot2::scale_color_manual(values = color)
    })
  }

  # shape
  if (!is.null(shape)) {
    fig <- lapply(seq_along(cols), function(x) {
      fig[[x]] +
        ggplot2::scale_shape_manual(values = shape)
    })
  }

  # linetype
  if (!is.null(linetype)) {
    fig <- lapply(seq_along(cols), function(x) {
      fig[[x]] +
        ggplot2::scale_linetype_manual(values = linetype)
    })
  }

  fig <- lapply(seq_along(cols), function(x) {
    fig[[x]] +
      ggplot2::labs(
        title = title[x],
        x = xlab, y = ylab[x]
      ) + theme
  })

  if (!is.null(hline)) {
    fig <- lapply(fig, function(x) {
      x + ggplot2::geom_hline(
        yintercept = hline,
        size = 0.5, linetype = "dashed",
        colour = "gray60"
      )
    })
  }

  names(fig) <- colnames(Y)[cols]

  return(fig)
}

#' @export pcaBySvd
#' @title Singular Value Decomposition for PCA analysis
#'
#' @description
#' Operates a Principal Component Analysis on the `Y` outcome/response matrix by a singular
#' Value Decomposition (the pre-processing involves the mean-centering of `Y`).
#' Outputs are represented with the functions \code{\link{pcaScorePlot}},
#' \code{\link{pcaLoading1dPlot}}, \code{\link{pcaLoading2dPlot}} and \code{\link{pcaScreePlot}}.
#'
#' @param Y The \eqn{n \times m} matrix with \eqn{n} observations and \eqn{m} (response) variables.
#' @param lmpDataList A list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.
#' @param nPC Number of Principal Components to extract.
#'
#' @return A list containing the following elements:
#' \describe{
#'   \item{\code{scores}}{Scores}
#'   \item{\code{loadings}}{Loadings}
#'   \item{\code{eigval}}{Eigenvalues}
#'   \item{\code{singvar}}{Singular values}
#'   \item{\code{var}}{Explained variances}
#'   \item{\code{cumvar}}{Cumulated explained variances}
#'   \item{\code{original.dataset}}{Original dataset}
#'   \item{\code{design}}{Design of the study}
#'
#' }
#'
#' @examples
#'
#' data("UCH")
#'
#' PCA.res1 <- pcaBySvd(Y = UCH$outcomes)
#'
#' PCA.res2 <- pcaBySvd(lmpDataList = UCH)
#'
#' identical(PCA.res1, PCA.res2)
pcaBySvd <- function(Y = NULL, lmpDataList = NULL, nPC = min(dim(Y))) {

  design <- NULL

  if (is.null(Y) & is.null(lmpDataList)) {
    stop("both Y and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(Y) & !is.null(lmpDataList)) {
    message("both Y and lmpDataList are non-NULL, Y will be used to perform PCA")
  } else if (is.null(Y) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE,
                     null_design = TRUE)
    Y <- lmpDataList$outcomes
    design <- lmpDataList$design
  }


  checkArg(Y, "matrix", can.be.null = TRUE)
  checkArg(nPC, c("num", "pos", "length1"), can.be.null = FALSE)

  # PCA =========================
  original.dataset <- Y
  outcomes <- Y
  # column centering of outcomes
  outcomes <- outcomes - matrix(apply(outcomes, 2, mean),
    nrow = dim(outcomes)[1],
    ncol = dim(outcomes)[2],
    byrow = TRUE
  ) # centering of outcomes over the columns

  # SVD of outcomes
  outcomes.svd <- svd(outcomes) # Compute the singular-value decomposition
  outcomes.scores <- outcomes.svd$u %*% diag(outcomes.svd$d) # scores
  outcomes.normscores <- outcomes.svd$u # normalised scores
  outcomes.loadings <- outcomes.svd$v # loadings
  outcomes.singularval <- outcomes.svd$d # singular values
  names(outcomes.singularval) <- paste0("PC", seq_along(outcomes.singularval))

  # X-Variance explained
  outcomes.vars <- outcomes.singularval^2 / (nrow(outcomes) - 1)
  outcomes.eigval <- outcomes.singularval^2
  names(outcomes.eigval) <- paste0("PC", seq_along(outcomes.eigval))

  outcomes.totalvar <- sum(outcomes.vars)
  outcomes.relvars <- outcomes.vars / outcomes.totalvar

  outcomes.variances <- 100 * outcomes.relvars # variance
  names(outcomes.variances) <- paste0(
    "PC",
    seq_along(outcomes.variances)
  )

  outcomes.cumvariances <- cumsum(outcomes.variances) # cumulative variance
  names(outcomes.cumvariances) <- paste0(
    "PC",
    seq_along(outcomes.cumvariances)
  )

  # as matrix

  outcomes.scores <- as.matrix(outcomes.scores)
  dimnames(outcomes.scores) <- list(
    rownames(outcomes),
    paste0("PC", seq_len(ncol(outcomes.scores)))
  )

  outcomes.normscores <- as.matrix(outcomes.normscores)
  dimnames(outcomes.normscores) <- list(
    rownames(outcomes),
    paste0("PC", seq_len(ncol(outcomes.normscores)))
  )

  outcomes.loadings <- as.matrix(outcomes.loadings)
  dimnames(outcomes.loadings) <- list(
    colnames(outcomes),
    paste0("PC", seq_len(ncol(outcomes.loadings)))
  )


  # selection of the first n components

  outcomes.scores <- outcomes.scores[, seq_len(nPC)]
  outcomes.normscores <- outcomes.normscores[, seq_len(nPC)]
  outcomes.loadings <- outcomes.loadings[, seq_len(nPC)]
  outcomes.singularval <- outcomes.singularval[seq_len(nPC)]

  outcomes.variances <- outcomes.variances[seq_len(nPC)]
  outcomes.cumvariances <- outcomes.cumvariances[seq_len(nPC)]



  res <- list(
    scores = outcomes.scores, loadings = outcomes.loadings,
    eigval = outcomes.eigval,
    singvar = outcomes.singularval, var = outcomes.variances,
    cumvar = outcomes.cumvariances,
    original.dataset = original.dataset,
    design = design
  )

  return(res)
}

#' @export pcaScorePlot
#' @title Score plots
#'
#' @description
#' Produces score plots from \code{\link{pcaBySvd}} output with the same graphical options as \code{\link{plotScatter}} as this is a wrapper of this function..
#'
#' @param resPcaBySvd A list corresponding to the output value of \code{\link{pcaBySvd}}.
#' @param design A \eqn{n \times k} "freely encoded" experimental design data.frame.
#' @param axes A numerical vector of length 2 with the Principal Components axes to be drawn.
#' @param title Plot title.
#' @param points_labs_rn Boolean indicating if the rownames of the scores matrix should be plotted.
#' @param ... Additional arguments to be passed to \code{\link{plotScatter}}.
#'
#' @return A `ggplot2` PCA score plot.
#'
#' @details
#' `pcaScorePlot` is a wrapper of \code{\link{plotScatter}}. See `?plotScatter` for more information on the additional arguments.
#'
#'
#' @examples
#'
#' data("UCH")
#'
#' # design is explicitly defined
#' ResPCA <- pcaBySvd(Y = UCH$outcomes)
#'
#' pcaScorePlot(
#'   resPcaBySvd = ResPCA, axes = c(1, 2),
#'   title = "PCA score plot UCH", design = UCH$design,
#'   color = "Hippurate", shape = "Citrate"
#' )
#'
#' # design is recovered from lmpDataList through pcaBySvd()
#' ResPCA <- pcaBySvd(lmpDataList = UCH)
#'
#' pcaScorePlot(
#'   resPcaBySvd = ResPCA, axes = c(1, 2),
#'   title = "PCA score plot UCH",
#'   color = "Hippurate", shape = "Citrate"
#' )
#'
#' @import ggplot2

pcaScorePlot <- function(resPcaBySvd, design = NULL,
                         axes = c(1, 2),
                         title = "PCA score plot",
                         points_labs_rn = FALSE, ...) {
  mcall <- as.list(match.call())[-1L]

  # checks ===================
  checkArg(resPcaBySvd, c("list"), can.be.null = FALSE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)
  checkArg(title, c("str", "length1"), can.be.null = FALSE)
  checkArg(points_labs_rn, c("bool"), can.be.null = FALSE)

  if (!identical(names(resPcaBySvd), c(
    "scores", "loadings", "eigval", "singvar",
    "var", "cumvar", "original.dataset", "design"
  ))) {
    stop("resPcaBySvd is not an output value of pcaBySvd")
  }

  # design
  if (is.null(design) & !is.null(resPcaBySvd$design)) {
    design <- resPcaBySvd$design
  }

  # scores
  scores <- resPcaBySvd$scores
  checkArg(scores, c("matrix"), can.be.null = FALSE)

  if (length(axes) != 2) {
    stop("axes is not of length 2")
  }

  if (max(axes) > ncol(scores)) {
    stop(
      "axes (", paste(axes, collapse = ","),
      ") is beyond the ncol of scores (", ncol(scores), ")"
    )
  }


  # percentage of explained variance   ===================
  pc_var <- resPcaBySvd$var
  pc_var_x <- format(pc_var[pc_var >= 0.1],
    digits = 2,
    trim = TRUE
  )
  pc_var_y <- format(pc_var[pc_var < 0.1],
    digits = 2,
    scientific = TRUE, trim = TRUE
  )
  pc_var_char <- as.character(pc_var)
  pc_var_char[pc_var >= 0.1] <- pc_var_x
  pc_var_char[pc_var < 0.1] <- pc_var_y

  pc_var_char <- paste0(
    "PC", axes, " (",
    pc_var_char[axes], "%)"
  )

  # graphical parameters   ===================
  xlab <- pc_var_char[1]
  ylab <- pc_var_char[2]

  xlim1 <- max(abs(scores[, axes[1]]))
  xlim_val <- c(-xlim1, xlim1)

  ylim1 <- max(abs(scores[, axes[2]]))
  ylim_val <- c(-ylim1, ylim1)

  if (points_labs_rn) {
    if (!"xlab" %in% names(mcall)) {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes, xlab = xlab, ylab = ylab,
          points_labs = rownames(scores),
          design = design,
          ...
        )
      } else {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes, xlab = xlab,
          points_labs = rownames(scores),
          design = design,
          ...
        )
      }
    } else {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes, ylab = ylab,
          points_labs = rownames(scores),
          design = design,
          ...
        )
      } else {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes,
          points_labs = rownames(scores),
          design = design,
          ...
        )
      }
    }
  } else {
    if (!"xlab" %in% names(mcall)) {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes, xlab = xlab,
          ylab = ylab,
          design = design,
          ...
        )
      } else {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes, xlab = xlab,
          design = design,
          ...
        )
      }
    } else {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes, ylab = ylab,
          design = design,
          ...
        )
      } else {
        fig <- plotScatter(
          Y = scores, title = title,
          xy = axes,
          design = design,
          ...
        )
      }
    }
  }

  # Scores plot  ===================
  fig <- fig + ggplot2::xlim(xlim_val) + ggplot2::ylim(ylim_val)

  return(fig)
}

#' @export lmpContributions
#' @title Summary of the contributions of each effect
#'
#' @description
#' Reports the contribution of each effect to the total variance,
#' but also the contribution of each PC to the total variance per effect.
#' These contributions are also summarized in a barplot.
#'
#'
#' @param resLmpPcaEffects A list corresponding to the output value of \code{\link{lmpPcaEffects}}.
#' @param nPC The number of Principal Components to display.
#'
#' @return A list of:
#' \describe{
#' \item{\code{totalContribTable}}{Table of the percentage of contribution of each effect to the total variance.}
#' \item{\code{effectTable}}{Table of the percentage of variance explained by each principal component in each model effect decomposition.}
#' \item{\code{contribTable}}{Table of the percentage of variance explained by each principal component of each effect reported to the percentage contribution of the given effect to the total variance.}
#' \item{\code{combinedEffectTable}}{Equivalent of the \emph{EffectTable} for combined effects.}
#' \item{\code{plotTotal}}{Plot of the ordered contributions of \emph{TotalContribTable}.}
#' \item{\code{plotContrib}}{Plot of the ordered contributions of \emph{ContribTable}.}
#' }
#'
#' @examples
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resLmpPcaEffects <- lmpPcaEffects(resLmpEffectMatrices, method = "ASCA-E")
#'
#' lmpContributions(resLmpPcaEffects)
#'
#' @import ggplot2

lmpContributions <- function(resLmpPcaEffects, nPC = 5) {
  if (resLmpPcaEffects$method == "APCA") {
    stop("Trying to compute the contributions based on the APCA method.
        The contribution of each principal component to the total variance
        per effect is only meaningful for ASCA and ASCA-E methods since
        the principal components are then derived from the
        pure effect matrices.")
  }
  neffect <- length(resLmpPcaEffects$effectsNamesUnique)

  # Effect table with the total contribution ===============
  total_contrib_table <- matrix(data = NA, nrow = neffect, ncol = 1)
  rownames(total_contrib_table) <- c(
    names(resLmpPcaEffects)[seq_len(neffect - 1)],
    "Residuals"
  )
  colnames(total_contrib_table) <- "Percentage of Variance"

  total_contrib_table[, 1] <- round(resLmpPcaEffects$variationPercentages, 2)

  # Effect table with the variance of each component ===============
  effect_table <- matrix(data = NA, nrow = neffect, ncol = (nPC + 1))
  rownames(effect_table) <- c(
    names(resLmpPcaEffects)[seq_len(neffect - 1)],
    "Residuals"
  )

  # Colnames
  temp_colnames <- vector()
  for (i in seq_len(nPC)) {
    temp_colnames[i] <- paste0("PC", i)
  }
  temp_colnames <- c(temp_colnames, "Sum")
  colnames(effect_table) <- temp_colnames

  # Filling table
  for (i in seq_len(neffect - 1)) {
    effect_table[i, seq_len(nPC)] <- round(resLmpPcaEffects[[i]]$var[seq_len(nPC)], 2)
  }
  effect_table[neffect, seq_len(nPC)] <- round(resLmpPcaEffects[["Residuals"]]$var[seq_len(nPC)], 2)
  effect_table[, nPC + 1] <- c(rep(0, neffect))
  effect_table[, nPC + 1] <- apply(X = effect_table, MARGIN = 1, sum)


  # Effect table with the contribution of each component
  # to the variance of the effect ===============

  contrib_table <- matrix(data = NA, nrow = neffect, (nPC + 1))
  rownames(contrib_table) <- c(
    names(resLmpPcaEffects)[seq_len(neffect - 1)],
    "Residuals"
  )
  temp_colnames <- c(temp_colnames[seq_len(nPC)], "Contrib")
  colnames(contrib_table) <- temp_colnames

  # Filling table
  for (i in seq_len(neffect)) {
    contrib_table[i, seq_len(nPC)] <- (effect_table[i, seq_len(nPC)] *
      resLmpPcaEffects$variationPercentages[i]) / 100
  }

  contrib_table[, (nPC + 1)] <- resLmpPcaEffects$variationPercentages
  contrib_table <- round(contrib_table, 2)


  # Effect table for combined effects ===============
  if (length(resLmpPcaEffects) - 6 != length(resLmpPcaEffects$effectsNamesUnique)) {
    neffectTot <- length(resLmpPcaEffects) - 6
    neffectComb <- neffectTot - neffect

    combinedEffect_table <- matrix(data = NA, nrow = neffectComb, ncol = (nPC + 1))
    rownames(combinedEffect_table) <- names(resLmpPcaEffects)[seq(neffect, (neffectTot - 1))]
    temp_colnames <- c(temp_colnames[seq_len(nPC)], "Sum")
    colnames(combinedEffect_table) <- temp_colnames

    # Filling table
    resCombined <- resLmpPcaEffects[seq(neffect, (neffectTot - 1))]

    for (i in seq_along(resCombined)) {
      combinedEffect_table[i, seq_len(nPC)] <- round(resCombined[[i]]$var[seq_len(nPC)], 2)
    }
    combinedEffect_table[, nPC + 1] <- c(rep(0, neffectComb))
    combinedEffect_table[, nPC + 1] <- apply(X = combinedEffect_table, MARGIN = 1, sum)
  } else {
    combinedEffect_table <- NULL
  }

  # Plots ===============

  # Plot of the total contribution
  effect_name <- ModelAbbrev(c(
    names(resLmpPcaEffects)[seq_len(neffect - 1)],
    "Residuals"
  ))
  dataTotal <- data.frame(
    effects = effect_name,
    varPercentage = unname(resLmpPcaEffects$variationPercentages)
  )

  plotTotal <- with(dataTotal, {
    ggplot2::ggplot(
      data = dataTotal,
      ggplot2::aes(
        x = stats::reorder(effects, -varPercentage),
        y = varPercentage
      )
    )
  }) +
    ggplot2::geom_bar(stat = "identity") +
    ggplot2::xlab("Effects") +
    ggplot2::ylab("Percentage of Variance") +
    ggplot2::theme_bw()

  # Plot of the contribution of each component to the variance of the effect
  tabname <- matrix(data = NA, nrow = neffect, ncol = nPC)

  for (j in seq_len(nPC)) {
    for (i in seq_len(neffect)) {
      tabname[i, j] <- paste(effect_name[i], colnames(contrib_table)[j],
        sep = "\n"
      )
    }
  }

  effect_vector <- as.vector(contrib_table[, seq_len(nPC)])
  names(effect_vector) <- as.vector(tabname)

  dataContrib <- as.data.frame(effect_vector[seq_len(10)])

  plotContrib <- with(dataContrib, {
    ggplot2::ggplot(
      data = dataContrib,
      ggplot2::aes(
        x = stats::reorder(
          rownames(dataContrib),
          -dataContrib[, 1]
        ),
        y = dataContrib[, 1]
      )
    )
  }) +
    ggplot2::geom_bar(stat = "identity") +
    ggplot2::xlab("Contributions") +
    ggplot2::ylab("Percentage of Variance") +
    ggplot2::scale_x_discrete(limits = rownames(dataContrib)) +
    ggplot2::theme_bw()


  # Output

  if (all(is.na(resLmpPcaEffects$type3SS)) &
    all(is.na(resLmpPcaEffects$variationPercentages))) {
    if (is.null(combinedEffect_table)) {
      resLmpContributions <- list(effectTable = effect_table)
    } else {
      resLmpContributions <- list(
        effectTable = effect_table,
        combinedEffectTable = combinedEffect_table
      )
    }
  } else {
    if (is.null(combinedEffect_table)) {
      resLmpContributions <- list(
        totalContribTable = total_contrib_table,
        effectTable = effect_table,
        contribTable = contrib_table,
        plotTotal = plotTotal,
        plotContrib = plotContrib
      )
    } else {
      resLmpContributions <- list(
        totalContribTable = total_contrib_table,
        effectTable = effect_table,
        contribTable = contrib_table,
        combinedEffectTable = combinedEffect_table,
        plotTotal = plotTotal,
        plotContrib = plotContrib
      )
    }
  }

  return(resLmpContributions)
}

#' @export lmpEffectPlot
#' @title Effect plot
#'
#' @description
#' Plots the ASCA scores by effect levels for a given model effect and for one PC at a time.
#' This graph is especially appealing to interpret interactions or combined effects.
#' It is a wrapper of plotMeans.
#'
#' @param resASCA A list corresponding to the ASCA output value of \code{\link{lmpPcaEffects}}.
#' @param effectName Name of the effect to be used to plot the scores.
#' @param axes A numerical vector with the Principal Components axes to be drawn.
#' @param x A character string giving the `design` factor whose levels will form the x axis.
#' @param z A character string giving the `design` factor whose levels will form the traces.
#' @param w A character string giving the `design` factor whose levels will be used for the facet.
#' @param hline If not \code{NULL}, draws one or several horizontal line(s) at values given in `hline`.
#' @param ... Additional arguments to be passed to \code{\link{plotMeans}}.
#'
#' @return An effect plot (ggplot).
#'
#' @details
#' `lmpEffectPlot` is a wrapper of \code{\link{plotMeans}}.
#'
#' @examples
#'
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resASCA <- lmpPcaEffects(
#'   resLmpEffectMatrices = resLmpEffectMatrices,
#'   method = "ASCA", combineEffects = list(c("Hippurate", "Time", "Hippurate:Time"))
#' )
#'
#' # Effect plot for an interaction effect
#' lmpEffectPlot(resASCA, effectName = "Hippurate:Time", x = "Hippurate", z = "Time")
#' # Effect plot for a combined effect
#' lmpEffectPlot(resASCA, effectName = "Hippurate+Time+Hippurate:Time", x = "Hippurate", z = "Time")

#'
#' @import ggplot2

lmpEffectPlot <- function(resASCA, effectName, axes = 1,
                          x, z = NULL, w = NULL, hline = 0, ...) {
  mcall <- as.list(match.call())[-1L]

  # checks =========================

  checkArg(resASCA, c("list"), can.be.null = FALSE)
  checkArg(effectName, c("str", "length1"), can.be.null = FALSE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)
  checkArg(x, c("str", "length1"), can.be.null = FALSE)
  checkArg(z, c("str", "length1"), can.be.null = TRUE)
  checkArg(w, c("str", "length1"), can.be.null = TRUE)
  checkArg(hline, "num", can.be.null = TRUE)

  if (!identical(
    names(resASCA[(length(resASCA) - 7):length(resASCA)]),
    c(
      "Residuals", "lmpDataList", "effectsNamesUnique",
      "effectsNamesUniqueCombined", "method",
      "type3SS", "variationPercentages", "combineEffects"
    )
  )) {
    stop("resLmpPcaEffects is not an output value of lmpPcaEffects")
  }

  if (resASCA$method != "ASCA") {
    stop("resASCA does not correspond to an ASCA result
         of LmpEffectMatrices")
  }

  if (!effectName %in% names(resASCA)) {
    stop(effectName, " is not an effect of resASCA")
  }

  if (max(axes) > ncol(resASCA[[effectName]][["scores"]])) {
    stop(
      "PC (", paste(axes, collapse = ","),
      ") is beyond the number of PC of scores (",
      ncol(resASCA[[effectName]][["scores"]]), ")"
    )
  }

  if (str_detect(string = effectName, pattern = "[+]")) {
    numComb <- str_split(effectName, "[+]")
    numEffects <- unlist(str_split(numComb[[1]], ":"))
  } else {
    numEffects <- unlist(str_split(effectName, ":"))
  }

  if (!x %in% numEffects) {
    stop("x should appear in effectName")
  }
  if (!is.null(z) && !z %in% numEffects) {
    stop("z should appear in effectName")
  }
  if (!is.null(w) && !w %in% numEffects) {
    stop("w should appear in effectName")
  }

  # prepare the arguments  ==============================
  matEffect <- resASCA[[effectName]][["scores"]]

  title <- paste0(effectName, " scores as a function of ", x, ": PC", axes)
  ylab <- paste0(
    "Scores (", round(resASCA[[effectName]][["var"]][axes], 2),
    "% of variation explained)"
  )

  if (!"title" %in% names(mcall)) {
    if (!"ylab" %in% names(mcall)) {
      fig <- plotMeans(
        Y = matEffect,
        design = resASCA$lmpDataList$design,
        cols = axes,
        x = x,
        z = z,
        w = w,
        title = title,
        ylab = ylab,
        hline = hline,
        ...
      )
    } else {
      fig <- plotMeans(
        Y = matEffect,
        design = resASCA$lmpDataList$design,
        cols = axes,
        x = x,
        z = z,
        w = w,
        title = title,
        hline = hline,
        ...
      )
    }
  } else {
    if (!"ylab" %in% names(mcall)) {
      fig <- plotMeans(
        Y = matEffect,
        design = resASCA$lmpDataList$design,
        cols = axes,
        x = x,
        z = z,
        w = w,
        ylab = ylab,
        hline = hline,
        ...
      )
    } else {
      fig <- plotMeans(
        Y = matEffect,
        design = resASCA$lmpDataList$design,
        cols = axes,
        x = x,
        z = z,
        w = w,
        hline = hline,
        ...
      )
    }
  }

  if (length(fig) == 1) {
    fig <- fig[[1]]
  }


  return(fig)
}

# library(devtools)
# load_all()
# data(UCH)
# lmpData = UCH
#
# lmpDataListCheck(lmpData)
#
# lmpData = UCH
# lmpData$design = NULL
# lmpDataListCheck(lmpData)
# lmpDataListCheck(lmpData, null_design = TRUE)
#
# lmpData = UCH
# lmpData$formula = NULL
# lmpDataListCheck(lmpData)
# lmpDataListCheck(lmpData, null_formula = TRUE)
#
# lmpData = UCH
# lmpData$outcomes = NULL
# lmpDataListCheck(lmpData)
# lmpDataListCheck(lmpData, null_outcomes = TRUE)
#
#
# lmpData = UCH
# lmpData$outcomes = lmpData$outcomes[sample(1:nrow(lmpData$outcomes), replace=FALSE),]
# lmpDataListCheck(lmpData)
#
# lmpData = UCH
# lmpData$outcomes = lmpData$outcomes[-1,]
# lmpDataListCheck(lmpData)
#
# lmpData = UCH
# lmpData$design = lmpData$design[-1,]
# lmpDataListCheck(lmpData)
#
# lmpData = UCH
# colnames(lmpData$design)[1] = "aaa"
# lmpDataListCheck(lmpData)
# lmpData$formula=NULL
# lmpDataListCheck(lmpData, null_formula = TRUE)
#
# lmpData = UCH
# rownames(lmpData$design)[1] = "aaa"
# lmpDataListCheck(lmpData)


#' @import dplyr
#' @importFrom stats as.formula

lmpDataListCheck <- function(lmpDataList,
                             null_formula = FALSE,
                             null_design = FALSE,
                             null_outcomes = FALSE) {
  # checks =========================

  # check lmpdata


  checkArg(lmpDataList, "list", can.be.null = FALSE)

  if (length(lmpDataList) > 3) {
    stop("length of lmpDataList is superior to 3, there is unnecessary objects")
  }

  if (!all(names(lmpDataList) %in% c("design", "outcomes", "formula"))) {
    stop("the names of lmpDataList do not correspond to \"design\", \"outcomes\" and \"formula\"")
  }

  out_formula <- lmpDataList$formula
  out_design <- lmpDataList$design
  out_outcomes <- lmpDataList$outcomes

  checkArg(out_formula, c("str", "length1"), can.be.null = null_formula)
  checkArg(out_design, "data.frame", can.be.null = null_design)
  checkArg(out_outcomes, "matrix", can.be.null = null_outcomes)


  # Checking formula ==================================

  if (!is.null(out_formula)) {
    formulaChar <- as.character(stats::as.formula(out_formula))

    if (length(formulaChar) == 3) {
      out_formula <- paste(
        formulaChar[1],
        formulaChar[3]
      )
    } else if (length(formulaChar) == 2) {
      out_formula <- paste(
        formulaChar[1],
        formulaChar[2]
      )
    } else {
      stop("Please put the formula argument in its right form: ~ model terms")
    }
  }


  # Checking outcomes ==================================

  if (!is.null(out_outcomes)) {
    # check if rownames are given for outcomes
    if (is.null(rownames(out_outcomes))) {
      stop("rownames for outcomes is not present and needs to be defined")
    }
  }

  # Checking design ==================================

  # Checking correspondence between formula names and design names ----------

  if (!is.null(out_formula) & !is.null(out_design)) {
    varNames <- all.vars(stats::as.formula(out_formula))
    matchesVarNames <- varNames %in% names(out_design)
    if (!all(matchesVarNames, na.rm = FALSE)) {
      stop(
        "Some of the variable names (", varNames[!matchesVarNames],
        "), present in the formula argument,
           do not correspond to one of the column names of the design argument.
           Please adapt either one of both arguments."
      )
    }
  }

  # Checking correspondence between the rows of design and outcomes ----------

  if (!is.null(out_outcomes) & !is.null(out_design)) {
    if (nrow(out_design) == nrow(out_outcomes)) {
      if (!identical(rownames(out_design), rownames(out_outcomes))) {
        # if same length but not well ordered/named
        if (all(rownames(out_design) %in% rownames(out_outcomes))) {
          warning("reordering the rownames of design to match those of outcomes")
          # reorder the rownames of design to match those of outcomes
          reorder_idx <- match(rownames(out_outcomes), rownames(out_design))
          out_design <- out_design[reorder_idx, ]
          if (!identical(rownames(out_design), rownames(out_outcomes))) {
            # if the reordering fails
            stop("mismatch between the rownames of design and outcomes")
          }
        } else {
          mismatch_names <- rownames(out_design)[!rownames(out_design) %in% rownames(out_outcomes)]
          stop(
            "some rownames of the design (", paste(mismatch_names, collapse = ", "),
            ") do not match the rownames of the outcomes"
          )
        }
      }
    } else {
      stop(
        "nrow of design (", nrow(out_design),
        ") is different from nrow of outcomes (", nrow(out_outcomes), ")"
      )
    }
  }
}

#' @export plotDesign
#' @title Plot of the design matrix
#'
#' @description
#' Provides a graphical representation of the experimental design. It allows to visualize factors levels and check the design balance.
#'
#' @param design A data.frame representing the \eqn{n \times k} "freely encoded" experimental design. Can be `NULL` if `lmpDataList` is defined.
#' @param lmpDataList If not `NULL`, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.
#' @param x By default, the first column of `design` ; otherwise if not `NULL`, a character string giving the column name of `design` to be used for the x-axis. The column needs to be a factor.
#' @param y By default, the second column of `design` if present ; otherwise if not `NULL`, a character string giving the column name of `design` to be used for the y-axis.
#' @param cols By default, the third column of `design` if present ; otherwise if not `NULL`, a character vector with one or several column name(s) of `design` to be used for faceting along the columns. The column needs to be a factor.
#' @param rows By default, the fourth column of `design` if present ; otherwise if not `NULL`, a character vector with one or several column name(s) of `design` to be used for faceting along the rows. The column needs to be a factor.
#' @param title Plot title.
#' @param theme The `ggplot2` theme, see `?ggtheme` for more info.
#'
#' @return A `ggplot2` plot of the design matrix.
#'
#' @details
#'
#' Either \code{design} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{design}.
#' The default behavior (parameters `x`, `y`, `cols` and `rows` are `NULL`) uses the first four columns of `df`. If at least one of these arguments is not `NULL`, the function will only use the non `NULL` parameters to be displayed.
#'
#' @examples
#' ### trout data
#' data(trout)
#'
#' plotDesign(design = trout$design, x = "Day", y = "Treatment")
#'
#' # equivalent to:
#' plotDesign(lmpDataList = trout, x = "Day", y = "Treatment")
#'
#' ### mtcars
#' data(mtcars)
#' library(tidyverse)
#' df <- mtcars %>%
#'   dplyr::select(cyl, vs, am, gear, carb) %>%
#'   as.data.frame() %>%
#'   dplyr::mutate(across(everything(), as.factor))
#'
#' # Default behavior: display the 4 first factors in the design
#' plotDesign(design = df)
#'
#' # 2 factors
#' plotDesign(
#'   design = df, x = "cyl", y = "vs",
#'   cols = NULL, rows = NULL
#' )
#' # 3 factors
#' plotDesign(
#'   design = df, x = "cyl", y = "vs",
#'   cols = NULL, rows = c("am")
#' )
#' # 4 factors
#' plotDesign(
#'   design = df, x = "cyl", y = "vs",
#'   cols = c("gear"), rows = c("am")
#' )
#' # 5 factors
#' plotDesign(
#'   design = df, x = "cyl", y = "vs",
#'   cols = c("gear"), rows = c("am", "carb")
#' )
#'
#' plotDesign(
#'   design = df, x = "cyl", y = "vs",
#'   cols = c("vs"), rows = c("am", "carb")
#' )
#'
#' ### UCH
#' data("UCH")
#' plotDesign(design = UCH$design, x = "Hippurate", y = "Citrate", rows = "Day")
#'
#' @import ggplot2
#' @import dplyr
#' @import tidyverse

plotDesign <- function(design = NULL, lmpDataList = NULL, x = NULL,
                       y = NULL, rows = NULL, cols = NULL,
                       title = "Plot of the design", theme = theme_bw()) {
  # checks arguments ===================

  checkArg(design, "data.frame", can.be.null = TRUE)
  checkArg(x, c("str", "length1"), can.be.null = TRUE)
  checkArg(y, c("str", "length1"), can.be.null = TRUE)
  checkArg(rows, c("str"), can.be.null = TRUE)
  checkArg(cols, c("str"), can.be.null = TRUE)
  checkArg(title, c("str", "length1"), can.be.null = TRUE)


  # define design
  if (is.null(design) & is.null(lmpDataList)) {
    stop("both design and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(design) & !is.null(lmpDataList)) {
    message("both design and lmpDataList are non-NULL, design will be used to perform plotDesign")
  } else if (is.null(design) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    design <- lmpDataList$design
  }

  # initialize x, y, rows, cols

  if (is.null(x) & is.null(y)) xynull <- TRUE else xynull <- FALSE

  if (length(colnames(design)) == 1) {
    design <- cbind(design,
      count = "n"
    )
  }

  if (!is.null(x)) {
    x <- match.arg(x, choices = colnames(design))
  } else {
    x <- colnames(design)[1]
  }

  if (!is.null(y)) {
    y <- match.arg(y, choices = colnames(design))
  } else {
    y <- colnames(design)[2]
  }

  if (!is.null(cols)) {
    if (!all(cols %in% colnames(design))) {
      stop(
        "cols (", paste(cols, collapse = ", "),
        ") is not matching column names of design: ",
        paste(colnames(design), collapse = ", ")
      )
    }
  } else {
    cn <- colnames(design)[3]
    if (xynull & ncol(design) > 2 & !cn %in% c(rows, x, y)) {
      cols <- colnames(design)[3]
    }
  }

  if (!is.null(rows)) {
    if (!all(rows %in% colnames(design))) {
      stop(
        "rows (", paste(rows, collapse = ", "),
        ") is not matching column names of design: ",
        paste(colnames(design), collapse = ", ")
      )
    }
  } else {
    cn <- colnames(design)[4]
    if (xynull & ncol(design) > 3 & !cn %in% c(cols, x, y)) {
      rows <- colnames(design)[4]
    }
  }


  vars <- c(x, rows, cols)
  if (!is.null(vars)) {
    testClass <- mapply(is.factor, design[, vars])
    if (!all(testClass)) {
      warning("at least one variable used as x, rows or cols is not a factor and will be converted as such")
    }
  }


  # Calculate replicates by design point ===================

  df_count <- design %>%
    dplyr::count(dplyr::across(dplyr::all_of(c(x, y, rows, cols))))

  # prepare plot ===================

  p <- ggplot2::ggplot(df_count, ggplot2::aes_string(x = x, y = y))

  if (is.null(rows)) {
    rows <- "."
  } else {
    rows <- paste0(rows, collapse = "+")
  }

  if (is.null(cols)) {
    cols <- "."
  } else {
    cols <- paste0(cols, collapse = "+")
  }

  form <- paste0(rows, "~", cols)

  p <- p + ggplot2::geom_text(aes_string(label = "n"))

  if (is.numeric(df_count[, y])) {
    # message("\n y numeric")
    maxy <- max(df_count[, y])
    miny <- min(df_count[, y])
    delta <- (maxy - miny) / 10
    p <- p + ylim(miny - delta, maxy + delta)
  }

  if (!is.null(rows) | !is.null(cols)) {
    p <- p + ggplot2::facet_grid(as.formula(form),
      labeller = label_both
    )
  }

  p <- p + ggplot2::ggtitle(title) + theme

  return(p)
}

#' @export plotLine
#' @title Line plot
#'
#' @description
#' Generates the response profile of one or more observations i.e. plots of one or more rows of the outcomes matrix on the y-axis against the \eqn{m} response variables on the x-axis. Depending on the response type (spectra, gene expression...), point, line or segment plots can be used.
#'
#' @param Y A numerical matrix containing the rows to be drawn. Can be `NULL` if `lmpDataList` is defined.
#' @param lmpDataList If not `NULL`, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.
#' @param rows A vector with either the row name(s) of the \eqn{Y} matrix to plot (character) or the row index position(s) (integer). Default to 1.
#' @param type Type of graph to be drawn: `"p"` for point, `"l"` for line (default) or `"s"` for segment.
#' @param title Plot title.
#' @param xlab If not \code{NULL}, label for the x-axis.
#' @param ylab If not \code{NULL}, label for the y-axis.
#' @param xaxis_type The data type of the x-axis: either `"numeric"` (default) or `"character"`.
#' @param stacked Logical. If `TRUE`, will draw stacked plots, otherwise will draw separate plots.
#' @param ncol If `stacked` is `FALSE`, the number of columns to represent the separate plots. Default to `1`.
#' @param nrow If `stacked` is `FALSE`, the number of rows to represent the separate plots.
#' @param facet_label If `stacked` is `FALSE`, the labels of the separate plots.
#' @param hline If not \code{NULL}, draws (a) horizontal line(s), by default at y intercept = 0.
#' @param size Argument of length 1 giving the points size (if `type` ==` "p"`) or the line size (if `type` == `"l"` or `"s"`).
#' @param color If not `NULL`, argument of length 1 with possible values: `"rows"`, a color name (character) or a numeric value representing a color.
#' @param shape The points shape (default = `1`) if `type` == `"p"`.
#' @param theme The `ggplot2` theme (default: `theme_bw()`), see `?ggtheme` for more info.
#' @param ang_x_axis If not `NULL`, rotation angle to rotate the x-axis text (based on the argument `axis.text.x` from `ggplot2::theme()`)
#'
#'
#' @return A `ggplot2` line plot.
#'
#' @details
#' Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
#'
#' @examples
#'
#' data("UCH")
#' plotLine(Y = UCH$outcomes)
#'
#' plotLine(lmpDataList = UCH)
#'
#' # separate plots
#' plotLine(Y = UCH$outcomes, rows = seq(1, 8), hline = NULL)
#' plotLine(Y = UCH$outcomes, rows = seq(1, 8), color = 2)
#' plotLine(Y = UCH$outcomes, rows = seq(1, 8), ncol = 2)
#' plotLine(
#'   Y = UCH$outcomes, type = "p",
#'   rows = seq(1, 8), ncol = 2
#' )
#'
#' # stacked plots
#' library(ggplot2)
#' plotLine(
#'   Y = UCH$outcomes, rows = seq(1, 1),
#'   stacked = TRUE, color = "rows"
#' ) +
#'   scale_color_brewer(palette = "Set1")
#'
#' @import tidyr
#' @import ggplot2
#' @import dplyr
#' @import tibble


plotLine <- function(Y = NULL, lmpDataList = NULL, rows = 1, type = c("l", "p", "s"),
                     title = "Line plot", xlab = NULL, ylab = NULL,
                     xaxis_type = c("numeric", "character"),
                     stacked = FALSE, ncol = 1, nrow = NULL,
                     facet_label = NULL, hline = 0, size = 0.5, color = NULL,
                     shape = 1, theme = theme_bw(),
                     ang_x_axis = NULL) {
  # checks =========================


  checkArg(Y, "matrix", can.be.null = TRUE)
  checkArg(title, c("str", "length1"), can.be.null = TRUE)
  checkArg(xlab, c("str", "length1"), can.be.null = TRUE)
  checkArg(ylab, c("str", "length1"), can.be.null = TRUE)
  if (!is.null(ncol)) {
    checkArg(ncol, c("int", "length1"), can.be.null = FALSE)
  }
  if (!is.null(nrow)) {
    checkArg(nrow, c("int", "length1"), can.be.null = FALSE)
  }
  checkArg(hline, "num", can.be.null = TRUE)
  checkArg(size, "num", can.be.null = FALSE)
  checkArg(color, "length1", can.be.null = TRUE)
  checkArg(shape, "length1", can.be.null = FALSE)
  checkArg(stacked, "bool", can.be.null = FALSE)
  checkArg(facet_label, c("str"), can.be.null = TRUE)
  checkArg(ang_x_axis, "int", can.be.null = TRUE)

  type <- match.arg(type)
  xaxis_type <- match.arg(xaxis_type)

  if (is.numeric(rows)) {
    checkArg(rows, c("pos", "int"), can.be.null = FALSE)
  }

  if (!is.numeric(rows) & !is.character(rows)) {
    stop("rows is neither numeric or character")
  }

  if (!is.null(facet_label) && length(rows) != length(facet_label)) {
    stop("the number of facet_label differs
                from the number of rows")
  }


  if (is.null(Y) & is.null(lmpDataList)) {
    stop("both Y and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(Y) & !is.null(lmpDataList)) {
    message("both Y and lmpDataList are non-NULL, Y will be used to perform plotLine")
  } else if (is.null(Y) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList,
      null_formula = TRUE,
      null_design = TRUE
    )
    Y <- lmpDataList$outcomes
  }

  # prepare the arguments  ==============================

  if (is.null(rownames(Y))) {
    rownames(Y) <- rownames(Y, do.NULL = FALSE, prefix = "X.")
  }

  if (is.numeric(rows)) {
    rows <- rownames(Y)[rows]
  }

  Y <- t(Y[rows, , drop = FALSE])

  # mn_xy <- make.names(rows) # corrects the naming of variables
  # colnames(Y) <- mn_xy

  Y <- Y %>%
    as.data.frame() %>%
    tibble::rownames_to_column(var = "x_axis")

  Y_long <- Y %>% tidyr::pivot_longer(all_of(rows), names_to = "rownames")

  if (xaxis_type == "numeric") {
    rn <- Y_long$x_axis
    rn <- as.numeric(rn)
    if (sum(is.na(rn)) > 0) {
      stop("cannot convert column names to numerical,
           change xaxis_type from numeric to character")
    } else {
      Y_long$x_axis <- rn
    }
  }

  if (!is.null(color)) {
    if (color == "rows") {
      fig <- with(Y_long, {
        ggplot2::ggplot(
          data = Y_long,
          ggplot2::aes(
            x = x_axis, y = value,
            group = rownames, color = rownames
          )
        )
      })
      color <- NULL
    } else {
      fig <- with(Y_long, {
        ggplot2::ggplot(
          data = Y_long,
          ggplot2::aes(
            x = x_axis, y = value,
            group = rownames
          )
        )
      })
    }
  } else {
    fig <- with(Y_long, {
      ggplot2::ggplot(
        data = Y_long,
        ggplot2::aes(
          x = x_axis, y = value,
          group = rownames
        )
      )
    })
  }


  fig <- fig + ggplot2::ggtitle(title) +
    ggplot2::xlab(xlab) + ggplot2::ylab(ylab) +
    theme

  # revert x axis if necessary
  if (xaxis_type == "numeric") {
    if ((Y_long$x_axis[1] - Y_long$x_axis[nrow(Y_long)]) > 0) {
      fig <- fig + ggplot2::scale_x_reverse()
    }
  }

  if (!is.null(hline)) {
    fig <- fig + ggplot2::geom_hline(
      yintercept = hline,
      size = 0.5, linetype = "dashed",
      colour = "gray60"
    )
  }

  if (type == "l") {
    if (!is.null(color)) {
      fig <- fig + ggplot2::geom_line(
        color = color,
        size = size
      )
    } else {
      fig <- fig + ggplot2::geom_line(size = size)
    }
  }



  if (type == "p") {
    if (!is.null(color)) {
      fig <- fig + ggplot2::geom_point(
        size = size,
        shape = shape, color = color
      )
    } else {
      fig <- fig + ggplot2::geom_point(
        size = size,
        shape = shape
      )
    }
  }

  if (type == "s") {
    if (!is.null(color)) {
      fig <- with(Y_long, {
        fig +
          ggplot2::geom_segment(aes(xend = x_axis, yend = 0),
            size = size, lineend = "round", color = color
          )
      })
    } else {
      fig <- with(Y_long, {
        fig +
          ggplot2::geom_segment(aes(xend = x_axis, yend = 0),
            size = size, lineend = "round"
          )
      })
    }
  }


  if (!stacked) {
    # facet_wrap
    if (is.null(facet_label)) {
      fig <- fig +
        ggplot2::facet_wrap(~rownames, ncol = ncol, nrow = nrow)
    } else {
      names(facet_label) <- rows
      fig <- fig +
        ggplot2::facet_wrap(~rownames,
          ncol = ncol, nrow = nrow,
          labeller = labeller(rownames = facet_label)
        )
    }
  }

  if (!is.null(ang_x_axis)) {
    fig <- fig +
      ggplot2::theme(axis.text.x = element_text(
        angle = ang_x_axis,
        vjust = 0.5, hjust = 1
      ))
  }

  return(fig)
}

#' @export plotScatter
#' @title Scatter plot
#'
#' @description
#' Produces a plot describing the relationship between two columns of the outcomes matrix \eqn{Y}. Colors and symbols can be chosen for the levels of the design factors. Ellipses, polygons or segments can be added to group different sets of points on the graph.
#'
#' @param Y A \eqn{n \times m} matrix with \eqn{n} observations and \eqn{m} variables. Can be `NULL` if `lmpDataList` is defined.
#' @param design A \eqn{n \times k} "freely encoded" experimental design data.frame. Can be `NULL` if `lmpDataList` is defined.
#' @param lmpDataList If not `NULL`, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.
#' @param xy x- and y-axis values: a vector of length 2 with either the column name(s) of the \eqn{Y} matrix to plot (character) or the index position(s) (integer).
#' @param color If not \code{NULL}, a character string giving the column name of `design` to be used as color. Currently treated as a discrete variable.
#' @param shape If not \code{NULL}, a character string giving the column name of `design` to be used as shape. Currently treated as a discrete variable.
#' @param points_labs If not \code{NULL}, a character vector with point labels.
#' @param title Plot title.
#' @param xlab If not \code{NULL}, label for the x-axis.
#' @param ylab If not \code{NULL}, label for the y-axis.
#' @param size The points size, by default `2`.
#' @param size_lab The size of points labels, by default `3`.
#' @param drawShapes Multiple shapes can be drawn based on the `color`: `"none"` for no shape (default), `"ellipse"` (ellipses with `ggplot2::stat_ellipse()`), `"polygon"` (polygons with `ggplot2::geom_polygon()`) or `"segment"` (segment from the centroids with `ggplot2::geom_segment()`).
#' @param typeEl The type of ellipse, either `"norm"` (multivariate normal distribution, the default), `"t"` (multivariate t-distribution) or `"euclid"` (draws a circle with the radius equal to level, representing the euclidean distance from the center).
#' @param levelEl The confidence level at which to draw an ellipse, by default `0.9`.
#' @param alphaPoly The degree of transparency for polygons, by default `0.4`.
#' @param theme The `ggplot2` theme (default: `theme_bw()`), see `?ggtheme` for more info.
#' @param drawOrigin If \code{TRUE}, draws horizontal and vertical intercepts at (0,0).
#'
#' @return A `ggplot2` scatter plot.
#'
#' @details
#' Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
#' The same rule applies for \code{design} or \code{lmpDataList}.
#'
#' @examples
#'
#' data("UCH")
#'
#' # Without the design info
#' plotScatter(Y = UCH$outcomes, xy = c(453, 369))
#'
#' # equivalent to:
#' plotScatter(lmpDataList = UCH, xy = c(453, 369))
#'
#' # With color and shape
#' plotScatter(
#'   lmpDataList = UCH,
#'   xy = c(453, 369), color = "Hippurate",
#'   shape = "Citrate"
#' )
#'
#' # equivalent to:
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), color = "Hippurate",
#'   shape = "Citrate"
#' )
#'
#' # With color and shapes
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), color = "Hippurate",
#'   drawShapes = "ellipse"
#' )
#'
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), color = "Hippurate",
#'   drawShapes = "polygon"
#' )
#'
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), color = "Hippurate",
#'   drawShapes = "segment"
#' )
#'
#' # With customized shapes
#' library(ggplot2)
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), shape = "Hippurate", size = 3
#' ) +
#'   scale_discrete_identity(
#'     aesthetics = "shape",
#'     guide = "legend"
#'   )
#'
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), shape = "Hippurate"
#' ) +
#'   scale_shape_discrete(solid = FALSE)
#'
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), shape = "Hippurate"
#' ) +
#'   scale_shape_manual(values = c(15, 16, 17))
#'
#' # With labels
#' plotScatter(
#'   Y = UCH$outcomes, design = UCH$design,
#'   xy = c(453, 369), points_labs = rownames(UCH$design)
#' )
#'
#' @import ggplot2
#' @import ggrepel
#' @import dplyr
#' @importFrom plyr ddply

plotScatter <- function(Y = NULL, design = NULL, lmpDataList = NULL,
                        xy, color = NULL, shape = NULL,
                        points_labs = NULL, title = "Scatter plot",
                        xlab = NULL, ylab = NULL, size = 2, size_lab = 3,
                        drawShapes = c(
                          "none", "ellipse",
                          "polygon", "segment"
                        ),
                        typeEl = c("norm", "t", "euclid"),
                        levelEl = 0.9, alphaPoly = 0.4,
                        theme = theme_bw(), drawOrigin = FALSE) {
  # checks ==============================
  checkArg(Y, "matrix", can.be.null = TRUE)
  checkArg(design, "data.frame", can.be.null = TRUE)
  checkArg(color, c("str", "length1"), can.be.null = TRUE)
  checkArg(shape, c("str", "length1"), can.be.null = TRUE)
  checkArg(points_labs, "str", can.be.null = TRUE)
  checkArg(title, c("str", "length1"), can.be.null = TRUE)
  checkArg(xlab, c("str", "length1"), can.be.null = TRUE)
  checkArg(ylab, c("str", "length1"), can.be.null = TRUE)
  checkArg(size, c("num", "pos", "length1"), can.be.null = FALSE)
  checkArg(size_lab, c("num", "pos", "length1"), can.be.null = FALSE)
  checkArg(levelEl, c("num", "pos", "length1"), can.be.null = FALSE)
  checkArg(alphaPoly, c("num", "pos", "length1"), can.be.null = FALSE)

  # define Y
  if (is.null(Y) & is.null(lmpDataList)) {
    stop("both Y and lmpDataList are NULL, at least one should be non NULL.")
  } else if (!is.null(Y) & !is.null(lmpDataList)) {
    message("both Y and lmpDataList are non-NULL, Y will be used to perform plotScatter")
  } else if (is.null(Y) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    Y <- lmpDataList$outcomes
  }

  # define design
  if (!is.null(design) & !is.null(lmpDataList)) {
    message("both design and lmpDataList are non-NULL, design will be used to perform plotScatter")
  } else if (is.null(design) & !is.null(lmpDataList)) {
    lmpDataListCheck(lmpDataList, null_formula = TRUE)
    design <- lmpDataList$design
  }


  typeEl <- match.arg(typeEl)
  drawShapes <- match.arg(drawShapes)

  if (drawShapes != "none") {
    if (is.null(color)) {
      stop(
        "drawShapes is set to ", drawShapes,
        " but color is NULL"
      )
    }
  }


  if (!is.null(design)) {
    match.arg(color, choices = c(colnames(design), NULL))
    match.arg(shape, choices = c(colnames(design), NULL))
  }

  if (length(color) == 1 | length(shape) == 1) {
    if (is.null(design)) {
      stop("color or shape is specified but design is NULL")
    }
  }

  vars <- c(color, shape)
  if (!is.null(vars)) {
    testClass <- mapply(is.factor, design[, vars])
    if (!all(testClass)) {
      warning("at least one variable used as color or shape is not a factor and will be converted as such")
    }
  }


  if (length(xy) != 2) {
    stop("xy is not of length 2")
  }

  if (is.numeric(xy)) {
    checkArg(xy, c("pos", "int"), can.be.null = FALSE)
  }


  if (!is.numeric(xy) & !is.character(xy)) {
    stop("xy is neither numeric or character")
  }

  if (!is.null(design)) {
    if (length(points_labs) != nrow(design) & !is.null(points_labs)) {
      stop("length of points_labs is different than the
           number of observations")
    }
  }




  # prepare the arguments  ==============================
  if (is.numeric(xy)) {
    if (is.null(colnames(Y)[xy])) {
      colnames(Y)[xy] <- xy <- paste0("C", xy)
    } else {
      xy <- colnames(Y)[xy]
    }
  } else {
    if (!xy[1] %in% colnames(Y) | !xy[2] %in% colnames(Y)) {
      stop("values in xy do not match the colnames of Y")
    }
  }

  mn_xy <- make.names(xy) # corrects the naming of variables

  if (is.null(xlab)) {
    xlab <- xy[1]
  }
  if (is.null(ylab)) {
    ylab <- xy[2]
  }

  out_df <- Y[, c(xy)]
  colnames(out_df) <- mn_xy

  if (!is.null(design)) {
    design_df <- design[, c(color, shape), drop = FALSE]
    design_df <- design_df %>% mutate_all(as.factor)
    df_tot <- cbind(out_df, design_df)
  } else {
    df_tot <- data.frame(out_df)
  }

  df_tot$points_labs <- points_labs

  # ScatterPlot  ==============================

  # ggplot ++++
  fig <- ggplot2::ggplot(
    data = df_tot,
    ggplot2::aes_string(
      x = mn_xy[1], y = mn_xy[2],
      label = "points_labs",
      color = color,
      shape = shape
    )
  )
  if (drawOrigin) {
    fig <- fig +
      geom_hline(yintercept = 0, linetype = 2, color = "gray70") +
      geom_vline(xintercept = 0, linetype = 2, color = "gray80")
  }

  fig <- fig +
    ggplot2::geom_point(size = size) +
    labs(x = xlab, y = ylab, title = title) +
    theme

  # points_labs ++++
  if (!is.null(points_labs)) {
    fig <- fig +
      ggrepel::geom_text_repel(
        show.legend = FALSE,
        size = size_lab
      )
  }

  # drawShapes   ++++

  if (drawShapes != "none") {
    if (drawShapes == "ellipse") {
      fig <- fig + ggplot2::stat_ellipse(
        type = typeEl,
        level = levelEl
      )
    } else if (drawShapes == "polygon") {
      # getting the convex hull of each unique point set
      find_hull <- function(df) df[chull(df[, mn_xy[1]], df[, mn_xy[2]]), ]
      hulls <- plyr::ddply(df_tot, unique(c(color, shape)), find_hull)

      fig <- fig + ggplot2::geom_polygon(
        data = hulls,
        aes_string(fill = color, color = color),
        alpha = alphaPoly
      )
    } else if (drawShapes == "segment") {
      centroids <- df_tot %>%
        dplyr::group_by(dplyr::across(dplyr::all_of(unique(c(color, shape))))) %>%
        dplyr::summarise(across(dplyr::all_of(mn_xy), mean))

      df_tot_centr <- df_tot %>% dplyr::left_join(centroids,
        by = unique(c(color, shape)),
        suffix = c("", ".centroid")
      )

      centr_name <- paste0(mn_xy, ".centroid")
      fig <- fig +
        ggplot2::geom_segment(
          data = df_tot_centr,
          aes_string(
            x = centr_name[1], y = centr_name[2],
            xend = mn_xy[1], yend = mn_xy[2],
            color = color
          )
        )
    }
  }



  return(fig)
}

#' @export lmpEffectMatrices
#' @title Computes the effect matrices
#'
#' @description
#' Estimates the model by OLS based on the outcomes and model matrices provided in the outputs of lmpModelMatrix function and calculates the estimated effect matrices
#' \eqn{\hat{\mathbf{M}}_0, \hat{\mathbf{M}}_1, ...\hat{\mathbf{M}}_F}, ...  and residual matrix \eqn{\hat{\mathbf{E}}}.
#' It calculates also the type III percentage of variance explained by each effect.
#'
#' @param resLmpModelMatrix A list of 5 elements from \code{\link{lmpModelMatrix}}.
#' @param SS Logical. If `FALSE`, won't compute the percentage of variance for each effect.
#' @param contrastList A list of contrasts for each parameter. If `NA`, the function creates automatically the list by default.
#'
#' @return A list with the following elements:
#'  \describe{
#'    \item{\code{lmpDataList}}{The initial object: a list with outcomes, design and formula.}
#'    \item{\code{modelMatrix}}{A \emph{nxp} model matrix specifically encoded for the ASCA-GLM method.}
#'    \item{\code{modelMatrixByEffect}}{A list of \emph{F+1} model matrices for each effect.}
#'    \item{\code{effectsNamesUnique}}{A character vector with the \emph{F+1} names of the model effects, each repeated once.}
#'    \item{\code{effectsNamesAll}}{A character vector with the \emph{p} names of the model effects ordered and repeated as the column names of the model matrix.}
#'    \item{\code{effectMatrices}}{A list of \emph{F+1} effect matrices for each model effect.}
#'    \item{\code{predictedvalues}}{The \emph{nxm} matrix of predicted outcome values.}
#'    \item{\code{residuals}}{The \emph{nxm} matrix of model residuals.}
#'    \item{\code{parameters}}{The \emph{pxm} matrix of the estimated parameters.}
#'    \item{\code{type3SS}}{A vector with the type III sum of squares for each model effect \emph{(If SS = TRUE)}.}
#'    \item{\code{variationPercentages}}{A vector with the percentage of variance for each model effect \emph{(If SS = TRUE)}.}
#'    \item{\code{varPercentagesPlot}}{A ggplot bar plot of the contributions of each model effect to the total variance \emph{(If SS = TRUE)}.}
#'  }
#'
#' @examples
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' reslmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' reslmpEffectMatrices$varPercentagesPlot
#'
#' @references Thiel M.,Feraud B. and Govaerts B. (2017) \emph{ASCA+ and APCA+: Extensions of ASCA and APCA
#' in the analysis of unbalanced multifactorial designs}, Journal of Chemometrics
#'
#' @import stringr
#' @importFrom plyr laply aaply



lmpEffectMatrices <- function(resLmpModelMatrix, SS = TRUE, contrastList = NA) {
  # Checking the object
  if (!is.list(resLmpModelMatrix)) {
    stop("Argument resLmpModelMatrix is not a list")
  }
  if (length(resLmpModelMatrix) != 5) {
    stop("List does not contain 5 elements")
  }
  if (names(resLmpModelMatrix)[1] != "lmpDataList" |
    names(resLmpModelMatrix)[2] != "modelMatrix" |
    names(resLmpModelMatrix)[3] != "modelMatrixByEffect" |
    names(resLmpModelMatrix)[4] != "effectsNamesUnique" |
    names(resLmpModelMatrix)[5] != "effectsNamesAll") {
    stop("Argument is not a resLmpModelMatrix object")
  }
  checkArg(SS, c("bool", "length1"), can.be.null = FALSE)

  # Attribute a name in the function environment
  formula <- resLmpModelMatrix$lmpDataList$formula
  design <- resLmpModelMatrix$lmpDataList$design
  outcomes <- resLmpModelMatrix$lmpDataList$outcomes
  lmpDataList <- resLmpModelMatrix$lmpDataList
  modelMatrix <- resLmpModelMatrix$modelMatrix
  modelMatrixByEffect <- resLmpModelMatrix$modelMatrixByEffect
  effectsNamesAll <- resLmpModelMatrix$effectsNamesAll
  effectsNamesUnique <- resLmpModelMatrix$effectsNamesUnique
  nEffect <- length(effectsNamesUnique)

  # Creating empty effects matrices
  effectMatrices <- list()
  length(effectMatrices) <- nEffect
  names(effectMatrices) <- effectsNamesUnique

  # GLM decomposition calculated by using glm.fit and alply on outcomes

  # The following line gives an error of type: Error in glm.fit(modelMatrix, xx) : NAs in V(mu),
  # it is temporarily replaced by a loop
  # resGLM <- plyr::alply(outcomes, 2, function(xx) glm.fit(modelMatrix, xx))

  resGLM <- list()
  for (i in seq_len(ncol(outcomes))) resGLM[[i]] <- stats::glm.fit(modelMatrix, outcomes[, i])
  parameters <- t(plyr::laply(resGLM, function(xx) xx$coefficients))
  predictedValues <- t(plyr::laply(resGLM, function(xx) xx$fitted.values))
  residuals <- t(plyr::laply(resGLM, function(xx) xx$residuals))
  colnames(residuals) <- colnames(predictedValues) <- colnames(outcomes)

  # Filling effectMatrices
  for (iEffect in seq_len(nEffect)) {
    selection <- which(effectsNamesAll == effectsNamesUnique[iEffect])
    selectionComplement <- which(effectsNamesAll != effectsNamesUnique[iEffect])
    # Effect matrices
    effectMatrices[[iEffect]] <- t(plyr::aaply(
      parameters, 2,
      function(xx) {
        as.matrix(modelMatrix[, selection]) %*%
          xx[selection]
      }
    ))
    colnames(effectMatrices[[iEffect]]) <- colnames(outcomes)
  }


  resLmpEffectMatrices <- list(
    lmpDataList = lmpDataList,
    modelMatrix = modelMatrix,
    modelMatrixByEffect = modelMatrixByEffect,
    effectsNamesUnique = effectsNamesUnique,
    effectsNamesAll = effectsNamesAll,
    effectMatrices = effectMatrices,
    predictedvalues = predictedValues,
    residuals = residuals,
    parameters = parameters
  )

  # Compute the Sum of Squares Type 3
  if (SS == TRUE) {
    if (is.na(contrastList)) {
      L <- contrastSS(resLmpModelMatrix)
    } else {
      L <- contrastList
    }
    resLmpSS <- lmpSS(resLmpEffectMatrices, L)

    # Plot of the total contribution
    contrib <- as.data.frame(resLmpSS$variationPercentages)
    rownames(contrib) <- ModelAbbrev(rownames(contrib))

    plot <- ggplot2::ggplot(
      data = contrib,
      ggplot2::aes(
        x = stats::reorder(
          rownames(contrib),
          -contrib[, 1]
        ),
        y = contrib[, 1]
      )
    ) +
      ggplot2::geom_bar(stat = "identity") +
      ggplot2::xlab("Effects") +
      ggplot2::ylab("Percentage of Variance") +
      ggplot2::theme_bw()

    resLmpEffectMatrices <- c(resLmpEffectMatrices, resLmpSS,
      varPercentagesPlot = list(plot)
    )
  } else {
    resLmpEffectMatrices <- c(resLmpEffectMatrices,
      type3SS = NA,
      variationPercentages = NA, varPercentagesPlot = NA
    )
  }

  return(resLmpEffectMatrices)
}

set.seed(1)
design <- data.frame(
    factA = factor(c(rep(1, 5), rep(2, 5))),
    factB = factor(rep(c(1, 2), 5))
)
outcomes <- matrix(rnorm(30), ncol = 3)
formula <- "~ factA+factB"
lmpDL <- list(design = design, outcomes = outcomes, formula = formula)
mM <- model.matrix(as.formula(lmpDL$formula), lmpDL$design)
mMBE <- list(
    Intercept = mM[, "(Intercept)", drop = FALSE],
    factA = mM[, "factA2", drop = FALSE],
    factB = mM[, "factB2", drop = FALSE]
)
eNU <- eNA <- c("Intercept", "factA", "factB")


resLmpModelMatrix <- list(
    lmpDataList = lmpDL,
    modelMatrix = mM,
    modelMatrixByEffect = mMBE,
    effectsNamesUnique = eNU,
    effectsNamesAll = eNA
)

res_EM <- lmpEffectMatrices(resLmpModelMatrix)


test_that("lmpEffectMatrices runs correctly with defaults", {
    expect_type(res_EM, "list")
    expect_length(res_EM, 12)
    expect_equal(res_EM$effectMatrices$Intercept, matrix(c(
        rep(0.008029017, 10),
        rep(0.3359562, 10),
        rep(0.3200447, 10)
    ), ncol = 3),
    tolerance = 1e-06, ignore_attr = "dimnames"
    )
    expect_equal(res_EM$predictedvalues[1, 1], 0.008029017, ignore_attr = "names")
    expect_equal(res_EM$parameters[2, 1], c(factA2 = -0.054754666))
    expect_equal(res_EM$type3SS["factA"], c(factA = 1.0191787))
    expect_equal(res_EM$variationPercentages["factA"],
        c(factA = 3.877278),
        tolerance = 1e-06
    )
})

#' @export pcaScreePlot
#' @title Scree Plot
#'
#' @description
#' Returns a bar plot of the percentage of variance explained by each Principal Component calculated by \code{\link{pcaBySvd}}.
#'
#' @param resPcaBySvd A list corresponding to the output value of \code{\link{pcaBySvd}}.
#' @param nPC An integer with the number of Principal Components to plot.
#' @param title Plot title.
#' @param theme `ggplot2` theme, see `?ggtheme` for more info.
#'
#' @return A `ggplot2` PCA scree plot
#'
#' @examples
#'
#' data("UCH")
#' resPCA <- pcaBySvd(UCH$outcomes)
#' pcaScreePlot(resPCA, nPC = 4)
#'
#' @import ggplot2

pcaScreePlot <- function(resPcaBySvd, nPC = 5,
                         title = "PCA scree plot",
                         theme = theme_bw()) {
  # checks ===================
  checkArg(resPcaBySvd, c("list"), can.be.null = FALSE)
  checkArg(nPC, c("num", "pos", "length1"), can.be.null = FALSE)
  checkArg(title, c("str", "length1"), can.be.null = FALSE)

  if (!identical(names(resPcaBySvd), c(
    "scores", "loadings", "eigval", "singvar",
    "var", "cumvar", "original.dataset", "design"
  ))) {
    stop("resPcaBySvd is not an output value of pcaBySvd")
  }

  # plot ===================
  res <- data.frame(var = resPcaBySvd$var[seq_len(nPC)])
  res <- res %>% rownames_to_column("PC")

  with(res, {
    ggplot2::ggplot(
      data = res,
      ggplot2::aes(
        x = factor(PC, levels = PC),
        y = var
      )
    ) +
      ggplot2::geom_bar(stat = "identity") +
      ggplot2::xlab("Principal Components") +
      ggplot2::ylab("Percentage of Variance") +
      ggplot2::ggtitle(title) +
      theme
  })
}

#' @export pcaLoading2dPlot
#' @title Loading plots on a 2D scatter plot
#'
#' @description
#' Produces 2D loading plots from \code{\link{pcaBySvd}} with the same graphical options as \code{\link{plotScatter}} as this is a wrapper of this function.
#'
#' @param resPcaBySvd A list corresponding to the output value of \code{\link{pcaBySvd}}.
#' @param axes A numerical vector of length 2 with the Principal Components axes to be drawn.
#' @param title Plot title.
#' @param addRownames Boolean indicating if the labels should be plotted. By default, uses the row names of the loadings matrix but it can be manually specified with the `points_labs` argument from \code{\link{plotScatter}}.
#' @param pl_n The number of labels that should be plotted, based on a distance measure (see Details).
#' @param metadata A \eqn{n \times k} "freely encoded" data.frame corresponding to the `design` argument in \code{\link{plotScatter}}.
#' @param drawOrigin if \code{TRUE}, draws horizontal and vertical intercepts at (0,0) based on the \code{\link{plotScatter}} function.
#' @param ... Additional arguments to be passed to \code{\link{plotScatter}}.
#'
#' @return A `ggplot2` object with the PCA loading plot.
#'
#' @details
#' `pcaLoading2dPlot` is a wrapper of \code{\link{plotScatter}}. See `?plotScatter` for more information on the additional arguments.
#'
#'
#' The distance measure \eqn{d}{d} that is used to rank the variables is based on the following formula:
#' \deqn{d = \sqrt(P_{ab}^2*\lambda_{ab}^2)}{d = sqrt(P_ab^2 * lambda_ab^2)} where \eqn{a}{a}
#' and \eqn{b}{b} are two selected Principal Components, \eqn{P_{ab}}{P_ab} represents their
#' loadings and \eqn{\lambda_{ab}}{lambda_ab} their singular values.
#'
#' @examples
#'
#' data("UCH")
#' ResPCA <- pcaBySvd(UCH$outcomes)
#'
#' pcaLoading2dPlot(
#'   resPcaBySvd = ResPCA, axes = c(1, 2),
#'   title = "PCA loading plot UCH"
#' )
#'
#' # adding color,  shape and labels to points
#' id_cit <- seq(446, 459)
#' id_hip <- c(seq(126, 156), seq(362, 375))
#' peaks <- rep("other", ncol(UCH$outcomes))
#' peaks[id_hip] <- "hip"
#' peaks[id_cit] <- "cit"
#' metadata <- data.frame(peaks)
#'
#' pcaLoading2dPlot(
#'   resPcaBySvd = ResPCA, axes = c(1, 2),
#'   title = "PCA loading plot UCH", metadata = metadata,
#'   color = "peaks", shape = "peaks", addRownames = TRUE
#' )
#'
#' # changing max.overlaps of ggrepel
#' options(ggrepel.max.overlaps = 30)
#' pcaLoading2dPlot(
#'   resPcaBySvd = ResPCA, axes = c(1, 2),
#'   title = "PCA loading plot UCH", metadata = metadata,
#'   color = "peaks", shape = "peaks", addRownames = TRUE,
#'   pl_n = 35
#' )
#'
#' @import ggplot2

pcaLoading2dPlot <- function(resPcaBySvd,
                             axes = c(1, 2),
                             title = "PCA loading plot",
                             addRownames = FALSE,
                             pl_n = 10,
                             metadata = NULL,
                             drawOrigin = TRUE,
                             ...) {
  mcall <- as.list(match.call())[-1L]

  # checks ===================
  checkArg(resPcaBySvd, c("list"), can.be.null = FALSE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)
  checkArg(title, c("str", "length1"), can.be.null = FALSE)
  checkArg(addRownames, c("bool"), can.be.null = FALSE)
  checkArg(pl_n, c("int", "pos", "length1"), can.be.null = FALSE)
  checkArg(metadata, "data.frame", can.be.null = TRUE)

  if (!identical(names(resPcaBySvd), c(
    "scores", "loadings", "eigval", "singvar",
    "var", "cumvar", "original.dataset", "design"
  ))) {
    stop("resPcaBySvd is not an output value of pcaBySvd")
  }

  # loadings   ===================
  loadings <- resPcaBySvd$loadings
  checkArg(loadings, c("matrix"), can.be.null = FALSE)

  if (length(axes) != 2) {
    stop("axes is not of length 2")
  }

  if (max(axes) > ncol(loadings)) {
    stop(
      "axes (", paste(axes, collapse = ","),
      ") is beyond the ncol of loadings (", ncol(loadings), ")"
    )
  }


  # percentage of explained variance   ===================
  pc_var <- resPcaBySvd$var
  pc_var_x <- format(pc_var[pc_var >= 0.1], digits = 2, trim = TRUE)
  pc_var_y <- format(pc_var[pc_var < 0.1],
    digits = 2,
    scientific = TRUE, trim = TRUE
  )
  pc_var_char <- as.character(pc_var)
  pc_var_char[pc_var >= 0.1] <- pc_var_x
  pc_var_char[pc_var < 0.1] <- pc_var_y

  pc_var_char <- paste0("PC", axes, " (", pc_var_char[axes], "%)")

  # distance measure and labels  ===================

  load <- resPcaBySvd$loadings[, axes]
  singvar <- resPcaBySvd$singvar[axes]

  dista <- load^2 %*% singvar^2

  if (methods::hasArg("points_labs")) {
    addRownames <- FALSE
  } else {
    pl_n <- min(ncol(loadings), pl_n)
    points_labels <- rownames(loadings)
    ids <- order(dista, decreasing = TRUE)[seq_len(pl_n)]
    points_labels[-ids] <- ""
  }

  # graphical parameters   ===================
  xlab <- pc_var_char[1]
  ylab <- pc_var_char[2]

  xlim1 <- max(abs(loadings[, axes[1]]))
  xlim_val <- c(-xlim1, xlim1)

  ylim1 <- max(abs(loadings[, axes[2]]))
  ylim_val <- c(-ylim1, ylim1)

  if (addRownames) {
    if (!"xlab" %in% names(mcall)) {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes, xlab = xlab, ylab = ylab,
          points_labs = points_labels,
          design = metadata,
          drawOrigin = drawOrigin,
          ...
        )
      } else {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes, xlab = xlab,
          points_labs = points_labels,
          design = metadata,
          drawOrigin = drawOrigin,
          ...
        )
      }
    } else {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes, ylab = ylab,
          points_labs = points_labels,
          design = metadata,
          drawOrigin = drawOrigin,
          ...
        )
      } else {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes,
          points_labs = points_labels,
          design = metadata,
          drawOrigin = drawOrigin,
          ...
        )
      }
    }
  } else {
    if (!"xlab" %in% names(mcall)) {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes, xlab = xlab, ylab = ylab,
          design = metadata,
          drawOrigin = drawOrigin, ...
        )
      } else {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes, xlab = xlab,
          design = metadata,
          drawOrigin = drawOrigin, ...
        )
      }
    } else {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes, ylab = ylab,
          design = metadata,
          drawOrigin = drawOrigin, ...
        )
      } else {
        fig <- plotScatter(
          Y = loadings, title = title,
          xy = axes,
          design = metadata,
          drawOrigin = drawOrigin, ...
        )
      }
    }
  }

  # loadings plot  ===================
  fig <- fig + ggplot2::xlim(xlim_val) + ggplot2::ylim(ylim_val)

  # if (length(fig) == 1) {
  #   fig <- fig[[1]]
  # }

  return(fig)
}

#' @export pcaLoading1dPlot
#' @title Loadings represented on a line plot.
#'
#' @description
#' Plots the loading vectors from \code{\link{pcaBySvd}} output with different available line types.
#'
#' @param resPcaBySvd A list corresponding to the output value of \code{\link{pcaBySvd}}.
#' @param axes A numerical vector of length 2 with the Principal Components axes to be drawn.
#' @param title Plot title.
#' @param ... Additional arguments to be passed to \code{\link{plotLine}}.
#'
#' @return A `ggplot2` object with the PCA loading plot.
#'
#' @details
#' `pcaLoading1dPlot` is a wrapper of \code{\link{plotLine}}. See `?plotLine` for more information on the additional arguments.
#'
#'
#'
#' @examples
#'
#' data("UCH")
#' ResPCA <- pcaBySvd(UCH$outcomes)
#'
#' pcaLoading1dPlot(
#'   resPcaBySvd = ResPCA, axes = c(1, 2),
#'   title = "PCA loading plot UCH", xlab = "ppm", ylab = "Values"
#' )
pcaLoading1dPlot <- function(resPcaBySvd,
                             axes = c(1, 2),
                             title = "PCA loading plot", ...) {
  # checks   ===================
  checkArg(resPcaBySvd, c("list"), can.be.null = FALSE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)
  checkArg(title, c("str", "length1"), can.be.null = FALSE)

  if (!identical(names(resPcaBySvd), c(
    "scores", "loadings", "eigval", "singvar",
    "var", "cumvar", "original.dataset", "design"
  ))) {
    stop("resPcaBySvd is not an output value of pcaBySvd")
  }

  # loadings
  loadings <- t(resPcaBySvd$loadings)
  checkArg(loadings, c("matrix"), can.be.null = FALSE)

  if (max(axes) > nrow(loadings)) {
    stop(
      "axes (", paste(axes, collapse = ","),
      ") is beyond the ncol of loadings (", nrow(loadings), ")"
    )
  }

  # percentage of explained variance   ===================
  pc_var <- resPcaBySvd$var
  pc_var_x <- format(pc_var[pc_var >= 0.1],
    digits = 2,
    trim = TRUE
  )
  pc_var_y <- format(pc_var[pc_var < 0.1],
    digits = 2,
    scientific = TRUE, trim = TRUE
  )
  pc_var_char <- as.character(pc_var)
  pc_var_char[pc_var >= 0.1] <- pc_var_x
  pc_var_char[pc_var < 0.1] <- pc_var_y

  pc_var_char <- paste0(
    "PC", axes, " (",
    pc_var_char[axes], "%)"
  )

  # Loadings plot  ===================
  fig <- plotLine(
    Y = loadings, title = title,
    rows = axes, facet_label = pc_var_char,
    ...
  )

  # if (length(fig) == 1) {
  #   fig <- fig[[1]]
  # }

  return(fig)
}

#' @export lmpLoading2dPlot
#' @title Loading plots on a 2D scatter plot
#'
#' @description
#' Draws a 2D loading plot of each effect matrix provided in \code{\link{lmpPcaEffects}} outputs. As a wrapper of the \code{\link{plotScatter}} function, it allows the visualization of effect loading matrices for two components at a time with all options available in \code{\link{plotScatter}}.
#'
#' @param resLmpPcaEffects A list corresponding to the output value of \code{\link{lmpPcaEffects}}.
#' @param effectNames Names of the effects to be plotted. If `NULL`, all the effects are plotted.
#' @param axes A numerical vector with the 2 Principal Components axes to be drawn.
#' @param addRownames Boolean indicating if the labels should be plotted. By default, uses the column names of the outcome matrix but it can be manually specified with the `points_labs` argument from \code{\link{plotScatter}}.
#' @param pl_n The number of labels that should be plotted, based on the distance measure \eqn{d} (*see* Details).
#' @param metadata A nxk "free encoded" data.frame corresponding to `design` in \code{\link{plotScatter}}.
#' @param drawOrigin if \code{TRUE}, draws horizontal and vertical intercepts at (0,0) based on the \code{\link{plotScatter}} function.
#' @param ... Additional arguments to be passed to \code{\link{plotScatter}}.
#'
#' @return A list of loading plots (ggplot).
#'
#' @details
#' `lmpLoading2dPlot` is a wrapper of \code{\link{plotScatter}}. See `?plotScatter` for more information on the additional arguments.
#'
#'
#' The distance measure \eqn{d}{d} that is used to rank the variables is based on the following formula:
#' \deqn{d = \sqrt(P_{ab}^2*\lambda_{ab}^2)}{d = sqrt(P_ab^2 * lambda_ab^2)} where \eqn{a}{a}
#' and \eqn{b}{b} are two selected Principal Components, \eqn{P_{ab}}{P_ab} represents their
#' loadings and \eqn{\lambda_{ab}}{lambda_ab} their singular values.
#'
#' @examples
#'
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resASCA <- lmpPcaEffects(resLmpEffectMatrices)
#'
#' lmpLoading2dPlot(resASCA, effectNames = "Hippurate")
#'
#' # adding color, shape and labels to points
#' id_hip <- c(seq(126, 156), seq(362, 375))
#' peaks <- rep("other", ncol(UCH$outcomes))
#' peaks[id_hip] <- "hip"
#' metadata <- data.frame(peaks)
#'
#' lmpLoading2dPlot(resASCA,
#'   effectNames = "Hippurate",
#'   metadata = metadata, addRownames = TRUE, color = "peaks",
#'   shape = "peaks"
#' )
#'
#' # changing max.overlaps of ggrepel
#' options(ggrepel.max.overlaps = 30)
#' lmpLoading2dPlot(resASCA,
#'   effectNames = "Hippurate",
#'   metadata = metadata, addRownames = TRUE, color = "peaks",
#'   shape = "peaks", pl_n = 20
#' )
#'
#' @importFrom methods hasArg


lmpLoading2dPlot <- function(resLmpPcaEffects,
                             effectNames = NULL,
                             axes = c(1, 2),
                             addRownames = FALSE,
                             pl_n = 10,
                             metadata = NULL,
                             drawOrigin = TRUE,
                             ...) {
  mcall <- as.list(match.call())[-1L]

  # checks ===================
  checkArg(resLmpPcaEffects, c("list"), can.be.null = FALSE)
  checkArg(effectNames, c("str"), can.be.null = TRUE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)
  checkArg(metadata, "data.frame", can.be.null = TRUE)

  if (!all(effectNames %in% names(resLmpPcaEffects))) {
    stop("One of the effects from effectNames is not in resLmpPcaEffects.")
  }

  if (!identical(
    names(resLmpPcaEffects[seq((length(resLmpPcaEffects) - 7), length(resLmpPcaEffects))]),
    c(
      "Residuals", "lmpDataList", "effectsNamesUnique",
      "effectsNamesUniqueCombined",
      "method", "type3SS", "variationPercentages",
      "combineEffects"
    )
  )) {
    stop("resLmpPcaEffects is not an output value of lmpPcaEffects")
  }


  if (is.null(effectNames)) {
    effectNames <- resLmpPcaEffects$effectsNamesUniqueCombined
    effectNames <- effectNames[effectNames != "Intercept"]
    effectNames <- c(effectNames, "Residuals")
  }


  # loadings   ===================

  loadings <- lapply(effectNames, function(x) {
    resLmpPcaEffects[[x]][["loadings"]]
  })
  names(loadings) <- effectNames

  if (length(axes) != 2) {
    stop("axes is not of length 2")
  }

  if (max(axes) > ncol(loadings[[effectNames[1]]])) {
    stop(
      "axes (", paste(axes, collapse = ","),
      ") is beyond the ncol of loadings (", ncol(loadings), ")"
    )
  }



  # percentage of explained variance   ===================

  pc_var_fun <- function(effect) {
    pc_var <- resLmpPcaEffects[[effect]][["var"]]
    pc_var_x <- format(pc_var[pc_var >= 0.1],
      digits = 2, trim = TRUE
    )
    pc_var_y <- format(pc_var[pc_var < 0.1],
      digits = 2,
      scientific = TRUE, trim = TRUE
    )
    pc_var_char <- as.character(pc_var)
    pc_var_char[pc_var >= 0.1] <- pc_var_x
    pc_var_char[pc_var < 0.1] <- pc_var_y

    pc_var_char <- paste0(
      "PC", axes, " (",
      pc_var_char[axes], "%)"
    )

    return(pc_var_char)
  }

  pc_axes <- lapply(effectNames, pc_var_fun)
  names(pc_axes) <- effectNames


  # distance measure and labels  ===================

  if (methods::hasArg("points_labs")) {
    addRownames <- FALSE
  } else {
    labs <- colnames(resLmpPcaEffects$lmpDataList$outcomes)
    dist_labels_fun <- function(effect, axes, labs) {
      load <- resLmpPcaEffects[[effect]][["loadings"]][, axes]
      singvar <- resLmpPcaEffects[[effect]][["singvar"]][axes]
      dista <- load^2 %*% singvar^2
      ids <- order(dista, decreasing = TRUE)[seq_len(pl_n)]
      labs[-ids] <- ""
      labs
    }

    points_labels <- lapply(effectNames, dist_labels_fun,
      axes = axes, labs = labs
    )

    names(points_labels) <- effectNames
  }





  # graphical parameters   ===================

  buildFig <- function(effect) {
    title <- paste(
      effect, ":", resLmpPcaEffects$method,
      "loading plot"
    )

    xlab_pc <- pc_axes[[effect]][1]
    ylab_pc <- pc_axes[[effect]][2]

    xlim_val <- c(
      1.4 * min(resLmpPcaEffects[[effect]][["loadings"]][, axes[1]]),
      1.4 * max(resLmpPcaEffects[[effect]][["loadings"]][, axes[1]])
    )

    # Checking the second component
    if (resLmpPcaEffects$method != "APCA") {
      if (resLmpPcaEffects[[effect]][["var"]][axes[2]] < 1) {
        warning("The variance of PC2 is inferior to 1%. Graph scaled")
        ylim_val <- c(
          100 * min(resLmpPcaEffects[[effect]][["loadings"]][, axes[2]]),
          100 * max(resLmpPcaEffects[[effect]][["loadings"]][, axes[2]])
        )
      } else {
        ylim_val <- c(
          1.4 * min(resLmpPcaEffects[[effect]][["loadings"]][, axes[2]]),
          1.4 * max(resLmpPcaEffects[[effect]][["loadings"]][, axes[2]])
        )
      }
    } else {
      ylim_val <- c(
        1.4 * min(resLmpPcaEffects[[effect]][["loadings"]][, axes[2]]),
        1.4 * max(resLmpPcaEffects[[effect]][["loadings"]][, axes[2]])
      )
    }

    # Building plots

    if (addRownames) {
      if (!"xlab" %in% names(mcall)) {
        if (!"ylab" %in% names(mcall)) {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            xlab = xlab_pc, ylab = ylab_pc,
            design = metadata,
            points_labs = points_labels[[effect]],
            drawOrigin = drawOrigin,
            ...
          )
        } else {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            xlab = xlab_pc,
            design = metadata,
            points_labs = points_labels[[effect]],
            drawOrigin = drawOrigin,
            ...
          )
        }
      } else {
        if (!"ylab" %in% names(mcall)) {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            ylab = ylab_pc,
            design = metadata,
            points_labs = points_labels[[effect]],
            drawOrigin = drawOrigin,
            ...
          )
        } else {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            design = metadata,
            points_labs = points_labels[[effect]],
            drawOrigin = drawOrigin,
            ...
          )
        }
      }
    } else {
      if (!"xlab" %in% names(mcall)) {
        if (!"ylab" %in% names(mcall)) {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            xlab = xlab_pc, ylab = ylab_pc,
            design = metadata,
            drawOrigin = drawOrigin, ...
          )
        } else {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            xlab = xlab_pc,
            design = metadata,
            drawOrigin = drawOrigin, ...
          )
        }
      } else {
        if (!"ylab" %in% names(mcall)) {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            ylab = ylab_pc,
            design = metadata,
            drawOrigin = drawOrigin, ...
          )
        } else {
          fig <- plotScatter(
            Y = loadings[[effect]], xy = axes,
            design = metadata,
            drawOrigin = drawOrigin, ...
          )
        }
      }
    }


    fig <- fig + ylim(ylim_val) + xlim(xlim_val) +
      ggtitle(title)
  }

  # loadings plot  ===================

  fig <- lapply(effectNames, buildFig)
  names(fig) <- effectNames

  if (length(fig) == 1) {
    fig <- fig[[1]]
  }


  return(fig)
}

#' @export lmpLoading1dPlot
#' @title Loadings represented on a line plot.
#'
#' @description
#' Plots the loading vectors for each effect matrix from the \code{\link{lmpPcaEffects}} outputs with line plots. This is a wrapper of plotLine.
#'
#' @param resLmpPcaEffects A list corresponding to the output value of \code{\link{lmpPcaEffects}}.
#' @param effectNames Names of the effects to be plotted. if `NULL`, all the effects are plotted.
#' @param axes A numerical vector with the Principal Components axes to be drawn.
#' @param ... Additional arguments to be passed to \code{\link{plotLine}} such as `xaxis_type`, `type` or `ang_x_axis`.
#'
#' @return A list of `ggplot` objects representing the loading plots.
#'
#' @details
#' `lmpLoading1dPlot` is a wrapper of \code{\link{plotLine}}. See `?plotLine` for more information on the additional arguments.
#'
#'
#' @examples
#'
#' # Example of "spectral" type loadings (line and numerical x-axis)
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resASCA <- lmpPcaEffects(resLmpEffectMatrices,
#'   combineEffects = list(c("Time", "Hippurate:Time"))
#' )
#' lmpLoading1dPlot(resASCA)
#' lmpLoading1dPlot(resASCA, effectNames = c("Hippurate", "Citrate"))
#'
#' # Example of "segment" and discrete type loadings (segments and character x-axis)
#' data("trout")
#' resLmpModelMatrix <- lmpModelMatrix(trout)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resASCA <- lmpPcaEffects(resLmpEffectMatrices)
#' lmpLoading1dPlot(resASCA,
#'   effectNames = "Day",
#'   xaxis_type = "character", type = "s", ang_x_axis = 90
#' )
lmpLoading1dPlot <- function(resLmpPcaEffects, effectNames = NULL,
                             axes = c(1, 2), ...) {
  # checks   ===================

  checkArg(resLmpPcaEffects, c("list"), can.be.null = FALSE)
  checkArg(effectNames, c("str"), can.be.null = TRUE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)

  if (!all(effectNames %in% names(resLmpPcaEffects))) {
    stop("One of the effects from effectNames is not in resLmpPcaEffects.")
  }

  if (!identical(
    names(resLmpPcaEffects[(length(resLmpPcaEffects) - 7):length(resLmpPcaEffects)]),
    c(
      "Residuals", "lmpDataList", "effectsNamesUnique",
      "effectsNamesUniqueCombined",
      "method", "type3SS", "variationPercentages",
      "combineEffects"
    )
  )) {
    stop("resLmpPcaEffects is not an output value of lmpPcaEffects")
  }

  if (is.null(effectNames)) {
    effectNames <- resLmpPcaEffects$effectsNamesUniqueCombined
    effectNames <- effectNames[effectNames != "Intercept"]
    effectNames <- c(effectNames, "Residuals")
  }



  # loadings

  loadings <- lapply(effectNames, function(x) {
    t(resLmpPcaEffects[[x]][["loadings"]])
  })
  names(loadings) <- effectNames

  # percentage of explained variance   ===================

  pc_var_fun <- function(effect) {
    pc_var <- resLmpPcaEffects[[effect]][["var"]]
    pc_var_x <- format(pc_var[pc_var >= 0.1],
      digits = 2, trim = TRUE
    )
    pc_var_y <- format(pc_var[pc_var < 0.1],
      digits = 2,
      scientific = TRUE, trim = TRUE
    )
    pc_var_char <- as.character(pc_var)
    pc_var_char[pc_var >= 0.1] <- pc_var_x
    pc_var_char[pc_var < 0.1] <- pc_var_y

    pc_var_char <- paste0(
      "PC", axes, " (",
      pc_var_char[axes], "%)"
    )

    return(pc_var_char)
  }

  lab <- lapply(effectNames, pc_var_fun)
  names(lab) <- effectNames


  # Loadings plot  ===================

  buildFig <- function(effect) {
    title <- paste(effect, ": loading plot")

    plotLine(
      Y = loadings[[effect]], title = title,
      rows = axes, facet_label = lab[[effect]], ...
    )
  }

  fig <- lapply(effectNames, buildFig)
  names(fig) <- effectNames


  if (length(fig) == 1) {
    fig <- fig[[1]]
  }

  return(fig)
}

#' @export lmpScorePlot
#' @title Score plots of effect matrices
#'
#' @description
#' Draws the score plots of each (augmented) effect matrix provided in \code{\link{lmpPcaEffects}}. As a wrapper of the \code{\link{plotScatter}} function, it allows to visualize the scores of the effect matrices for two components at a time with all the available options in  \code{\link{plotScatter}}.
#'
#' @param resLmpPcaEffects A list corresponding to the output value of \code{\link{lmpPcaEffects}}.
#' @param effectNames Names of the effects to be plotted. If `NULL`, all the effects are plotted.
#' @param axes A numerical vector with the 2 Principal Components axes to be drawn.
#' @param ... Additional arguments to be passed to \code{\link{plotScatter}}.
#'
#' @return A list of score plots (ggplot).
#'
#' @details
#' `lmpScorePlot` is a wrapper of \code{\link{plotScatter}}.
#'
#'
#' @examples
#'
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#'
#' # PCA decomposition of effect matrices (ASCA)
#' resASCA <- lmpPcaEffects(resLmpEffectMatrices)
#' # Score plot of Hippurate effect matrix
#' lmpScorePlot(resASCA,
#'   effectNames = "Hippurate",
#'   color = "Hippurate", shape = "Hippurate"
#' )
#'
#' # PCA decomposition of augmented effect matrices (APCA)
#' resASCA <- lmpPcaEffects(resLmpEffectMatrices, method = "APCA")
#' # Score plot of Hippurate augmented effect matrix
#' lmpScorePlot(resASCA,
#'   effectNames = "Hippurate",
#'   color = "Hippurate", shape = "Hippurate", drawShapes = "ellipse"
#' )
lmpScorePlot <- function(resLmpPcaEffects, effectNames = NULL,
                         axes = c(1, 2), ...) {
  mcall <- as.list(match.call())[-1L]

  # checks ===================
  checkArg(resLmpPcaEffects, c("list"), can.be.null = FALSE)
  checkArg(effectNames, c("str"), can.be.null = TRUE)
  checkArg(axes, c("int", "pos"), can.be.null = FALSE)

  if (!all(effectNames %in% names(resLmpPcaEffects))) {
    stop("One of the effects from effectNames is not in resLmpPcaEffects.")
  }

  if (!identical(
    names(resLmpPcaEffects[(length(resLmpPcaEffects) - 7):length(resLmpPcaEffects)]),
    c(
      "Residuals", "lmpDataList", "effectsNamesUnique",
      "effectsNamesUniqueCombined",
      "method", "type3SS", "variationPercentages",
      "combineEffects"
    )
  )) {
    stop("resLmpPcaEffects is not an output value of
         lmpPcaEffects")
  }

  if (is.null(effectNames)) {
    effectNames <- resLmpPcaEffects$effectsNamesUniqueCombined
    effectNames <- effectNames[effectNames != "Intercept"]
    effectNames <- c(effectNames, "Residuals")
  }


  # scores
  scores <- lapply(effectNames, function(x) {
    resLmpPcaEffects[[x]][["scores"]]
  })
  names(scores) <- effectNames

  if (length(axes) != 2) {
    stop("axes is not of length 2")
  }

  if (max(axes) > ncol(scores[[effectNames[1]]])) {
    stop(
      "axes (", paste(axes, collapse = ","),
      ") is beyond the ncol of scores (", ncol(scores), ")"
    )
  }

  # percentage of explained variance   ===================

  pc_var_fun <- function(effect) {
    pc_var <- resLmpPcaEffects[[effect]][["var"]]
    pc_var_x <- format(pc_var[pc_var >= 0.1],
      digits = 2, trim = TRUE
    )
    pc_var_y <- format(pc_var[pc_var < 0.1],
      digits = 2,
      scientific = TRUE, trim = TRUE
    )
    pc_var_char <- as.character(pc_var)
    pc_var_char[pc_var >= 0.1] <- pc_var_x
    pc_var_char[pc_var < 0.1] <- pc_var_y

    pc_var_char <- paste0(
      "PC", axes, " (",
      pc_var_char[axes], "%)"
    )

    return(pc_var_char)
  }

  pc_axes <- lapply(effectNames, pc_var_fun)
  names(pc_axes) <- effectNames


  # graphical parameters   ===================

  buildFig <- function(effect) {
    title <- paste0(
      effect, ": ",
      resLmpPcaEffects$method, " score plot"
    )

    xlab <- pc_axes[[effect]][1]
    ylab <- pc_axes[[effect]][2]

    xlim_val <- c(
      1.4 * min(resLmpPcaEffects[[effect]][["scores"]][, axes[1]]),
      1.4 * max(resLmpPcaEffects[[effect]][["scores"]][, axes[1]])
    )

    # Checking the second component
    if (resLmpPcaEffects$method != "APCA") {
      if (resLmpPcaEffects[[effect]][["var"]][axes[2]] < 1) {
        warning("The variance of PC2 is inferior to 1%. Graph scaled")
        ylim_val <- c(
          -100 * abs(min(resLmpPcaEffects[[effect]][["scores"]][, axes[2]])),
          100 * abs(min(resLmpPcaEffects[[effect]][["scores"]][, axes[2]]))
        )
      } else {
        ylim_val <- c(
          1.4 * min(resLmpPcaEffects[[effect]][["scores"]][, axes[2]]),
          1.4 * max(resLmpPcaEffects[[effect]][["scores"]][, axes[2]])
        )
      }
    } else {
      ylim_val <- c(
        1.4 * min(resLmpPcaEffects[[effect]][["scores"]][, axes[2]]),
        1.4 * max(resLmpPcaEffects[[effect]][["scores"]][, axes[2]])
      )
    }

    # Building plots

    if (!"xlab" %in% names(mcall)) {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = scores[[effect]], xy = axes,
          xlab = xlab, ylab = ylab,
          design = resLmpPcaEffects$lmpDataList$design, ...
        )
      } else {
        fig <- plotScatter(
          Y = scores[[effect]], xy = axes,
          xlab = xlab,
          design = resLmpPcaEffects$lmpDataList$design, ...
        )
      }
    } else {
      if (!"ylab" %in% names(mcall)) {
        fig <- plotScatter(
          Y = scores[[effect]], xy = axes,
          ylab = ylab,
          design = resLmpPcaEffects$lmpDataList$design, ...
        )
      } else {
        fig <- plotScatter(
          Y = scores[[effect]], xy = axes,
          design = resLmpPcaEffects$lmpDataList$design, ...
        )
      }
    }

    fig <- fig + ylim(ylim_val) + xlim(xlim_val) +
      ggtitle(title)
  }

  # Scores plot  ===================

  fig <- lapply(effectNames, buildFig)
  names(fig) <- effectNames

  if (length(fig) == 1) {
    fig <- fig[[1]]
  }


  return(fig)
}

ModelAbbrev <- function(vectorname) {
  # formula = ResLMobject[[1]]
  #
  # ModelTerms_sansE <- attr(terms(formula),"term.labels") # without residuals
  # ModelTerms <- c(ModelTerms_sansE, "residuals") # with residuals
  ModelTerms <- vectorname

  ModelTerms_abbrev <- ModelTerms # Abbreviated model terms
  index <- gregexpr(":", ModelTerms)
  for (i in seq_along(ModelTerms)) {
    if (index[[i]][1] != -1) {
      ModelTerms_abbrev[i] <- substr(ModelTerms[i], 1, 1)
      index2 <- index[[i]]
      for (k in seq_along(index2)) {
        ModelTerms_abbrev[i] <- paste(ModelTerms_abbrev[i],
          substr(
            ModelTerms[i],
            index2[k] + 1,
            index2[k] + 1
          ),
          sep = "x"
        )
      }
    }
  }

  ModelTerms_abbrev[length(ModelTerms_abbrev)] <- "Residuals"

  # ResLMobject = utils::modifyList(ResLMobject,list(ModelTermsAbbrev=ModelTerms_abbrev))
  # return(ResLMobject)
  return(ModelTerms_abbrev)
}

#' @export lmpScreePlot
#' @title Scree Plot
#'
#' @description
#' Provides a barplot of the percentage of variance associated to the PCs of the effect matrices ordered by importance based on the outputs of \code{\link{lmpContributions}}.
#'
#' @param resLmpContributions A resLmpContributions list from the function \code{\link{lmpContributions}}.
#' @param effectNames Names of the effects to be plotted. if `NULL`, all the effects are plotted.
#' @param nPC An integer with the number of components to plot.
#' @param theme `ggplot` theme
#'
#' @return A scree plot (ggplot).
#'
#' @examples
#'
#' data("UCH")
#' resLmpModelMatrix <- lmpModelMatrix(UCH)
#' resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
#' resASCAE <- lmpPcaEffects(resLmpEffectMatrices, method = "ASCA-E")
#' resLmpContributions <- lmpContributions(resASCAE)
#' lmpScreePlot(resLmpContributions, effectNames = "Hippurate:Citrate", nPC = 4)
#'
#' @import ggplot2

lmpScreePlot <- function(resLmpContributions, effectNames = NULL,
                         nPC = 5, theme = theme_bw()) {
  # checks ===================
  checkArg(resLmpContributions, c("list"), can.be.null = FALSE)
  checkArg(effectNames, c("str"), can.be.null = TRUE)
  checkArg(nPC, c("num", "pos"), can.be.null = FALSE)

  effectTable <- resLmpContributions$effectTable
  if (!nPC < ncol(effectTable)) {
    stop("nPC must be inferior or equal to the nPC chosen
         in lmpContributions")
  }

  if (is.null(effectNames)) {
    effectNames <- c(rownames(resLmpContributions$effectTable))
  }

  if (!all(effectNames %in% rownames(resLmpContributions$effectTable))) {
    stop("One of the effects from effectNames is not
         in resLmpContributions.")
  }

  # selecting the effect

  resdf <- as.data.frame(t(effectTable[, c(seq_len(nPC))]))

  buildFig <- function(x) {
    iEffect <- which(colnames(resdf) == effectNames[x])

    # Plotting the effect
    ggplot(
      data = resdf,
      aes(x = rownames(resdf), y = resdf[, iEffect])
    ) +
      geom_bar(stat = "identity") +
      xlab("Principal Components") +
      ylab("Variance Percentage") +
      ggtitle(paste(
        "Percentage of variance by PC for:",
        effectNames[x]
      )) +
      theme
  }

  fig <- lapply(c(seq_along(effectNames)), FUN = buildFig)
  names(fig) <- effectNames

  return(fig)
}

#' @export data2LmpDataList
#' @title Converts data to a lmpDataList.
#'
#' @description
#' Creates the lmpDataList from a SummarizedExperiment or by manually defining the design, the outcomes and the model formula.
#' lmpDataList serves as an input for the \code{\link{lmpModelMatrix}} function to start the limpca modeling.
#'
#' @param se A \code{\link[SummarizedExperiment]{SummarizedExperiment}} object.
#' @param assay_name If not \code{NULL} (default), a character string naming the assay from the \code{\link[SummarizedExperiment]{SummarizedExperiment}} object \code{se}. If \code{NULL}, the first assay is selected.
#' @param outcomes If not \code{NULL} (default), a numerical matrix with \emph{n} observations and \emph{m} response variables. The rownames needs to be non-NULL and match those of the design matrix.
#' @param design If not \code{NULL} (default), a data.frame with the experimental design of \emph{n} observations and \emph{q} explanatory variables. The rownames of design has to match the rownames of outcomes.
#' @param formula If not \code{NULL} (default), a character string with the formula that will be used to analyze the data. Only the right part of the formula is necessary, eg: \code{"~ A + B"}, The names of the formula should match the column names of the design
#' @param verbose If \code{TRUE}, prints useful information about the outputted list.
#'
#' @return A list with the 3 following named elements:
#' \describe{
#'    \item{\code{outcomes}}{A \emph{nxm} matrix with the \emph{m} response variables.}
#'    \item{\code{design}}{A \emph{nxq} data.frame with the experimental design.}
#'    \item{\code{formula}}{A character string with the model formula.}
#'    }
#'
#' @details
#' Data can be included as a \code{\link[SummarizedExperiment]{SummarizedExperiment}} (SE) object or by manually defining one or multiple
#' elements of \code{outcomes}, \code{design} and \code{formula}. If a SE is provided,
#' the \code{outcomes} corresponds to a transposed assay of the SE (by default the first one),
#' the \code{design} corresponds to the \code{\link[SummarizedExperiment]{colData}} of the SE and the \code{formula} can be provided as a
#' \code{formula} element in the \code{S4Vectors::metadata} of SE (\code{metadata(se)$formula}).
#'
#' In the outputted list, the outcomes are structured in a standard statistical fashion,
#' i.e. with observations in rows and the variables (features) in column.
#' If the \code{outcomes} argument is not \code{NULL}, it has to be formatted that way (see Arguments).
#'
#' Note that there is a priority to the \code{outcomes}, \code{design} and \code{formula}
#' arguments if they are not \code{NULL} (e.g. if both \code{se} and \code{outcomes} arguments are provided,
#' the resulting outcomes matrix will be from the \code{outcomes} argument). \code{outcomes} and \code{design} elements are mandatory.
#'
#' Multiple checks are performed to ensure that the data are correctly formatted:
#' - the rownames of \code{design} and \code{outcomes} should match
#' - the names of the model terms in the \code{formula} should match column names from the \code{design}
#'
#'
#' @seealso \code{\link[SummarizedExperiment]{SummarizedExperiment}}
#'
#' @examples
#'
#' data(UCH)
#'
#' ### create manually the dataset
#'
#' res <- data2LmpDataList(
#'   outcomes = UCH$outcomes,
#'   design = UCH$design[, 1, drop = FALSE], formula = "~ Hippurate"
#' )
#'
#' ### create the dataset from a SummarizedExperiment
#'
#' library(SummarizedExperiment)
#'
#' se <- SummarizedExperiment(
#'   assays = list(
#'     counts = t(UCH$outcomes),
#'     counts2 = t(UCH$outcomes * 2)
#'   ), colData = UCH$design,
#'   metadata = list(formula = "~ Hippurate + Citrate")
#' )
#'
#' res <- data2LmpDataList(se, assay_name = "counts2")
#'
#' # changing the formula:
#' res <- data2LmpDataList(se,
#'   assay_name = "counts2",
#'   formula = "~ Hippurate + Citrate + Time"
#' )
#'
#' @import SummarizedExperiment
#' @importFrom stats as.formula
#' @importFrom S4Vectors metadata
#' @importFrom methods is

data2LmpDataList <- function(se = NULL, assay_name = NULL,
                             outcomes = NULL, design = NULL,
                             formula = NULL,
                             verbose = TRUE) {
  ## checks ===========

  # se
  if (!methods::is(se, "SummarizedExperiment") & !is.null(se)) {
    stop("se is not a SummarizedExperiment nor NULL")
  }

  checkArg(
    arg = assay_name, c("str", "length1"),
    can.be.null = TRUE
  )
  checkArg(verbose, "bool", can.be.null = FALSE)
  # checkArg(formula, c("str", "length1"), can.be.null = TRUE)
  # checkArg(design, "data.frame", can.be.null = TRUE)
  # checkArg(outcomes, "matrix", can.be.null = TRUE)

  ## if se is specified ===========
  coldat_se <- form_se <- assay_se <- NULL
  if (!is.null(se)) {
    coldat_se <- as.data.frame(SummarizedExperiment::colData(se))
    form_se <- S4Vectors::metadata(se)$formula

    if (!is.null(assay_name)) {
      if (!assay_name %in% assayNames(se)) {
        # checks that assay_name is in assayNames(se)
        stop("assay_name is not in assayNames(se)")
      }
      id <- which(assayNames(se) == assay_name)
      assay_se <- SummarizedExperiment::assay(se, i = id)
    } else if (is.null(assay_name)) {
      # if assay_name is null, take the first one
      assay_se <- SummarizedExperiment::assay(se, i = 1)
    }
  }

  # creating the output values
  out_design <- out_formula <- out_outcomes <- NULL

  if (!is.null(outcomes)) {
    out_outcomes <- outcomes
  } else if (!is.null(assay_se)) {
    out_outcomes <- t(assay_se)
  } else {
    stop("outcomes information is missing in the arguments")
  }

  if (!is.null(formula)) {
    out_formula <- formula
  } else if (!is.null(form_se)) {
    out_formula <- form_se
  } else {
    out_formula <- NULL
    warning("formula information is missing in the arguments")
  }

  # checkArg(out_formula, "formula", can.be.null = TRUE)

  if (!is.null(design)) {
    out_design <- design
  } else if (!is.null(coldat_se)) {
    out_design <- coldat_se
  } else {
    stop("design information is missing in the arguments")
  }


  ## format the data ====================
  lmpData <- list(
    outcomes = out_outcomes,
    design = out_design,
    formula = out_formula
  )

  lmpDataListCheck(
    lmpDataList = lmpData,
    null_formula = TRUE
  )


  # formatting formula

  if (!is.null(out_formula)) {
    formulaChar <- as.character(stats::as.formula(out_formula))

    if (length(formulaChar) == 3) {
      out_formula <- paste(
        formulaChar[1],
        formulaChar[3]
      )
    } else if (length(formulaChar) == 2) {
      out_formula <- paste(
        formulaChar[1],
        formulaChar[2]
      )
    } else {
      stop("Please put the formula argument in its right form: ~ model terms")
    }
  }


  # output ==============================

  l_out <- list(
    design = out_design, outcomes = out_outcomes,
    formula = out_formula
  )

  if (verbose) {
    # Function returning compact column classes
    col_classes <- function(df) {
      cc_l <- t(as.data.frame(lapply(df, function(x) {
        paste(class(x), collapse = ",")
      })))
      cc <- as.vector(cc_l)
      names(cc) <- rownames(cc_l)
      cc
    }
    cc <- col_classes(out_design)
    cn <- colnames(out_design)
    cc_p <- paste0("(", cc, ")")

    message("| dim outcomes: ", nrow(out_outcomes), "x", ncol(out_outcomes))
    message("| formula: ", as.character(out_formula))
    message(
      "| design variables (", ncol(out_design), "): \n",
      paste("*",paste(cn, cc_p), collapse = "\n")
    )
  }

  return(l_out)
}

set.seed(1)
design <- data.frame(
  factA = factor(c(rep(1, 5), rep(2, 5))),
  factB = factor(rep(c(1, 2), 5))
)
rownames(design) = as.character(seq_len(10))

outcomes <- matrix(rnorm(30), ncol = 3)
rownames(outcomes) = seq_len(10)
formula <- "~ factA+factB"

se_UCH = SummarizedExperiment(assays = list(counts = t(outcomes),
                                            counts2 = t(outcomes*2)),
                              colData = design,
                              metadata = list(formula = formula))


res = data2LmpDataList(se_UCH)

test_that("data2LmpDataList based on SE runs correctly", {
  expect_type(res, "list")
  expect_length(res, 3)
  expect_equal(res$design, design)
  expect_equal(res$outcomes, outcomes)
  expect_equal(as.formula(res$formula), as.formula(formula))
})

res = data2LmpDataList(se_UCH, assay_name = "counts2")

test_that("data2LmpDataList based on SE runs correctly", {
  expect_type(res, "list")
  expect_length(res, 3)
  expect_equal(res$design, design)
  expect_equal(res$outcomes, outcomes*2)
  expect_equal(as.formula(res$formula), as.formula(formula))
})


res = data2LmpDataList(outcomes = outcomes, design = design, formula = formula)

test_that("data2LmpDataList manually defined runs correctly", {
  expect_type(res, "list")
  expect_length(res, 3)
  expect_equal(res$design, design)
  expect_equal(res$outcomes, outcomes)
  expect_equal(as.formula(res$formula), as.formula(formula))
})


res = data2LmpDataList(se_UCH, assay_name = "counts",
                       outcomes = outcomes*4)

test_that("data2LmpDataList defined both manually and based on SE runs correctly", {
  expect_type(res, "list")
  expect_length(res, 3)
  expect_equal(res$design, design)
  expect_equal(res$outcomes, outcomes*4)
  expect_equal(as.formula(res$formula), as.formula(formula))
})




test_that("data2LmpDataList with sampled rownames of design runs correctly", {
  expect_warning(
    data2LmpDataList(outcomes = outcomes[sample(1:10,replace = FALSE),],
                     design = design, formula = formula, verbose = FALSE)
  )
})

### misspecified

test_that("data2LmpDataList with misspecified arguments fails", {
  expect_error(data2LmpDataList(se = outcomes))
  expect_error(data2LmpDataList(se_UCH, assay_name = "aaa"))
  expect_error(data2LmpDataList(outcomes = outcomes[-1,],
                                design = design, formula = formula))
  expect_error(data2LmpDataList())
  expect_error(data2LmpDataList(design = design, formula = formula))
  expect_error(data2LmpDataList(outcomes = outcomes, formula = formula))
  expect_warning(data2LmpDataList(outcomes = outcomes, design = design))
})


test_that("data2LmpDataList with misspecified design rownames fails", {
  design2 = design
  rownames(design2)[1:2] = c("aaa", "bbb")
  expect_error(data2LmpDataList(outcomes = outcomes,
                                design = design2, formula = formula))
})


test_that("data2LmpDataList with no outcomes rownames fails", {
  outcomes2 = outcomes
  rownames(outcomes2) = NULL
  expect_error(data2LmpDataList(outcomes = outcomes2,
                                design = design, formula = formula))
})

---
title: "Get started with limpca."
author: "Benaiche Nadia, Sébastien Franceschini, Martin Manon, Thiel Michel, Govaerts Bernadette"
date: '`r format(Sys.time(), "%B %d, %Y")`'
package: limpca
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 4
    toc_float: true
vignette: >
   %\VignetteIndexEntry{Get started with limpca}
   %\VignetteEncoding{UTF-8}
   %\VignetteEngine{knitr::rmarkdown}
editor_options: 
   chunk_output_type: console
bibliography: references.bib  
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>",
    out.width = "70%",
    fig.align = "center",
    eval = TRUE
)
rm(list = ls())
library("limpca")

if (!requireNamespace("SummarizedExperiment", quietly = TRUE)) {
    stop("Install 'SummarizedExperiment' to knit this vignette")
}
library(SummarizedExperiment)
```  


<!-- badges: start -->
[![R-CMD-check](https://github.com/ManonMartin/limpca/actions/workflows/check-standard.yaml/badge.svg)](https://github.com/ManonMartin/limpca/actions/workflows/check-standard.yaml)
<!-- badges: end -->

# Introduction

## About the package

This package was created to analyse models with high-dimensional data and a multi-factor design of experiment. `limpca` stands for **li**near **m**odeling of high-dimensional designed data based on the ASCA (ANOVA-Simultaneous Component Analysis) and A**PCA** (ANOVA-Principal Component Analysis) family of methods. These methods combine ANOVA with a General Linear Model (GLM) decomposition and PCA. They provide powerful visualization tools for multivariate structures in the space of each effect of the statistical model linked to the experimental design. Details on the methods used and the package implementation can be found in the articles of @Thiel2017, @Guisset2019 and @Thiel2023.

Therefore, ASCA/APCA are highly informative modeling and visualisation tools to analyse -omics data tables in a multivariate framework and act as a complement to differential expression analyses methods such as `limma` (@limma2015). 

## Vignettes description

- `Get started with limpca` (this vignette): This vignette is a short application of `limpca` on the `UCH` dataset with data visualisation, exploration (PCA), GLM decomposition and ASCA modelling. The ASCA model used in this example is a three-way ANOVA with fixed effects.

- [Analysis of the UCH dataset with limpca](UCH.html): This vignette is an extensive application of `limpca` on the `UCH` dataset with data visualisation, exploration (PCA), GLM decomposition and ASCA/APCA/ASCA-E modelling. The applied model is a three-way ANOVA with fixed effects. This document presents all the usual steps of the analysis, from importing the data to visualising the results.

- [Analysis of the Trout dataset with limpca](Trout.html): This vignette is an extensive application of `limpca` on the `Trout` dataset with data visualisation, exploration (PCA), GLM decomposition and ASCA/APCA/ASCA-E modelling. The applied model involves three main effects and their two-way interaction terms. It also compares the results of ASCA to a univariate ANOVA modeling. 





# Installation and loading of the `limpca` package

`limpca` can be installed from Bioconductor:

```{r Install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("limpca")
```

And then loaded into your R session: 

```{r Load, results=FALSE, message=FALSE}
library("limpca")
```


For any enquiry, you can send an email to the package authors: [bernadette.govaerts@uclouvain.be](mailto:bernadette.govaerts@uclouvain.be) ; [michel.thiel@uclouvain.be](mailto:michel.thiel@uclouvain.be) or [manon.martin@uclouvain.be](mailto:manon.martin@uclouvain.be)


# Short application on the `UCH` dataset

## Data object

In order to use the limpca core functions, the data need to be formatted as a list (informally called an lmpDataList) with the following elements: `outcomes` (multivariate matrix), `design` (data.frame) and `formula` (character string). 
The `UCH` data set is already formatted appropriately and can be loaded from `limpca` with the `data` function.

```{r}
data("UCH")
str(UCH)
```



Alternatively, the lmpDataList can be created with the function `data2LmpDataList` :

- from scratch:

```{r, message = TRUE}
UCH2 <- data2LmpDataList(
   outcomes = UCH$outcomes,
   design = UCH$design, 
   formula = UCH$formula
 )
```

- or from a `SummarizedExperiment`:

```{r, message = TRUE}
se <- SummarizedExperiment(
   assays = list(
     counts = t(UCH$outcomes)), colData = UCH$design,
   metadata = list(formula = UCH$formula)
 )

UCH3 <- data2LmpDataList(se, assay_name = "counts")
```

`SummarizedExperiment` is a generic data container that stores rectangular matrices of experimental results. See @SummarizedExperiment23 for more information.


## Data visualisation

The design can be visualised with `plotDesign()`.

```{r dataVisu}
# design
plotDesign(
    design = UCH$design, x = "Hippurate",
    y = "Citrate", rows = "Time",
    title = "Design of the UCH dataset"
)

# row 3 of outcomes
plotLine(
    Y = UCH$outcomes,
    title = "H-NMR spectrum",
    rows = c(3),
    xlab = "ppm",
    ylab = "Intensity"
)
```

## PCA

```{r PCA}
ResPCA <- pcaBySvd(UCH$outcomes)
pcaScreePlot(ResPCA, nPC = 6)
pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA scores plot: PC1 and PC2",
    design = UCH$design,
    color = "Hippurate", shape = "Citrate",
    points_labs_rn = FALSE
)
```

## Model estimation and effect matrix decomposition

```{r modelEst}
# Model matrix generation
resMM <- lmpModelMatrix(UCH)

# Model estimation and effect matrices decomposition
resEM <- lmpEffectMatrices(resMM)
```

## Effect matrix test of significance and importance measure

```{r effectImpSign}
# Effects importance
resEM$varPercentagesPlot

# Bootstrap tests
resBT <- lmpBootstrapTests(resLmpEffectMatrices = resEM, nboot = 100)
resBT$resultsTable
```


## ASCA decomposition

```{r ASCA}
# ASCA decomposition
resASCA <- lmpPcaEffects(resLmpEffectMatrices = resEM, method = "ASCA")

# Scores Plot for the hippurate
lmpScorePlot(resASCA,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate"
)

# Loadings Plot for the hippurate
lmpLoading1dPlot(resASCA,
    effectNames = c("Hippurate"),
    axes = 1, xlab = "ppm"
)

# Scores ScatterPlot matrix
lmpScoreScatterPlotM(resASCA,
    PCdim = c(1, 1, 1, 1, 1, 1, 1, 2),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "ASCA scores scatterplot matrix"
)
```


# sessionInfo
```{r}
sessionInfo()
```

---
title: "Supplementary information"
output: html_document
bibliography: reference.bib  
# nocite: '@*'
---


The list of articles related to this package is provided here:

> Thiel, Michel, Benaiche, Nadia, Martin, Manon, Franceschini, Sébastien, Van Oirbeek, Robin, Govaerts, Bernadette. 2023. limpca: An R package for the linear modeling of high-dimensional designed data based on ASCA/APCA family of methods. Journal of Chemometrics. 37( 7):e3482. doi:10.1002/cem.3482

**Open access** to the preprint: http://hdl.handle.net/2078.1/277096

> Govaerts, Bernadette, Bernard G. Francq, Rebecca Marion, Manon Martin, and Michel Thiel. 2020. “The Essentials on Linear Regression, ANOVA, General Linear and Linear Mixed Models for the Chemist.” In *Comprehensive Chemometrics, Chemical and Biochemical Data Analysis*, edited by 2nd Edition. Elsevier. http://hdl.handle.net/2078.1/230895.

**Open access** to the preprint: http://hdl.handle.net/2078.1/230335

> Martin, Manon, and Bernadette Govaerts. 2020. “LiMM‐PCA: Combining ASCA+ and Linear Mixed Models to Analyse High‐dimensional Designed Data.” Journal of Chemometrics 34. https://doi.org/10.1002/cem.3232.

**Open access** to the preprint: http://hdl.handle.net/2078.1/219772

> Guisset, Séverine, Manon Martin, and Bernadette Govaerts. 2019. “Comparison of PARAFASCA, AComDim, and AMOPLS Approaches in the Multivariate GLM Modelling of Multi-Factorial Designs.” *Chemometrics and Intelligent Laboratory Systems* 184: 44–63. https://doi.org/10.1016/j.chemolab.2018.11.006.

**Open access** to the preprint: http://hdl.handle.net/2078.1/207560

> Thiel, Michel, Baptiste Féraud, and Bernadette Govaerts. 2017. “ASCA+ and APCA+: Extensions of ASCA and APCA in the Analysis of Unbalanced Multifactorial Designs.” *Journal of Chemometrics* 31 (6): e2895. https://doi.org/10.1002/cem.2895.

**Open access** to the preprint: http://hdl.handle.net/2078.1/176979

> Rousseau, Réjane. 2011. “Statistical Contributrion to the Analysis of Metabonomic Data in 1H-NMR Spectroscopy.” PhD thesis, UCL - SSH/LIDAM/ISBA - Institut de Statistique, Biostatistique et Sciences Actuarielles. http://hdl.handle.net/2078.1/75532.


<!-- <div id="refs"></div> -->

---
title: "Application of `limpca` on the UCH metabolomics dataset."
author: "Benaiche Nadia, Sébastien Franceschini, Martin Manon, Thiel Michel, Govaerts Bernadette"
date: '`r format(Sys.time(), "%B %d, %Y")`'
package: limpca
output:
  BiocStyle::html_document:
    toc: true
    toc_depth: 4
    toc_float: true
vignette: >
   %\VignetteIndexEntry{Analysis of the UCH dataset with limpca}
   %\VignetteEncoding{UTF-8}
   %\VignetteEngine{knitr::rmarkdown}
editor_options: 
   chunk_output_type: console
bibliography: references.bib  
resource_files:
  - figs/ASCAMethodo.png
  - figs/limpcaDEfunctions.png
  - figs/limpcaASCAfunctions.png
---

```{r Setup,include=FALSE}
# require(knitr)
knitr::opts_chunk$set(
    message = FALSE, warning = TRUE, comment = NA,
    crop = NULL,
    tidy.opts = list(width.cutoff = 60),
    tidy = TRUE, dpi = 50, dev = "png",
    fig.width = 5, fig.height = 3.5
)
rm(list = ls())

if (!requireNamespace("pander", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(pander)

library(ggplot2)
if (!requireNamespace("gridExtra", quietly = TRUE)) {
    stop("Install 'gridExtra' to knit this vignette")
}
library(gridExtra)

if (!requireNamespace("SummarizedExperiment", quietly = TRUE)) {
    stop("Install 'SummarizedExperiment' to knit this vignette")
}
library(SummarizedExperiment)
```

Package loading

```{r, eval=FALSE}

library(pander)
library(gridExtra)
library(ggplot2)
library(SummarizedExperiment)
```

# Introduction

The model used in this example is a three-way ANOVA with fixed effects. This document presents all the usual steps of the analysis, from importing the data to visualising the results. Details on the methods used and the package implementation can be found in the articles of @Thiel2017, @Guisset2019 and @Thiel2023.


# Installation and loading of the `limpca` package

`limpca` can be installed from Bioconductor:

```{r Install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("limpca")
```

And then loaded into your R session: 

```{r Load, results=FALSE, message=FALSE}
library("limpca")
```


# Data import

In order to use the limpca core functions, the data need to be formatted as a list (informally called an lmpDataList) with the following elements: `outcomes` (multivariate matrix), `design` (data.frame) and `formula` (character string). 
The `UCH` data set is already formatted appropriately and can be loaded from `limpca` with the `data` function.

```{r Data_Importation}
data("UCH")

str(UCH)
```


Alternatively, the lmpDataList can be created with the function `data2LmpDataList` :

- from scratch:

```{r, message = TRUE}
UCH2 <- data2LmpDataList(
   outcomes = UCH$outcomes,
   design = UCH$design, 
   formula = UCH$formula
 )
```

- or from a `SummarizedExperiment`:

```{r, message = TRUE}
se <- SummarizedExperiment(
   assays = list(
     counts = t(UCH$outcomes)), colData = UCH$design,
   metadata = list(formula = UCH$formula)
 )

UCH3 <- data2LmpDataList(se, assay_name = "counts")
```

`SummarizedExperiment` is a generic data container that stores rectangular matrices of experimental results. See @SummarizedExperiment23 for more information.


# Data exploration

The UCH (Urine-Citrate-Hippurate) data set is described in @Thiel2017 and @Guisset2019 and is issued form a metabolomics experiment.  In this experiment, 36 samples of a pool of rat urine samples were spiked with two molecules Citrate and Hippurate according to a $3^2$ full factorial design in the quantities of these two molecules.  The spiked samples were analyzed by ^1^H NMR at two different time after defrozing and over two different days.  Two of the spectra where finally missing at the end of the experiment.    

The UCH data set is a list containing 2 elements: 

* an `outcomes` matrix with 34 observations of 600 response variables representing the spectra from the ^1^H NMR spectroscopy, 
* a `design` matrix with 34 observations and 4 explanatory variables 

A `formula` with the general linear model to be estimated will be added to this list below. 

For the purpose of this example, only 3 factors of interest will be studied : concentrations of Hippurate and Citrate and Time after defrozing.

Here are the `limpca` functions that are available for data exploration.  


![limpca data exploration functions](figs/limpcaDEfunctions.png "limpca data exploration functions")


## Design

The design matrix contains the information about each observation for the four variables: Hippurate, Citrate, Day and Time. Only 3 of these variables are used in the model. The function `plotDesign` is useful to visualise the design.

```{r Design_visualization}
pander(head(UCH$design))
plotDesign(
    design = UCH$design, x = "Hippurate",
    y = "Citrate", rows = "Time",
    title = "Design of the UCH dataset"
)
```

This plot confirms that the design is a full 3x3x2 factorial design replicated twice with 2 missing values.  Hence, the design is not balanced. 

## Outcomes visualization

The 600 response (`outcomes`) variables represent, for each observation, the intensities of the ^1^H NMR spectra.   These spectra can be visualized by the `plotLine` function.

### `plotLine` function

Here, annotations (`cit_peaks` and `hip_peaks`) are added to the `ggplot` objects in order to highlight the Hippurate (green) and Citrate (red) peaks. 

```{r Spectrum_visualization}
p1 <- plotLine(
    Y = UCH$outcomes,
    title = "H-NMR spectrum",
    rows = c(3),
    xlab = "ppm",
    ylab = "Intensity"
)

cit_peaks <- annotate("rect",
    xmin = c(2.509), xmax = c(2.709),
    ymin = -Inf, ymax = Inf, alpha = 0.2,
    fill = c("tomato")
)

hip_peaks <- annotate("rect",
    xmin = c(7.458, 3.881), xmax = c(7.935, 4.041),
    ymin = -Inf, ymax = Inf, alpha = 0.2,
    fill = c("yellowgreen")
)

p1 <- plotLine(
    Y = UCH$outcomes,
    title = "H-NMR spectrum",
    rows = c(3),
    xlab = "ppm",
    ylab = "Intensity"
)

p1 + cit_peaks + hip_peaks
```

### `plotScatter` function

The `plotScatter` function enables to visualize the values of two outcomes variables with different colors or markers for the values of the design factors.  Here, it is used to show that the $3^2$ factorial design can be recovered from the intensities of the Hippurate and Citrate peaks in the spectra.  

```{r}
# xy corresponds to citrate (453) and hippurate peaks (369)
plotScatter(
    Y = UCH$outcomes,
    xy = c(453, 369),
    design = UCH$design,
    color = "Hippurate",
    shape = "Citrate"
)
# Or refering to the variables names (ppm values here)
plotScatter(
    Y = UCH$outcomes,
    xy = c("2.6092056", "3.9811536"),
    design = UCH$design,
    color = "Hippurate",
    shape = "Citrate"
)
```

### `plotScatterM` function

The `plotScatter` function allows to visualize the values of a series of outcomes variables with different colors or markers for the values of the design factors. Here, it is done for the 4 peaks of Hippurate and single peak of Citrate.

```{r, fig.width=6,fig.height=7}
plotScatterM(
    Y = UCH$outcomes, cols = c(133, 145, 150, 369, 453),
    design = UCH$design, varname.colorup = "Hippurate",
    varname.colordown = "Citrate"
)
```

### `plotMeans` function

The `plotMeans` represents the mean values of a response variable for different levels of the design factors. Here we show the evolution of the Citrate peak height with respect to the three design factors of interest. 
Note that the results of this function must be interpreted with caution for unbalanced designs because simple means are biased estimators of expected means. 

```{r}
plotMeans(
    Y = UCH$outcomes,
    design = UCH$design,
    cols = c(453),
    x = c("Citrate"),
    w = c("Hippurate"),
    z = c("Time"),
    ylab = "Intensity",
    title = c("Mean reponse for main Citrate peak")
)
```

## PCA {#pca}

The function `pcaBySvd` is useful to compute a Principal Component Analysis (PCA) decomposition of the `outcomes` matrix. Several functions can be applied to the output value of `pcaBySvd`:

* `pcaScreePlot` to obtaine a scree plot
* `pcaLoading1dPlot` for the loading plots
* `pcaScorePlot` for the score plots

```{r PCA}
ResPCA <- pcaBySvd(UCH$outcomes)
pcaScreePlot(ResPCA, nPC = 6)
```

The score plots below indicate that all tree factors from the design affect the spectral profiles, which will be more clearly highlighted by ASCA and APCA.  

```{r Scores}
pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA scores plot: PC1 and PC2",
    design = UCH$design,
    color = "Hippurate", shape = "Citrate",
    points_labs_rn = FALSE
)

pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA scores plot: PC1 and PC2",
    design = UCH$design,
    color = "Time", shape = "Hippurate",
    points_labs_rn = FALSE
)

pcaScorePlot(
    resPcaBySvd = ResPCA, axes = c(3, 4),
    title = "PCA scores plot: PC3 and PC4",
    design = UCH$design,
    color = "Time", shape = "Citrate",
    points_labs_rn = FALSE
)
```

In the first two loading plots, a mixture of Citrate and Hippurate peaks already appears but they are not separated.  

```{r Loadings}
p2 <- pcaLoading1dPlot(
    resPcaBySvd = ResPCA, axes = c(1, 2),
    title = "PCA loadings plot UCH", xlab = "ppm",
    ylab = "Intensity"
)

p2 + hip_peaks + cit_peaks
```

# Application of ASCA+ and APCA+ 

Here below, ASCA+ and APCA+ steps are illustrated on the `UCH` data set.  The following graph represents the steps of the methodology.   
![ASCA+/APCA+ methodology ](figs/ASCAMethodo.png "ASCA+/APCA+ methodology")
The next graph presents the functions available in `limpca` for this purpose.  They are all illustrated in the next sections.  

![limpca ASCA/APCA functions](figs/limpcaASCAfunctions.png "limpca ASCA/PCA functions")

# Model estimation and effect matrix decomposition

## Model formula

The formula of the ANOVA-GLM model used in this analysis is the 3 ways crossed ANOVA model:

```{r Formula}
UCH$formula <- "outcomes ~ Hippurate + Citrate + Time + Hippurate:Citrate +
                  Time:Hippurate + Time:Citrate + Hippurate:Citrate:Time"
```

## Model matrix generation

The first step of ASCA+ is to build the (GLM) model matrix from the experimental design matrix and the model. Each factor is reencoded with multiple binary variables using `contr.sum` coding. The size of the model matrix is  34xp  with p being the total number of parameters in the ANOVA model for one response.

The function `lmpModelMatrix` encodes the design matrix as a model matrix.

```{r ModelMatrix}
resLmpModelMatrix <- lmpModelMatrix(UCH)
pander::pander(head(resLmpModelMatrix$modelMatrix))
```

## Model estimation and effect matrices decomposition

`lmpEffectMatrices` is the used to estimate the linear model and decomposes the multivariate outcomes into effect matrices for every model term. This function calculates also type III effect contributions (in %) and generates a `barpot` to visualise these contributions.  

```{r EffectMatrices}
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
```

## Effects importance

The contributions from each effect is outputted from `lmpEffectMatrices`.

```{r}
pander(resLmpEffectMatrices$variationPercentages)
resLmpEffectMatrices$varPercentagesPlot
```

## Bootstrap tests and quantification of effects importance

`lmpBootstrapTests` applies a parametric bootstrap test to determine whether an effect is significant or not. We recommend the user to choose first a small value of `nboot` (e.g. nboot=100) to develop its code and increase it later on (e.g. nboot=1000) in order to get an accurate value for the p-values.    

```{r Bootstrap}
resLmpBootstrapTests <-
    lmpBootstrapTests(
        resLmpEffectMatrices = resLmpEffectMatrices,
        nboot = 100
    )

# Print P-values
pander::pander(t(resLmpBootstrapTests$resultsTable))
```


# ASCA/APCA/ASCA-E decomposition

The ASCA/APCA/ASCA-E decomposition enables to represent the information from the effect matrices in a space of reduced dimension through PCA. The function `lmpPcaEffects` has a method argument to define which method to use, namely `ASCA`, `APCA` or `ASCA-E`.  Combined effects matrices can also be built and visualized by PCA.  

## ASCA

The ASCA method performs PCA on the pure effect matrices.  Here a combined effect matrix `Hippurate+Time+Hippurate:Time` is also built.  

```{r ASCA_PCA}
resASCA <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "ASCA",
    combineEffects = list(c(
        "Hippurate", "Time",
        "Hippurate:Time"
    ))
)
```

### Contributions

The contribution of each principal component of the effects is calculated and reported in different tables and plots with the function `lmpContributions`. 

```{r ASCA_Contrib}
resLmpContributions <- lmpContributions(resASCA)
```

The tables are:

- `totalContribTable`: Table of the **contribution of each effect to the total variance in percentage** as outputted from `lmpEffectMatrices`.
```{r}
pander::pander(resLmpContributions$totalContribTable)
```

- `effectTable`: Table of the **percentage of variance explained by each Principal Component in each model effect decomposition**.

```{r}
pander::pander(resLmpContributions$effectTable)
```

- `combinedEffectTable`: Equivalent of the previous `effectTable` but for the *combination of effects* mentioned in `lmpPcaEffects`, here for `Hippurate+Time+Hippurate:Time`.
```{r}
pander::pander(resLmpContributions$combinedEffectTable)
```

- `contribTable`: Table of the **percentage of variance explained by each Principal Component of each effect reported to the percentage contribution of the given effect to the total variance**. This table gives an overview of the importance of each PC regardless of the effects.


```{r}
pander::pander(resLmpContributions$contribTable)
```

- Moreover `lmpContributions` also produces a barplot either with the ordered contributions per effect (`plotTotal`) or across all the PCs of the different effects (`plotContrib`).


```{r}
pander("Ordered contributions per effect:")
resLmpContributions$plotTotal

pander("For each PC of the different effects:")
resLmpContributions$plotContrib
```

In the following analysis, we will focus only on the first PC of the three main effects, the interaction `Hippurate:Time` and the residuals since the other PCs and effects account for less than 1% of the total variance.


### Scores and loadings Plots

The loadings can be represented either on a line plot with the function `lmpLoading1dPlot` to conveniently compare them with the original spectral profiles or on a scatterplot with the function `lmpLoading2dPlot`. 

Here we create an object including the loading plots (as line plots) for all the effects included in the model as well as the combined effect and the residuals.

```{r, include=TRUE}
all_loadings_pl <- lmpLoading1dPlot(resASCA,
    effectNames = c(
        "Hippurate", "Citrate", "Time",
        "Hippurate:Time",
        "Hippurate+Time+Hippurate:Time",
        "Residuals"
    ),
    axes = 1, xlab = "ppm"
)
```

The score matrices are represented two components at a time on a scatterplot with the function `lmpScorePlot`.   

#### Main effects

The scores and loadings of the main effects included in the model are represented below.

They show that, thank to the ASCA analysis, Hippurate and Citrate peaks are clearly and separately retrieved in the respective loading plots compared to the classical PCA (see Section \@ref(pca)) where peaks of these two chemicals are both present in the two first PCs. The loading profile of the `Time` effect shows a high peak on the left side of the removed water area.

```{r ASCA_ScoresXY}
# Hippurate
hip_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate"
)

hip_loadings_pl <- all_loadings_pl$Hippurate + hip_peaks

grid.arrange(hip_scores_pl, hip_loadings_pl, ncol = 2)

# Citrate
cit_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Citrate",
    color = "Citrate", shape = "Citrate"
)
cit_loadings_pl <- all_loadings_pl$Citrate + cit_peaks

grid.arrange(cit_scores_pl, cit_loadings_pl, ncol = 2)

# Time
tim_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Time", color = "Time",
    shape = "Time"
)

time_peaks <- annotate("rect",
    xmin = c(5.955364), xmax = c(6.155364),
    ymin = -Inf, ymax = Inf, alpha = 0.2,
    fill = c("royalblue")
)

tim_loadings_pl <- all_loadings_pl$Time + time_peaks

grid.arrange(tim_scores_pl, tim_loadings_pl, ncol = 2)
```


#### Interaction `Hippurate:Time`

The scores and loadings fot the interaction term is represented here. It is not straighforward to interpret the scores plot of such an interaction term but the loadings of PC1 indicate a shift in the spectrum, along the whole spectral profile (but mostly around 3 ppm). 

```{r}
# Hippurate:Time
hiptim_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time"
)
hiptim_loadings_pl <- all_loadings_pl$`Hippurate:Time` +
    time_peaks +
    hip_peaks

grid.arrange(hiptim_scores_pl, hiptim_loadings_pl, ncol = 2)
```

#### Combination of effects `Hippurate+Time+Hippurate:Time`

The scores and the loadings of a combination of effects, in this case `"Hippurate+Time+Hippurate:Time"` can also be visualised.

```{r}
# Hippurate+Time+Hippurate:Time
hiptimInter_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Hippurate+Time+Hippurate:Time",
    color = "Hippurate", shape = "Time"
)

hiptimInter_loadings_pl <- all_loadings_pl$`Hippurate:Time` +
    time_peaks + hip_peaks

grid.arrange(hiptimInter_scores_pl, hiptimInter_loadings_pl, ncol = 2)
```

However, note that a better graphical representation is possible with the function `lmpEffectPlot` (see Section \@ref(effects-plot)) to depict interaction terms or combinations of effects.


#### Model residuals

PCA on the model residuals can also be applied to spot outliers or an unexpected underlying structure of the data. The scores plot shows that the effect of `Day`, which was excluded in our modeling step, could have been included as well as it spans the two first PCs of the residuals. 

```{r Plot_Residuals}
resid_scores_pl <- lmpScorePlot(resASCA,
    effectNames = "Residuals",
    color = "Day", shape = "Day",
    drawShapes = "segment"
)


resid_loadings_pl <- all_loadings_pl$Residuals

grid.arrange(resid_scores_pl, resid_loadings_pl, ncol = 2)
```


### Other graphs

#### Scores scatter plot

We can also represent the scores as a matrix of plots with `lmpScoreScatterPlotM`. This graph has the advantage to compare more than 2 variables simultaneously. For example, the PC1 scores of `Hippurate`  and `Citrate` clearly represent the orthogonal design of this experiment. The interaction `Hippurate:Time` can also be clearly represented when comparing les PC1s of `Hippurate` and the interaction term.

```{r ASCA_ScoresMatrix, fig.height=7}
lmpScoreScatterPlotM(resASCA,
    PCdim = c(1, 1, 1, 1, 1, 1, 1, 2),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "ASCA scores scatterplot matrix"
)
```

#### 2D Loadings

Finally the loadings can also be represented 2-by-2 as a scatterplot with `lmpLoading2dPlot`.  Such graphic is of course of limited interest for spectral data.  

```{r ASCA_loadings}
lmpLoading2dPlot(
    resLmpPcaEffects = resASCA,
    effectNames = c("Hippurate"),
    axes = c(1, 2),
    addRownames = TRUE, pl_n = 10
)

# adding manually labels to points for the Hippurate peaks
labels <- substr(colnames(UCH$outcomes), 1, 4)
labels[-c(369, 132, 150, 133, 149, 144, 145, 368, 151)] <- ""

lmpLoading2dPlot(
    resLmpPcaEffects = resASCA,
    effectNames = c("Hippurate"),
    axes = c(1, 2), points_labs = labels
)
```


### Effects plot {#effects-plot}

The `lmpEffectPlot` function is particularly interesting to visualise an interaction term or a combination of effects. Note that this function can only be applied with the `ASCA` method.


#### Main effects for `Hippurate`

Here the first PC of the `Hippurate` is represented along the levels of this effect showing an expected linear effect of Hippurate dose on the PC value.

```{r}
lmpEffectPlot(resASCA, effectName = "Hippurate", x = "Hippurate")
```


#### Interaction `Hippurate:Time`

A more interesting application is the visualization of interaction terms as line plots, here `Hippurate:Time` along the `Hippurate` or the `Time` effect. This representation gives the impression of a quite important interaction effect.

```{r}
lmpEffectPlot(resASCA,
    effectName = "Hippurate:Time",
    x = "Hippurate", z = "Time"
)
lmpEffectPlot(resASCA,
    effectName = "Hippurate:Time",
    x = "Time", z = "Hippurate"
)
```


#### Combination of effects `Hippurate+Time+Hippurate:Time`

An alternative visualisation of this interaction combines the main effects of `Hippurate`, `Time` and the interaction `Hippurate:Time`. Even if significant, the interaction effect is actually quite small compared to the main effects (`Hippurate` for PC1 and `Time` for PC2).

```{r ASCA_effects}
lmpEffectPlot(resASCA,
    effectName = "Hippurate+Time+Hippurate:Time",
    x = "Hippurate", z = "Time"
)
lmpEffectPlot(resASCA,
    effectName = "Hippurate+Time+Hippurate:Time",
    axes = c(2), x = "Time", z = "Hippurate"
)
```




## APCA

The APCA method performs PCA on the effect matrices augmented by the residuals. The same functions already used for ASCA can be applied.

```{r APCA_PCA}
resAPCA <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "APCA"
)
```

### Scores Plot

Different shapes with the `drawShapes` argument highlight the data structure recovery.

```{r APCA_ScoresXY}
# Hippurate main effect
lmpScorePlot(resAPCA,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate", drawShapes = "ellipse"
)

# Citrate main effect
lmpScorePlot(resAPCA,
    effectNames = "Citrate",
    color = "Citrate", shape = "Citrate", drawShapes = "ellipse"
)

# Time main effect
lmpScorePlot(resAPCA,
    effectNames = "Time",
    color = "Time", shape = "Time", drawShapes = "ellipse"
)
lmpScorePlot(resAPCA,
    effectNames = "Time",
    color = "Time", shape = "Time", drawShapes = "polygon"
)
lmpScorePlot(resAPCA,
    effectNames = "Time",
    color = "Time", shape = "Time", drawShapes = "segment"
)

# Interaction term
lmpScorePlot(resAPCA,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time", drawShapes = "segment"
)
lmpScorePlot(resAPCA,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time", drawShapes = "polygon"
)
```

A scatterplot matrix can also be applied to visualise the relationship between the scores profiles of interest.

```{r APCA_ScoresMatrix}
lmpScoreScatterPlotM(resAPCA,
    effectNames = c(
        "Hippurate", "Citrate", "Time",
        "Hippurate:Time"
    ),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "APCA scores scatterplot matrix"
)
```

### Loadings plot


Note that the APCA loadings contain the residuals of the model and differ from the ASCA loadings in that respect.

```{r APCA_loadings}
lmpLoading1dPlot(resAPCA, effectNames = c(
    "Hippurate", "Citrate",
    "Time", "Hippurate:Time"
), axes = 1)
```

## ASCA-E

The ASCA-E method performs PCA on the pure effect matrices then projects the effect matrices augmented with residuals on the PC obtained.  

```{r ASCAE_PCA}
resASCAE <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "ASCA-E"
)
```

The contributions and loadings are identical to the ASCA results.

### Scores Plot

For the main effects, all score plots show a clear separation between the different levels of the considered effects on the first PC. This separation of the groups suggests a strong effect of those factors, confirmed by their significance.

```{r ASCAE_ScoresXY}
lmpScorePlot(resASCAE,
    effectNames = "Hippurate",
    color = "Hippurate", shape = "Hippurate"
)
lmpScorePlot(resASCAE,
    effectNames = "Citrate",
    color = "Citrate", shape = "Citrate"
)
lmpScorePlot(resASCAE,
    effectNames = "Time",
    color = "Time", shape = "Time"
)
lmpScorePlot(resASCAE,
    effectNames = "Hippurate:Time",
    color = "Hippurate", shape = "Time"
)
```

Again, the scores profiles can be compared 2 by 2 with ASCA-E.
```{r ASCAE_ScoresMatrix}
lmpScoreScatterPlotM(resASCAE,
    effectNames = c(
        "Hippurate", "Citrate", "Time",
        "Hippurate:Time"
    ),
    modelAbbrev = TRUE,
    varname.colorup = "Citrate",
    varname.colordown = "Time",
    varname.pchup = "Hippurate",
    varname.pchdown = "Time",
    title = "ASCA-E scores scatterplot matrix"
)
```


# sessionInfo
```{r}
sessionInfo()
```

# References

---
title: "Application of `limpca` on the Trout transcriptomic dataset."
subtitle: "Balanced crossed three way design for fixed factors"
author: "Benaiche Nadia, Bernadette Govaerts"
date: '`r format(Sys.time(), "%B %d, %Y")`'
package: limpca
output:
  BiocStyle::html_document:
    toc: yes
    toc_depth: 4
    toc_float: yes
vignette: >
   %\VignetteIndexEntry{Analysis of the Trout dataset with limpca}
   %\VignetteEncoding{UTF-8}
   %\VignetteEngine{knitr::rmarkdown}
---

```{r setup,include=FALSE}
# Install package if necessary (to be used before running the vignette)
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("limpca")

library(ggplot2)
library(limpca)
if (!requireNamespace("gridExtra", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(gridExtra)

if (!requireNamespace("pander", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(pander)
if (!requireNamespace("car", quietly = TRUE)) {
    stop("Install 'pander' to knit this vignette")
}
library(car)


knitr::opts_chunk$set(
    message = FALSE, warning = TRUE,
    comment = NA, crop = NULL,
    width = 60, dpi = 50,
    fig.width = 5,
    fig.height = 3.5, dev = "png"
)
```

Package loading

```{r, eval=FALSE}
library(car)
library(pander)
library(gridExtra)
library(ggplot2)
```


# Installation and loading of the `limpca` package

`limpca` can be installed from Bioconductor:

```{r Install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("limpca")
```

And then loaded into your R session: 

```{r Load, eval=FALSE}
library("limpca")
```


# Data and model presentation

This data set comes from the study of the modulation of immunity in rainbow trout (Oncorhynchus mykiss) by exposure to cadmium (Cd) combined with polyunsaturated fatty acids (PUFAs) enriched diets [Cornet et al., 2018].

The responses were quantified by measuring the modification of the expression of 15 immune-related genes (m = 15) by RT-qPCR (reverse transcription quantitative polymerase chain reaction). The experiment was carried out on 72 trouts and 3 factors were considered in the experimental design:

-   **Day**: Measurements on trouts were collected on days 28, 70 and 72
-   **Treatment** : Four polyunsaturated fatty acid diets: alpha-linolenic acid (ALA), linoleic acid (LA), eicosapentaenoic acid (EPA) and docosahexaenoic acid (DHA)
-   **Exposure**: Trouts were exposed (level = 2) or not (level = 0) to high cadmium concentrations.

This gives a 3 × 4 × 2 factorial design. Each of the 24 trials corresponds to a different aquarium. Three fishes were analysed (3 replicates) for each condition, giving a total of 72 observations.

In this `limpca` vignette, the data are first explored in order to prepare an appropriate data set for ASCA/APCA analysis. Data are first represented by PCA and outliers removed. The remaining observations are then log transformed. Next, the data of each aquarium are mean aggregated in order to avoid the inclusion of an aquarium random factor in the statistical model because limpca is not yet able to handle mixed linear models. The data are finally centered and scaled by column.

The estimated model in then a (general) linear model for fixed factors including main effects and all two way interactions. The three way interaction is not included because the aggregated design has no replicate.     

A detailed presentation and analysis of this dataset is also available in [Benaiche, 2022].

# Data import and exploration

## Data import and design visualization

The `trout` data set is list of three objects : the model `outcomes`, the `design` and the model `formula`. 

```{r Data Importation}
data("trout")
# print number of and response names
cat("\n Nb of Responses :  ", ncol(trout$outcomes), "\n ")
cat("\nResponses :\n", colnames(trout$outcomes), "\n ")
# Order responses by alphabetic order
trout$outcomes <- trout$outcomes[, order(dimnames(trout$outcomes)[[2]])]
cat("\n Ordered responses :\n  ", colnames(trout$outcomes), "\n ")
# print factor names
cat("\nDesign factors :  ", colnames(trout$design), "\n ")
# plot the design with plotDesign function
limpca::plotDesign(
    design = trout$design, x = "Treatment",
    y = "Day", cols = "Exposure",
    title = "Initial design of the trout dataset"
)
```

This graph confirms that the design is balanced.

## Principal Component Analysis of row data

Functions `pcaBySvd`, `pcaScreePlot` and `pcaScorePlot` are used to take a first look at the data by PCA. 

This PCA shows that the data should be log transformed.

```{r Exploratory PCA}
resPCA <- limpca::pcaBySvd(trout$outcomes)
limpca::pcaScreePlot(resPCA, nPC = 8)
limpca::pcaScorePlot(
    resPcaBySvd = resPCA, axes = c(1, 2),
    title = "Score plot of original data ",
    design = trout$design, color = "Aquarium",
    points_labs_rn = TRUE
)
```

## Log10 transformation of the data and new PCA

Data are log transfomed and a new PCA is applied.  

The score plot show clearly two outliers (i.e. fishes D72EPA2.1 and D28EPA2.2).  They will removed of the analysis for the next steps.

```{r Transformation}
# Log Transformation
trout_log <- trout
trout_log$outcomes <- as.matrix(log10(trout$outcomes))

# new PCA
resPCA1 <- limpca::pcaBySvd(trout_log$outcomes)
limpca::pcaScreePlot(resPCA1, nPC = 8)
limpca::pcaScorePlot(
    resPcaBySvd = resPCA1, axes = c(1, 2),
    title = "Score plot of Log10 data ",
    design = trout_log$design, color = "Aquarium",
    drawShapes = "polygon", points_labs_rn = TRUE
)
```

## New PCA without the outliers 

New data are created without the 2 outliers and a new PCA performed.  The option `polygon` of `pcaScorePlot` function allows to group data by aquarium. In some aquariums, the 3 fishes shows very similar results and in other not.

The data are now clean for further analysis

```{r Remove Outliers}
# Remove outliers and create new dataset
trout_clean <- trout_log
outliers <- match(
    c("D72EPA2.1", "D28EPA2.2"),
    rownames(trout_log$outcomes)
)
trout_clean$outcomes <- trout_log$outcomes[-outliers, ]
trout_clean$design <- trout_log$design[-outliers, ]
```

```{r new PCA}
# PCA
resPCA2 <- limpca::pcaBySvd(trout_clean$outcomes)
limpca::pcaScreePlot(resPCA2, nPC = 8)
# Score plot Components 1 and 2
limpca::pcaScorePlot(
    resPcaBySvd = resPCA2, axes = c(1, 2),
    title = "Score plot of Log10 data without outliers (PC 1&2)",
    design = trout_clean$design, color = "Aquarium",
    drawShapes = "polygon",
    points_labs_rn = FALSE
)
# Score plot Components 3 and 4
limpca::pcaScorePlot(
    resPcaBySvd = resPCA2, axes = c(3, 4),
    title = "Score plot of Log10 data without outliers (PC 3&4)",
    design = trout_clean$design, color = "Aquarium",
    drawShapes = "polygon", points_labs_rn = FALSE
)
```

The option `polygon` of `pcaScorePlot` function allows to group data by aquarium. In some aquariums, the 3 fishes shows very similar results and in other not.

## Mean agregation by aquarium and scaling

Data are now mean aggregated by aquarium. This will remove the hierarchy in the design and allow to apply a classical fixed effect general linear model to the data.

Data are next centered and scaled by column. This will give the same importance to each response in the analysis.

```{r Mean aggregation}
# Mean aggregation
mean_outcomes <- matrix(0, nrow = 24, ncol = 15)
mean_design <- matrix(0, nrow = 24, ncol = 3)
y <- list(
    trout_clean$design[["Day"]],
    trout_clean$design[["Treatment"]],
    trout_clean$design[["Exposure"]]
)
for (i in 1:15) {
    mean_outcomes[, i] <- aggregate(trout_clean$outcomes[, i], by = y, mean)[, 4]
}
mean_design <- aggregate(trout_clean$outcomes[, 1], by = y, mean)[, c(1:3)]

# Set row and col names
colnames(mean_outcomes) <- colnames(trout_clean$outcomes)
colnames(mean_design) <- colnames(trout_clean$design)[1:3]
trout_mean_names <- apply(mean_design, 1, paste, collapse = "")
rownames(mean_outcomes) <- trout_mean_names
rownames(mean_design) <- trout_mean_names
```

```{r Centering and Scaling}
# Outcomes centering and Scaling
mean_outcomes <- scale(mean_outcomes, center = TRUE, scale = TRUE)
# New data object creation
trout_mean <- list(
    "outcomes" = mean_outcomes,
    "design" = mean_design,
    "formula" = trout$formula
)
# Clean objects
rm(
    resPCA, resPCA1, resPCA2, y, mean_design, mean_outcomes,
    trout_mean_names
)
```

# Exploration of aggregated data

Aggregated data are now explored. 

## Design

Note that there is no replicate in this new design. There is only one observation (i.e. one aquarium) for each of the 24 factor combinations.

```{r Design}
pander(head(trout_mean$design))
limpca::plotDesign(
    design = trout_mean$design,
    title = "Design of mean aggregated trout dataset"
)
```

## Example of `lineplot` of the responses for two observations

```{r Lineplot}
limpca::plotLine(trout_mean$outcomes,
    rows = c(1, 24),
    xaxis_type = "character", type = "s"
) +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))
```

## PCA aggregated data

```{r PCA}
resPCA_mean <- limpca::pcaBySvd(trout_mean$outcomes)
pcaScreePlot(resPCA_mean, nPC = 6)
```

### Score plots 

The three score plots below show clearly that the Day is the more important effect. 

```{r}
limpca::pcaScorePlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA score plot by Exposure and Day",
    design = trout_mean$design,
    shape = "Exposure", color = "Day",
    points_labs_rn = FALSE
)

limpca::pcaScorePlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA scores plot by Treatment and Day",
    design = trout_mean$design,
    shape = "Day", color = "Treatment",
    points_labs_rn = FALSE
)

limpca::pcaScorePlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA scores plot by Exposure and Treatment",
    design = trout_mean$design,
    shape = "Treatment", color = "Exposure",
    points_labs_rn = FALSE
)
```

### 1D Loading plots

```{r}
limpca::pcaLoading1dPlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA loadings plot trout", xlab = " ",
    ylab = "Expression", xaxis_type = "character", type = "s"
) +
    ggplot2::theme(axis.text.x = element_text(angle = 45, hjust = 1))
```

### 2D Loading plots

This 2D loading plot allows already to observe that most  responses are correlated with each others.  Igm, SOD and TLR9 behave quite differently.  

```{r}
limpca::pcaLoading2dPlot(
    resPcaBySvd = resPCA_mean, axes = c(1, 2),
    title = "PCA loadings plot trout", addRownames = TRUE
)
```


## Scatterplot matrix of all 15 responses

The `plotScatterM` function allows to visualize the 2 by 2 relation between all (or some of) the responses simultaneously and choose different markers and colors above and below the diagonal according to factor levels.  Strong relations between expressions are confirmed here for most genes.  

```{r Scatterplot Matrix,fig.width=10,fig.height=10}
limpca::plotScatterM(
    Y = trout_mean$outcomes, cols = c(1:15),
    design = trout_mean$design,
    varname.colorup = "Day",
    vec.colorup = c("CadetBlue4", "pink", "orange"),
    varname.colordown = "Day",
    vec.colordown = c("CadetBlue4", "pink", "orange"),
    varname.pchup = "Treatment",
    varname.pchdown = "Exposure"
)
```

# GLM decomposition

The estimated model is the following :

`outcomes ~ Day + Treatment + Exposure + Day:Treatment + Day:Exposure + Treatment:Exposure`

Since the design has only one replicate, the three way interaction has been removed because it is confounded with residuals.

## Model matrix X generation

```{r ModelMatrix}
resLmpModelMatrix <- limpca::lmpModelMatrix(trout_mean)
pander::pander(head(resLmpModelMatrix$modelMatrix))
```

## Computation of effect matrices and importances

As observed before, the more important effect in the model is the Day main effect.

```{r EffectMatrices}
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resLmpEffectMatrices$varPercentagesPlot
```

## Bootstrap test of effect significance

The bootstrap test shows that, in addition to the Day effect, the Treatment effect is also significant (p\<0.05) and the DxT effect is nearly significant (p close to 0.1).  The corresponding effect matrices will be studied more deeply by PCA in the next sections. 

```{r Bootstrap}
resLmpBootstrapTests <- lmpBootstrapTests(
    resLmpEffectMatrices = resLmpEffectMatrices,
    nboot = 1000
)

# Print p-values
pander::pander(t(resLmpBootstrapTests$resultsTable))
```

# ASCA and APCA

Visualization of single or combined effect matrices using ASCA and APCA. ASCA-E is also provided in `limpca` but not shown here.

## ASCA

### PCA decomposition of effect matrices

In addition to single model effects, a combined effect matrix `Day+Treatment+Day:Treatment` also is computed in order to visualize the combined effect of these two most important factors.

```{r ASCA PCA}
resASCA <- lmpPcaEffects(
    resLmpEffectMatrices = resLmpEffectMatrices,
    method = "ASCA",
    combineEffects = list(c(
        "Day", "Treatment",
        "Day:Treatment"
    ))
)
```

### Contributions

Print contributions of each model effect and of each PC for each  effect matrix decomposition by PCA.  This last result is given effect by effect and then reported to the global variance.   

```{r ASCA Contrib}
resLmpContributions <- lmpContributions(resASCA)
```

```{r ASCA ContribP}
pander::pander(resLmpContributions$totalContribTable)
pander::pander(resLmpContributions$effectTable)
pander::pander(resLmpContributions$contribTable)
pander::pander(resLmpContributions$combinedEffectTable)

## Visualize the more important contributions
resLmpContributions$plotContrib
```

### Scores and loadings plots

2D Score plots of the most important effects and of the residual matrix are given below with their related loading plots. These allow to see for which response each effect is or is not important

#### Day effect

```{r ASCA Day}
A <- lmpScorePlot(resASCA,
    effectNames = "Day",
    color = "Day", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Day",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)
```

#### Treatment effect

```{r ASCA Treatment}
A <- lmpScorePlot(resASCA,
    effectNames = "Treatment",
    color = "Treatment", shape = "Treatment"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Treatment",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)
```

#### Day:Treatment effect

```{r ASCA DayTreatment}
A <- lmpScorePlot(resASCA,
    effectNames = "Day:Treatment",
    color = "Treatment", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Day:Treatment",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)
```

#### Combined Day+Treatment+Day:Treatment effect

```{r ASCA Combined effect}
A <- lmpScorePlot(resASCA,
    effectNames = "Day+Treatment+Day:Treatment",
    color = "Treatment", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Day+Treatment+Day:Treatment",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)
```

#### Residual matrix decomposition

The score and loading plot of the residual matrix does not show any special pattern.

```{r ASCA Residuals}
A <- lmpScorePlot(resASCA,
    effectNames = "Residuals",
    color = "Treatment", shape = "Day"
)
B <- lmpLoading2dPlot(resASCA,
    effectNames = "Residuals",
    points_labs = colnames(trout$outcomes)
)
grid.arrange(A, B, ncol = 2)
```

### Effect plots on scores

Interaction effects are difficult to visualize in 2D score plots. Effect plots are interesting in this context. It show the effect of one factor on a PC for different level of the other.

Below the Day:Treament interaction effect is drawn alone and then combined with the two related main effects.

The second graph shows that their is some interaction effect but small compared to the Day and even treatment main effects.

```{r ASCA Effects}
A <- lmpEffectPlot(resASCA,
    effectName = "Day:Treatment",
    x = "Day", z = "Treatment", axes = c(1, 2)
)
A$PC1 <- A$PC1 + ggtitle("PC1: Day:Treatment effect alone")
A$PC2 <- A$PC2 + ggtitle("PC2: Day:Treatment effect alone")
grid.arrange(A$PC1, A$PC2, ncol = 2)

A <- lmpEffectPlot(resASCA,
    effectName = "Day+Treatment+Day:Treatment",
    x = "Day", z = "Treatment", axes = c(1, 2)
)
A$PC1 <- A$PC1 + ggtitle("PC1: Combined D+T+D:T effects")
A$PC2 <- A$PC2 + ggtitle("PC2: Combined D+T+D:T effects")
grid.arrange(A$PC1, A$PC2, ncol = 2)
```

## APCA

APCA allows to visualize by PCA each model effect added to model residuals. It gives an idea of the effects signal to noise ratio and significance. Be care that the significance depends also crucially of the number of observations in the experiment.

In APCA the score plots are the more interesting graphics to look at. We then only give them here for all model effects.

```{r APCA PCA}
resAPCA <- lmpPcaEffects(
    resLmpEffectMatrices =
        resLmpEffectMatrices, method = "APCA"
)
```

```{r APCA Scores}
# Day Effect
lmpScorePlot(resAPCA,
    effectNames = "Day",
    color = "Day", shape = "Day", drawShapes = "ellipse"
)

# Treatment Effect
lmpScorePlot(resAPCA,
    effectNames = "Treatment",
    color = "Treatment", shape = "Treatment", drawShapes = "ellipse"
)


# Exposure Effect
lmpScorePlot(resAPCA,
    effectNames = "Exposure",
    color = "Exposure", shape = "Exposure", drawShapes = "ellipse"
)

# Day:Treatment Effect
lmpScorePlot(resAPCA,
    effectNames = "Day:Treatment",
    color = "Treatment", shape = "Day", drawShapes = "polygon"
)
```

# Univariate ANOVA

This part of the vignette links the results of the ASCA/APCA analysis to a more classical analysis applied in -omics transcriptomic data analysis (see e.g. `limma` package).

An ANOVA model is fitted to each response separately and then, for each model effect, p-values of effect significance are corrected by FDR and ordered. These results allow to detect for which responses each effect of interest is significant (e.g. find which gene is a potential biomarker to differentiate patients with or without a given disease).

The code below applies such analysis and compares then responses importance based on FDR p-values to response loadings obtained by ASCA/APCA.

## Parallel ANOVA modeling and FDR p-value corrections

```{r Anova}
# Creation of a matrix to store the p-values
m <- ncol(trout_mean$outcomes)
mat_pval <- matrix(nrow = m, ncol = 6)
dimnames(mat_pval) <- list(
    dimnames(trout_mean$outcomes)[[2]],
    c(
        "Day", "Treatment", "Exposure", "Day:Treatment",
        "Day:Exposure", "Treatment:Exposure"
    )
)

# Parallel ANOVA modeling
for (i in 1:m) {
    data <- cbind(y = trout_mean$outcomes[, i], trout_mean$design)
    Modl <- lm(y ~ Day + Treatment + Exposure + Day:Treatment + Day:Exposure + Treatment:Exposure,
        contrasts = list(Day = contr.sum, Treatment = contr.sum, Exposure = contr.sum),
        data = data
    )
    tabanova <- Anova(Modl, type = 3)
    mat_pval[i, ] <- tabanova[2:7, 4]
}

# FDR p-values correction
for (i in 1:6) mat_pval[, i] <- p.adjust(mat_pval[, i], method = "BH")
```

## FDR corrected p_values (q-values)

```{r}
pander(mat_pval)
heatmap(-log10(mat_pval),
    Rowv = NA, Colv = "Rowv",
    cexCol = 0.8, scale = "none", main = "Heatmap of -log10(q-values)"
)
```

## Plot ASCA loadings versus -log10(q-values)

Plot the relation between the ASCA loadings and FDR p-values for the three more important effects of the model : Day, Treatment and Day:Treatment.

FDR p-values are log transformed and loadings are summarized over the 2 first components.

```{r, eval=TRUE}
Effects <- c("Day", "Treatment", "Day:Treatment")

for (i in 1:3) {
    Pval_log <- -log10(mat_pval[, Effects[i]])
    resA <- resASCA[[Effects[i]]]
    PC12Load <- as.vector(sqrt(resA$loadings[, 1:2]^2 %*% resA$singvar[1:2]^2))
    matres <- cbind(PC12Load, Pval_log)
    A[[i]] <- plotScatter(
        Y = matres, xy = c(1, 2),
        points_labs = rownames(matres),
        xlab = "PC1&2 ASCA loadings", ylab = "-log10(p-values)",
        title = paste(Effects[i], "effect")
    )
}

A[[1]]
A[[2]]
A[[3]]
```

# sessionInfo

```{r}
sessionInfo()
```

# References

Benaiche, N., (2022), Stabilisation of the R package LMWiRe -- Linear Models for Wide Responses. *UCLouvain*, <http://hdl.handle.net/2078.1/thesis:33996>

Cornet, V., Ouaach, A., Mandiki, S., Flamion, E., Ferain, A., Van Larebeke, M., Lemaire, B., Reyes Lopez F., Tort, L., Larondelle, Y. and Kestemont, P., (2018), Environmentally-realistic concentration of cadmium combined with polyunsaturated fatty acids enriched diets modulated non-specific immunity in rainbow trout. *Aquatic Toxicology*, **196**, 104--116. <https://doi.org/10.1016/j.aquatox.2018.01.012>

Thiel, M., Feraud, B. and Govaerts, B. (2017), ASCA+ and APCA+: exten- sions of ASCA and APCA in the analysis of unbalanced multifactorial designs. *Journal of Chemometrics*. **31** <https://doi.org/10.1002/cem.2895>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpPcaEffects.R
\name{lmpPcaEffects}
\alias{lmpPcaEffects}
\title{PCA on the effect matrices}
\usage{
lmpPcaEffects(
  resLmpEffectMatrices,
  method = c("ASCA", "APCA", "ASCA-E"),
  combineEffects = NULL,
  verbose = FALSE
)
}
\arguments{
\item{resLmpEffectMatrices}{A resLmpEffectMatrices list resulting of \code{\link{lmpEffectMatrices}}.}

\item{method}{The method used to compute the PCA. One of \code{c("ASCA","APCA","ASCA-E")}.}

\item{combineEffects}{If not \code{NULL}, a list of vectors containing the names of the effects to be combined.}

\item{verbose}{If \code{TRUE}, will display a message with the duration of execution.}
}
\value{
A list with first,the PCA results from \code{\link{pcaBySvd}} for each effect matrix. Those results contain :
\describe{
\item{\code{scores}}{Scores from the PCA for each principal component.}
\item{\code{loadings}}{Loadings from the PCA for each principal component.}
\item{\code{eigval}}{Eigenvalues of each principal component.}
\item{\code{singvar}}{Singular values of each principal component.}
\item{\code{var}}{Explained variances of each principal component.}
\item{\code{cumvar}}{Cumulated explained variances of each principal component.}
\item{\code{original.dataset}}{Original dataset.}
}

There are also others outputs :
\describe{
\item{\code{lmpDataList}}{The initial object: a list of outcomes, design and formula.}
\item{\code{effectsNamesUnique}}{A character vector with the \emph{F+1} names of the model terms, each repeated once.}
\item{\code{method}}{The dimension reduction method used: \code{c("ASCA","APCA","ASCA-E")}.}
\item{\code{type3SS}}{A vector with the type III SS for each model term.}
\item{\code{variationPercentages}}{A vector with the percentage of variance explained by each model term.}
}
}
\description{
Performs a PCA on each of the effect matrices from the outputs of \code{\link{lmpEffectMatrices}}. It has an option to choose the method applied: ASCA, APCA or ASCA-E. Combined effects (i.e. linear combinations of original effect matrices) can also be created and decomposed by PCA.
}
\details{
The function allows 3 different methods :

\describe{
\item{ASCA}{PCA is applied directly on each pure effect matrix \eqn{\hat{\mathbf{M}}_f, f=1...F}.}
\item{ASCA-E}{PCA is applied on each pure effect matrix but then the augmented effect matrix is projected in the space of the ASCA components.}
\item{APCA}{PCA is applied on each augmented effect matrix : \eqn{\hat{\mathbf{M}}_f+\hat{\mathbf{E}}}.}
}
}
\examples{
data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resLmpPcaEffects <- lmpPcaEffects(resLmpEffectMatrices, method = "ASCA-E")

}
\references{
Thiel M.,Feraud B. and Govaerts B. (2017) ASCA+ and APCA+: Extensions of ASCA and APCA
in the analysis of unbalanced multifactorial designs. \emph{Journal of Chemometrics}. 31:e2895.
\url{https://doi.org/10.1002/cem.2895}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data2lmpDataList.R
\name{data2LmpDataList}
\alias{data2LmpDataList}
\title{Converts data to a lmpDataList.}
\usage{
data2LmpDataList(
  se = NULL,
  assay_name = NULL,
  outcomes = NULL,
  design = NULL,
  formula = NULL,
  verbose = TRUE
)
}
\arguments{
\item{se}{A \code{\link[SummarizedExperiment]{SummarizedExperiment}} object.}

\item{assay_name}{If not \code{NULL} (default), a character string naming the assay from the \code{\link[SummarizedExperiment]{SummarizedExperiment}} object \code{se}. If \code{NULL}, the first assay is selected.}

\item{outcomes}{If not \code{NULL} (default), a numerical matrix with \emph{n} observations and \emph{m} response variables. The rownames needs to be non-NULL and match those of the design matrix.}

\item{design}{If not \code{NULL} (default), a data.frame with the experimental design of \emph{n} observations and \emph{q} explanatory variables. The rownames of design has to match the rownames of outcomes.}

\item{formula}{If not \code{NULL} (default), a character string with the formula that will be used to analyze the data. Only the right part of the formula is necessary, eg: \code{"~ A + B"}, The names of the formula should match the column names of the design}

\item{verbose}{If \code{TRUE}, prints useful information about the outputted list.}
}
\value{
A list with the 3 following named elements:
\describe{
\item{\code{outcomes}}{A \emph{nxm} matrix with the \emph{m} response variables.}
\item{\code{design}}{A \emph{nxq} data.frame with the experimental design.}
\item{\code{formula}}{A character string with the model formula.}
}
}
\description{
Creates the lmpDataList from a SummarizedExperiment or by manually defining the design, the outcomes and the model formula.
lmpDataList serves as an input for the \code{\link{lmpModelMatrix}} function to start the limpca modeling.
}
\details{
Data can be included as a \code{\link[SummarizedExperiment]{SummarizedExperiment}} (SE) object or by manually defining one or multiple
elements of \code{outcomes}, \code{design} and \code{formula}. If a SE is provided,
the \code{outcomes} corresponds to a transposed assay of the SE (by default the first one),
the \code{design} corresponds to the \code{\link[SummarizedExperiment]{colData}} of the SE and the \code{formula} can be provided as a
\code{formula} element in the \code{S4Vectors::metadata} of SE (\code{metadata(se)$formula}).

In the outputted list, the outcomes are structured in a standard statistical fashion,
i.e. with observations in rows and the variables (features) in column.
If the \code{outcomes} argument is not \code{NULL}, it has to be formatted that way (see Arguments).

Note that there is a priority to the \code{outcomes}, \code{design} and \code{formula}
arguments if they are not \code{NULL} (e.g. if both \code{se} and \code{outcomes} arguments are provided,
the resulting outcomes matrix will be from the \code{outcomes} argument). \code{outcomes} and \code{design} elements are mandatory.

Multiple checks are performed to ensure that the data are correctly formatted:
\itemize{
\item the rownames of \code{design} and \code{outcomes} should match
\item the names of the model terms in the \code{formula} should match column names from the \code{design}
}
}
\examples{

data(UCH)

### create manually the dataset

res <- data2LmpDataList(
  outcomes = UCH$outcomes,
  design = UCH$design[, 1, drop = FALSE], formula = "~ Hippurate"
)

### create the dataset from a SummarizedExperiment

library(SummarizedExperiment)

se <- SummarizedExperiment(
  assays = list(
    counts = t(UCH$outcomes),
    counts2 = t(UCH$outcomes * 2)
  ), colData = UCH$design,
  metadata = list(formula = "~ Hippurate + Citrate")
)

res <- data2LmpDataList(se, assay_name = "counts2")

# changing the formula:
res <- data2LmpDataList(se,
  assay_name = "counts2",
  formula = "~ Hippurate + Citrate + Time"
)

}
\seealso{
\code{\link[SummarizedExperiment]{SummarizedExperiment}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaLoading2dPlot.R
\name{pcaLoading2dPlot}
\alias{pcaLoading2dPlot}
\title{Loading plots on a 2D scatter plot}
\usage{
pcaLoading2dPlot(
  resPcaBySvd,
  axes = c(1, 2),
  title = "PCA loading plot",
  addRownames = FALSE,
  pl_n = 10,
  metadata = NULL,
  drawOrigin = TRUE,
  ...
)
}
\arguments{
\item{resPcaBySvd}{A list corresponding to the output value of \code{\link{pcaBySvd}}.}

\item{axes}{A numerical vector of length 2 with the Principal Components axes to be drawn.}

\item{title}{Plot title.}

\item{addRownames}{Boolean indicating if the labels should be plotted. By default, uses the row names of the loadings matrix but it can be manually specified with the \code{points_labs} argument from \code{\link{plotScatter}}.}

\item{pl_n}{The number of labels that should be plotted, based on a distance measure (see Details).}

\item{metadata}{A \eqn{n \times k} "freely encoded" data.frame corresponding to the \code{design} argument in \code{\link{plotScatter}}.}

\item{drawOrigin}{if \code{TRUE}, draws horizontal and vertical intercepts at (0,0) based on the \code{\link{plotScatter}} function.}

\item{...}{Additional arguments to be passed to \code{\link{plotScatter}}.}
}
\value{
A \code{ggplot2} object with the PCA loading plot.
}
\description{
Produces 2D loading plots from \code{\link{pcaBySvd}} with the same graphical options as \code{\link{plotScatter}} as this is a wrapper of this function.
}
\details{
\code{pcaLoading2dPlot} is a wrapper of \code{\link{plotScatter}}. See \code{?plotScatter} for more information on the additional arguments.

The distance measure \eqn{d}{d} that is used to rank the variables is based on the following formula:
\deqn{d = \sqrt(P_{ab}^2*\lambda_{ab}^2)}{d = sqrt(P_ab^2 * lambda_ab^2)} where \eqn{a}{a}
and \eqn{b}{b} are two selected Principal Components, \eqn{P_{ab}}{P_ab} represents their
loadings and \eqn{\lambda_{ab}}{lambda_ab} their singular values.
}
\examples{

data("UCH")
ResPCA <- pcaBySvd(UCH$outcomes)

pcaLoading2dPlot(
  resPcaBySvd = ResPCA, axes = c(1, 2),
  title = "PCA loading plot UCH"
)

# adding color,  shape and labels to points
id_cit <- seq(446, 459)
id_hip <- c(seq(126, 156), seq(362, 375))
peaks <- rep("other", ncol(UCH$outcomes))
peaks[id_hip] <- "hip"
peaks[id_cit] <- "cit"
metadata <- data.frame(peaks)

pcaLoading2dPlot(
  resPcaBySvd = ResPCA, axes = c(1, 2),
  title = "PCA loading plot UCH", metadata = metadata,
  color = "peaks", shape = "peaks", addRownames = TRUE
)

# changing max.overlaps of ggrepel
options(ggrepel.max.overlaps = 30)
pcaLoading2dPlot(
  resPcaBySvd = ResPCA, axes = c(1, 2),
  title = "PCA loading plot UCH", metadata = metadata,
  color = "peaks", shape = "peaks", addRownames = TRUE,
  pl_n = 35
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotScatterM.R
\name{plotScatterM}
\alias{plotScatterM}
\title{Scatter plot matrix}
\usage{
plotScatterM(
  Y = NULL,
  design = NULL,
  lmpDataList = NULL,
  cols,
  labelVector = NULL,
  title = "Scatterplot matrix",
  varname.colorup = NULL,
  varname.colordown = NULL,
  varname.pchup = NULL,
  varname.pchdown = NULL,
  vec.colorup = NULL,
  vec.colordown = NULL,
  vec.pchup = NULL,
  vec.pchdown = NULL
)
}
\arguments{
\item{Y}{\eqn{n \times m} matrix with \eqn{n} observations and \eqn{m} variables. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{design}{A \eqn{n \times k} "freely encoded" experimental design data.frame. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{lmpDataList}{If not \code{NULL}, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}

\item{cols}{A vector with either the column names of the \eqn{Y} matrix to plot (character) or the column index positions.}

\item{labelVector}{Labels to display on the diagonal. If \code{NULL}, the column names are deduced from \code{cols}.}

\item{title}{Title of the graph.}

\item{varname.colorup}{A character string with the name of the variable used to color the upper triangle.}

\item{varname.colordown}{A character string with the name of the variable used to color the lower triangle.}

\item{varname.pchup}{A character string with the name of the variable used to mark points for the upper triangle.}

\item{varname.pchdown}{A character string with the name of the variable used to mark points for the lower triangle.}

\item{vec.colorup}{A color vector (character or numeric) with a length equal to the number of levels of \code{varname.colorup}.}

\item{vec.colordown}{A color vector (character or numeric) with a length equal to the number of levels of \code{varname.colordown}.}

\item{vec.pchup}{A symbol vector (character or numeric) with a length equal to the number of levels of \code{varname.pchup}.}

\item{vec.pchdown}{A symbol vector (character or numeric) with a length equal to the number of levels of \code{varname.pchdown}.}
}
\value{
A matrix of scatter plots.
}
\description{
Produces a scatter plot matrix between the selected columns of the outcomes matrix \eqn{Y} choosing specific colors and symbols for up to four factors from the design on the upper and lower diagonals.
}
\details{
Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
The same rule applies for \code{design} or \code{lmpDataList}.
}
\examples{

data("UCH")

# basic usage
plotScatterM(
  Y = UCH$outcomes, design = UCH$design, cols = c(1:4)
)

# equivalent to:
plotScatterM(
  lmpDataList = UCH, cols = c(1:4)
)

# with optionnal arguments
plotScatterM(
  Y = UCH$outcomes, design = UCH$design, cols = c(1:4),
  varname.colorup = "Hippurate", varname.colordown = "Citrate",
  varname.pchup = "Time", varname.pchdown = "Day",
  vec.colorup = c(2, 4, 1),
  vec.colordown = c("orange", "purple", "green"),
  vec.pchup = c(1, 2), vec.pchdown = c("a", "b")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpEffectMatrices.R
\name{lmpEffectMatrices}
\alias{lmpEffectMatrices}
\title{Computes the effect matrices}
\usage{
lmpEffectMatrices(resLmpModelMatrix, SS = TRUE, contrastList = NA)
}
\arguments{
\item{resLmpModelMatrix}{A list of 5 elements from \code{\link{lmpModelMatrix}}.}

\item{SS}{Logical. If \code{FALSE}, won't compute the percentage of variance for each effect.}

\item{contrastList}{A list of contrasts for each parameter. If \code{NA}, the function creates automatically the list by default.}
}
\value{
A list with the following elements:
\describe{
\item{\code{lmpDataList}}{The initial object: a list with outcomes, design and formula.}
\item{\code{modelMatrix}}{A \emph{nxp} model matrix specifically encoded for the ASCA-GLM method.}
\item{\code{modelMatrixByEffect}}{A list of \emph{F+1} model matrices for each effect.}
\item{\code{effectsNamesUnique}}{A character vector with the \emph{F+1} names of the model effects, each repeated once.}
\item{\code{effectsNamesAll}}{A character vector with the \emph{p} names of the model effects ordered and repeated as the column names of the model matrix.}
\item{\code{effectMatrices}}{A list of \emph{F+1} effect matrices for each model effect.}
\item{\code{predictedvalues}}{The \emph{nxm} matrix of predicted outcome values.}
\item{\code{residuals}}{The \emph{nxm} matrix of model residuals.}
\item{\code{parameters}}{The \emph{pxm} matrix of the estimated parameters.}
\item{\code{type3SS}}{A vector with the type III sum of squares for each model effect \emph{(If SS = TRUE)}.}
\item{\code{variationPercentages}}{A vector with the percentage of variance for each model effect \emph{(If SS = TRUE)}.}
\item{\code{varPercentagesPlot}}{A ggplot bar plot of the contributions of each model effect to the total variance \emph{(If SS = TRUE)}.}
}
}
\description{
Estimates the model by OLS based on the outcomes and model matrices provided in the outputs of lmpModelMatrix function and calculates the estimated effect matrices
\eqn{\hat{\mathbf{M}}_0, \hat{\mathbf{M}}_1, ...\hat{\mathbf{M}}_F}, ...  and residual matrix \eqn{\hat{\mathbf{E}}}.
It calculates also the type III percentage of variance explained by each effect.
}
\examples{
data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
reslmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
reslmpEffectMatrices$varPercentagesPlot

}
\references{
Thiel M.,Feraud B. and Govaerts B. (2017) \emph{ASCA+ and APCA+: Extensions of ASCA and APCA
in the analysis of unbalanced multifactorial designs}, Journal of Chemometrics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaBySvd.R
\name{pcaBySvd}
\alias{pcaBySvd}
\title{Singular Value Decomposition for PCA analysis}
\usage{
pcaBySvd(Y = NULL, lmpDataList = NULL, nPC = min(dim(Y)))
}
\arguments{
\item{Y}{The \eqn{n \times m} matrix with \eqn{n} observations and \eqn{m} (response) variables.}

\item{lmpDataList}{A list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}

\item{nPC}{Number of Principal Components to extract.}
}
\value{
A list containing the following elements:
\describe{
\item{\code{scores}}{Scores}
\item{\code{loadings}}{Loadings}
\item{\code{eigval}}{Eigenvalues}
\item{\code{singvar}}{Singular values}
\item{\code{var}}{Explained variances}
\item{\code{cumvar}}{Cumulated explained variances}
\item{\code{original.dataset}}{Original dataset}
\item{\code{design}}{Design of the study}

}
}
\description{
Operates a Principal Component Analysis on the \code{Y} outcome/response matrix by a singular
Value Decomposition (the pre-processing involves the mean-centering of \code{Y}).
Outputs are represented with the functions \code{\link{pcaScorePlot}},
\code{\link{pcaLoading1dPlot}}, \code{\link{pcaLoading2dPlot}} and \code{\link{pcaScreePlot}}.
}
\examples{

data("UCH")

PCA.res1 <- pcaBySvd(Y = UCH$outcomes)

PCA.res2 <- pcaBySvd(lmpDataList = UCH)

identical(PCA.res1, PCA.res2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaScreePlot.R
\name{pcaScreePlot}
\alias{pcaScreePlot}
\title{Scree Plot}
\usage{
pcaScreePlot(
  resPcaBySvd,
  nPC = 5,
  title = "PCA scree plot",
  theme = theme_bw()
)
}
\arguments{
\item{resPcaBySvd}{A list corresponding to the output value of \code{\link{pcaBySvd}}.}

\item{nPC}{An integer with the number of Principal Components to plot.}

\item{title}{Plot title.}

\item{theme}{\code{ggplot2} theme, see \code{?ggtheme} for more info.}
}
\value{
A \code{ggplot2} PCA scree plot
}
\description{
Returns a bar plot of the percentage of variance explained by each Principal Component calculated by \code{\link{pcaBySvd}}.
}
\examples{

data("UCH")
resPCA <- pcaBySvd(UCH$outcomes)
pcaScreePlot(resPCA, nPC = 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMeans.R
\name{plotMeans}
\alias{plotMeans}
\title{Means plot}
\usage{
plotMeans(
  Y = NULL,
  design = NULL,
  lmpDataList = NULL,
  cols = NULL,
  x,
  z = NULL,
  w = NULL,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  color = NULL,
  shape = NULL,
  linetype = NULL,
  size = 2,
  hline = NULL,
  theme = theme_bw()
)
}
\arguments{
\item{Y}{A numerical matrix containing the columns to be drawn. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{design}{A \eqn{n \times k} "freely encoded" experimental design data.frame. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{lmpDataList}{If not \code{NULL}, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}

\item{cols}{A vector with either the column name(s) of the \eqn{Y} matrix to plot (character) or the column index position(s) (integer).}

\item{x}{A character string giving the \code{design} factor whose levels will form the x-axis.}

\item{z}{A character string giving the \code{design} factor whose levels will form the traces.}

\item{w}{A character string giving the \code{design} factor whose levels will be used for the facet.}

\item{title}{Plot title.}

\item{xlab}{If not \code{NULL}, the label for the x-axis.}

\item{ylab}{If not \code{NULL}, the label for the y-axis.}

\item{color}{If not \code{NULL}, the color of the points and the line.}

\item{shape}{If not \code{NULL}, the points shape.}

\item{linetype}{If not \code{NULL}, the line type.}

\item{size}{Points size.}

\item{hline}{If not \code{NULL}, draws (a) horizontal line(s).}

\item{theme}{The \code{ggplot2} theme, see \code{?ggtheme} for more info.}
}
\value{
A list of \code{ggplot2} means plot(s).
}
\description{
For a given response variable, draws a plot of the response means by levels of up to three categorical factors from the design. When the design is balanced, it allows to visualize main effects or interactions for the response of interest. For unbalanced designs, this plot must be used with caution.
}
\details{
Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
The same rule applies for \code{design} or \code{lmpDataList}.
}
\examples{

data("UCH")
# 1 factor
plotMeans(
  Y = UCH$outcomes, design = UCH$design, cols = "4.0628702",
  x = "Hippurate", color = "blue"
)

# equivalent to:
plotMeans(
  lmpDataList = UCH, cols = "4.0628702",
  x = "Hippurate", color = "blue"
)

# 2 factors
plotMeans(
  Y = UCH$outcomes, design = UCH$design, cols = c(364, 365),
  x = "Hippurate", z = "Time", shape = c(15, 1)
)

# 3 factors
plotMeans(
  Y = UCH$outcomes, design = UCH$design, cols = c(364, 365),
  x = "Hippurate", z = "Time", w = "Citrate", linetype = c(3, 3)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotLine.R
\name{plotLine}
\alias{plotLine}
\title{Line plot}
\usage{
plotLine(
  Y = NULL,
  lmpDataList = NULL,
  rows = 1,
  type = c("l", "p", "s"),
  title = "Line plot",
  xlab = NULL,
  ylab = NULL,
  xaxis_type = c("numeric", "character"),
  stacked = FALSE,
  ncol = 1,
  nrow = NULL,
  facet_label = NULL,
  hline = 0,
  size = 0.5,
  color = NULL,
  shape = 1,
  theme = theme_bw(),
  ang_x_axis = NULL
)
}
\arguments{
\item{Y}{A numerical matrix containing the rows to be drawn. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{lmpDataList}{If not \code{NULL}, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}

\item{rows}{A vector with either the row name(s) of the \eqn{Y} matrix to plot (character) or the row index position(s) (integer). Default to 1.}

\item{type}{Type of graph to be drawn: \code{"p"} for point, \code{"l"} for line (default) or \code{"s"} for segment.}

\item{title}{Plot title.}

\item{xlab}{If not \code{NULL}, label for the x-axis.}

\item{ylab}{If not \code{NULL}, label for the y-axis.}

\item{xaxis_type}{The data type of the x-axis: either \code{"numeric"} (default) or \code{"character"}.}

\item{stacked}{Logical. If \code{TRUE}, will draw stacked plots, otherwise will draw separate plots.}

\item{ncol}{If \code{stacked} is \code{FALSE}, the number of columns to represent the separate plots. Default to \code{1}.}

\item{nrow}{If \code{stacked} is \code{FALSE}, the number of rows to represent the separate plots.}

\item{facet_label}{If \code{stacked} is \code{FALSE}, the labels of the separate plots.}

\item{hline}{If not \code{NULL}, draws (a) horizontal line(s), by default at y intercept = 0.}

\item{size}{Argument of length 1 giving the points size (if \code{type} ==\code{ "p"}) or the line size (if \code{type} == \code{"l"} or \code{"s"}).}

\item{color}{If not \code{NULL}, argument of length 1 with possible values: \code{"rows"}, a color name (character) or a numeric value representing a color.}

\item{shape}{The points shape (default = \code{1}) if \code{type} == \code{"p"}.}

\item{theme}{The \code{ggplot2} theme (default: \code{theme_bw()}), see \code{?ggtheme} for more info.}

\item{ang_x_axis}{If not \code{NULL}, rotation angle to rotate the x-axis text (based on the argument \code{axis.text.x} from \code{ggplot2::theme()})}
}
\value{
A \code{ggplot2} line plot.
}
\description{
Generates the response profile of one or more observations i.e. plots of one or more rows of the outcomes matrix on the y-axis against the \eqn{m} response variables on the x-axis. Depending on the response type (spectra, gene expression...), point, line or segment plots can be used.
}
\details{
Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
}
\examples{

data("UCH")
plotLine(Y = UCH$outcomes)

plotLine(lmpDataList = UCH)

# separate plots
plotLine(Y = UCH$outcomes, rows = seq(1, 8), hline = NULL)
plotLine(Y = UCH$outcomes, rows = seq(1, 8), color = 2)
plotLine(Y = UCH$outcomes, rows = seq(1, 8), ncol = 2)
plotLine(
  Y = UCH$outcomes, type = "p",
  rows = seq(1, 8), ncol = 2
)

# stacked plots
library(ggplot2)
plotLine(
  Y = UCH$outcomes, rows = seq(1, 1),
  stacked = TRUE, color = "rows"
) +
  scale_color_brewer(palette = "Set1")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDesign.R
\name{plotDesign}
\alias{plotDesign}
\title{Plot of the design matrix}
\usage{
plotDesign(
  design = NULL,
  lmpDataList = NULL,
  x = NULL,
  y = NULL,
  rows = NULL,
  cols = NULL,
  title = "Plot of the design",
  theme = theme_bw()
)
}
\arguments{
\item{design}{A data.frame representing the \eqn{n \times k} "freely encoded" experimental design. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{lmpDataList}{If not \code{NULL}, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}

\item{x}{By default, the first column of \code{design} ; otherwise if not \code{NULL}, a character string giving the column name of \code{design} to be used for the x-axis. The column needs to be a factor.}

\item{y}{By default, the second column of \code{design} if present ; otherwise if not \code{NULL}, a character string giving the column name of \code{design} to be used for the y-axis.}

\item{rows}{By default, the fourth column of \code{design} if present ; otherwise if not \code{NULL}, a character vector with one or several column name(s) of \code{design} to be used for faceting along the rows. The column needs to be a factor.}

\item{cols}{By default, the third column of \code{design} if present ; otherwise if not \code{NULL}, a character vector with one or several column name(s) of \code{design} to be used for faceting along the columns. The column needs to be a factor.}

\item{title}{Plot title.}

\item{theme}{The \code{ggplot2} theme, see \code{?ggtheme} for more info.}
}
\value{
A \code{ggplot2} plot of the design matrix.
}
\description{
Provides a graphical representation of the experimental design. It allows to visualize factors levels and check the design balance.
}
\details{
Either \code{design} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{design}.
The default behavior (parameters \code{x}, \code{y}, \code{cols} and \code{rows} are \code{NULL}) uses the first four columns of \code{df}. If at least one of these arguments is not \code{NULL}, the function will only use the non \code{NULL} parameters to be displayed.
}
\examples{
### trout data
data(trout)

plotDesign(design = trout$design, x = "Day", y = "Treatment")

# equivalent to:
plotDesign(lmpDataList = trout, x = "Day", y = "Treatment")

### mtcars
data(mtcars)
library(tidyverse)
df <- mtcars \%>\%
  dplyr::select(cyl, vs, am, gear, carb) \%>\%
  as.data.frame() \%>\%
  dplyr::mutate(across(everything(), as.factor))

# Default behavior: display the 4 first factors in the design
plotDesign(design = df)

# 2 factors
plotDesign(
  design = df, x = "cyl", y = "vs",
  cols = NULL, rows = NULL
)
# 3 factors
plotDesign(
  design = df, x = "cyl", y = "vs",
  cols = NULL, rows = c("am")
)
# 4 factors
plotDesign(
  design = df, x = "cyl", y = "vs",
  cols = c("gear"), rows = c("am")
)
# 5 factors
plotDesign(
  design = df, x = "cyl", y = "vs",
  cols = c("gear"), rows = c("am", "carb")
)

plotDesign(
  design = df, x = "cyl", y = "vs",
  cols = c("vs"), rows = c("am", "carb")
)

### UCH
data("UCH")
plotDesign(design = UCH$design, x = "Hippurate", y = "Citrate", rows = "Day")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpLoading2dPlot.R
\name{lmpLoading2dPlot}
\alias{lmpLoading2dPlot}
\title{Loading plots on a 2D scatter plot}
\usage{
lmpLoading2dPlot(
  resLmpPcaEffects,
  effectNames = NULL,
  axes = c(1, 2),
  addRownames = FALSE,
  pl_n = 10,
  metadata = NULL,
  drawOrigin = TRUE,
  ...
)
}
\arguments{
\item{resLmpPcaEffects}{A list corresponding to the output value of \code{\link{lmpPcaEffects}}.}

\item{effectNames}{Names of the effects to be plotted. If \code{NULL}, all the effects are plotted.}

\item{axes}{A numerical vector with the 2 Principal Components axes to be drawn.}

\item{addRownames}{Boolean indicating if the labels should be plotted. By default, uses the column names of the outcome matrix but it can be manually specified with the \code{points_labs} argument from \code{\link{plotScatter}}.}

\item{pl_n}{The number of labels that should be plotted, based on the distance measure \eqn{d} (\emph{see} Details).}

\item{metadata}{A nxk "free encoded" data.frame corresponding to \code{design} in \code{\link{plotScatter}}.}

\item{drawOrigin}{if \code{TRUE}, draws horizontal and vertical intercepts at (0,0) based on the \code{\link{plotScatter}} function.}

\item{...}{Additional arguments to be passed to \code{\link{plotScatter}}.}
}
\value{
A list of loading plots (ggplot).
}
\description{
Draws a 2D loading plot of each effect matrix provided in \code{\link{lmpPcaEffects}} outputs. As a wrapper of the \code{\link{plotScatter}} function, it allows the visualization of effect loading matrices for two components at a time with all options available in \code{\link{plotScatter}}.
}
\details{
\code{lmpLoading2dPlot} is a wrapper of \code{\link{plotScatter}}. See \code{?plotScatter} for more information on the additional arguments.

The distance measure \eqn{d}{d} that is used to rank the variables is based on the following formula:
\deqn{d = \sqrt(P_{ab}^2*\lambda_{ab}^2)}{d = sqrt(P_ab^2 * lambda_ab^2)} where \eqn{a}{a}
and \eqn{b}{b} are two selected Principal Components, \eqn{P_{ab}}{P_ab} represents their
loadings and \eqn{\lambda_{ab}}{lambda_ab} their singular values.
}
\examples{

data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resASCA <- lmpPcaEffects(resLmpEffectMatrices)

lmpLoading2dPlot(resASCA, effectNames = "Hippurate")

# adding color, shape and labels to points
id_hip <- c(seq(126, 156), seq(362, 375))
peaks <- rep("other", ncol(UCH$outcomes))
peaks[id_hip] <- "hip"
metadata <- data.frame(peaks)

lmpLoading2dPlot(resASCA,
  effectNames = "Hippurate",
  metadata = metadata, addRownames = TRUE, color = "peaks",
  shape = "peaks"
)

# changing max.overlaps of ggrepel
options(ggrepel.max.overlaps = 30)
lmpLoading2dPlot(resASCA,
  effectNames = "Hippurate",
  metadata = metadata, addRownames = TRUE, color = "peaks",
  shape = "peaks", pl_n = 20
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaLoading1dPlot.R
\name{pcaLoading1dPlot}
\alias{pcaLoading1dPlot}
\title{Loadings represented on a line plot.}
\usage{
pcaLoading1dPlot(resPcaBySvd, axes = c(1, 2), title = "PCA loading plot", ...)
}
\arguments{
\item{resPcaBySvd}{A list corresponding to the output value of \code{\link{pcaBySvd}}.}

\item{axes}{A numerical vector of length 2 with the Principal Components axes to be drawn.}

\item{title}{Plot title.}

\item{...}{Additional arguments to be passed to \code{\link{plotLine}}.}
}
\value{
A \code{ggplot2} object with the PCA loading plot.
}
\description{
Plots the loading vectors from \code{\link{pcaBySvd}} output with different available line types.
}
\details{
\code{pcaLoading1dPlot} is a wrapper of \code{\link{plotLine}}. See \code{?plotLine} for more information on the additional arguments.
}
\examples{

data("UCH")
ResPCA <- pcaBySvd(UCH$outcomes)

pcaLoading1dPlot(
  resPcaBySvd = ResPCA, axes = c(1, 2),
  title = "PCA loading plot UCH", xlab = "ppm", ylab = "Values"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpModelMatrix.R
\name{lmpModelMatrix}
\alias{lmpModelMatrix}
\title{Creates the model matrix \code{X}}
\usage{
lmpModelMatrix(lmpDataList)
}
\arguments{
\item{lmpDataList}{A list containing the outcomes, the experimental design and the formula.}
}
\value{
A list with the 5 following named elements :
\describe{
\item{\code{lmpDataList}}{The initial object: a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}
\item{\code{modelMatrix}}{A \emph{nxK} model matrix specifically encoded for the ASCA-GLM method.}
\item{\code{modelMatrixByEffect}}{A list of \emph{p} model matrices for each model effect.}
\item{\code{effectsNamesUnique}}{A character vector with the \emph{p} names of the model effects, each repeated once.}
\item{\code{effectsNamesAll}}{A character vector with the \emph{K} names of the model effects ordered and repeated as the column names of the model matrix.}
}
}
\description{
Creates the model matrix \code{X} from the design matrix and the model formula.
}
\details{
In typical ASCA-GLM (ASCA+) analysis, the effects of the GLM model must first be used to transform the design matrix to a model matrix where the design factors encoded usign \emph{sum coding} commonly used in industrial experimental design.
Suppose the design matrix is \emph{nxk} with n observations and \emph{k} factors. After the transformation, the model matrix
will be of size \emph{nxp}. For a fator with \emph{a} levels, the \emph{sum coding} creates \emph{a-1} columns in the model matrix with 0 and 1 for the \emph{a-1} first levels and -1 for the last one.
\emph{p} is the total number parameter for each response (outcome) in the ASCA model.
More information is available in the article (\emph{Thiel et al}, 2017)
Note that at the moment, only factors can be used as explanatory variables.
}
\examples{

data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)

head(resLmpModelMatrix$modelMatrix)

}
\references{
Thiel M.,Feraud B. and Govaerts B. (2017) \emph{ASCA+ and APCA+: Extensions of ASCA and APCA
in the analysis of unbalanced multifactorial designs}, Journal of Chemometrics
}
\seealso{
\code{\link{model.matrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpLoading1dPlot.R
\name{lmpLoading1dPlot}
\alias{lmpLoading1dPlot}
\title{Loadings represented on a line plot.}
\usage{
lmpLoading1dPlot(resLmpPcaEffects, effectNames = NULL, axes = c(1, 2), ...)
}
\arguments{
\item{resLmpPcaEffects}{A list corresponding to the output value of \code{\link{lmpPcaEffects}}.}

\item{effectNames}{Names of the effects to be plotted. if \code{NULL}, all the effects are plotted.}

\item{axes}{A numerical vector with the Principal Components axes to be drawn.}

\item{...}{Additional arguments to be passed to \code{\link{plotLine}} such as \code{xaxis_type}, \code{type} or \code{ang_x_axis}.}
}
\value{
A list of \code{ggplot} objects representing the loading plots.
}
\description{
Plots the loading vectors for each effect matrix from the \code{\link{lmpPcaEffects}} outputs with line plots. This is a wrapper of plotLine.
}
\details{
\code{lmpLoading1dPlot} is a wrapper of \code{\link{plotLine}}. See \code{?plotLine} for more information on the additional arguments.
}
\examples{

# Example of "spectral" type loadings (line and numerical x-axis)
data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resASCA <- lmpPcaEffects(resLmpEffectMatrices,
  combineEffects = list(c("Time", "Hippurate:Time"))
)
lmpLoading1dPlot(resASCA)
lmpLoading1dPlot(resASCA, effectNames = c("Hippurate", "Citrate"))

# Example of "segment" and discrete type loadings (segments and character x-axis)
data("trout")
resLmpModelMatrix <- lmpModelMatrix(trout)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resASCA <- lmpPcaEffects(resLmpEffectMatrices)
lmpLoading1dPlot(resASCA,
  effectNames = "Day",
  xaxis_type = "character", type = "s", ang_x_axis = 90
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpBootstrapTests.R
\name{lmpBootstrapTests}
\alias{lmpBootstrapTests}
\title{Tests the significance of model effects by bootstrap.}
\usage{
lmpBootstrapTests(
  resLmpEffectMatrices,
  nboot = 100,
  nCores = 2,
  verbose = FALSE
)
}
\arguments{
\item{resLmpEffectMatrices}{A list of 12 from \code{\link{lmpEffectMatrices}}.}

\item{nboot}{An integer with the number of bootstrap sample to be drawn.}

\item{nCores}{The number of cores to use for parallel execution.}

\item{verbose}{If \code{TRUE}, will display a message with the duration of execution.}
}
\value{
A list with the following elements:
\describe{
\item{\code{f.obs}}{A vector of size F (number of effects in the model) with the F statistics for each model term calculated on the initial data.}
\item{\code{f.boot}}{ b × F matrix with the F statistics calculated on the bootstrap samples.}
\item{\code{p.values}}{A vector of size F with the p-value for each model effect.}
\item{\code{resultsTable}}{A 2 × F matrix with the p-value and the percentage of variance for each model effect.}
}
}
\description{
Tests the significance of the effects from the model using bootstrap. This function is based on the outputs of \code{\link{lmpEffectMatrices}}. Tests on combined effects are also provided.
}
\examples{
data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix = resLmpModelMatrix)

res <- lmpBootstrapTests(
  resLmpEffectMatrices = resLmpEffectMatrices,
  nboot = 10, nCores = 2, verbose = TRUE
)

}
\references{
Thiel M.,Feraud B. and Govaerts B. (2017) \emph{ASCA+ and APCA+: Extensions of ASCA and APCA
in the analysis of unbalanced multifactorial designs}, Journal of Chemometrics

Thiel, M., Benaiche, N., Martin, M., Franceschini, S., Van Oirbeek, R., & Govaerts, B. (2023)
\emph{limpca: an R package for the linear modeling of high dimensional designed data based on
ASCA/APCA family of methods}, Journal of Chemometrics
}

\name{limpca}
\alias{limpca}
\docType{package}
\title{Linear modeling of high-dimensional designed data based on ASCA/APCA family of methods}

\description{
This package has for objectives to provide a method to make Linear Models for high-dimensional designed data. This method handles unbalanced design. More features should be included in the future (e.g. generalized linear models, random effects, ...).


\strong{The core functions of the package are:}

\describe{
  \item{\code{\link{data2LmpDataList}}}{Converts data to a lmpDataList, the input argument for \code{lmpModelMatrix}.}
  \item{\code{\link{lmpModelMatrix}}}{Creates the model matrix \eqn{\mathbf{X}} from the design matrix and the model formula.}
  \item{\code{\link{lmpEffectMatrices}}}{Estimates the model by OLS based on the outcomes and model matrices provided in the outputs of the \code{\link{lmpModelMatrix}} function and calculates the estimated effect matrices \eqn{\hat{\mathbf{M}}_0, \hat{\mathbf{M}}_1, ...\hat{\mathbf{M}}_F} and residual matrix \eqn{\hat{\mathbf{E}}}. It calculates also the type III percentage of variance explained by each effect.}
  \item{\code{\link{lmpBootstrapTests}}}{Tests the significance of one or a combination of the model effects using bootstrap. This function is based on the outputs of the \code{\link{lmpEffectMatrices}} function.}
  \item{\code{\link{lmpPcaEffects}}}{Performs a PCA on each of the effect matrices from the outputs of \code{\link{lmpEffectMatrices}}. It has an option to choose the method applied: ASCA, APCA or ASCA-E. Combined effects (i.e. linear combinations of original effect matrices) can also be created and decomposed by PCA.}
}

\strong{The functions allowing the visualisation of the Linear Models results are:}

\describe{
  \item{\code{\link{lmpScreePlot}}}{Provides a barplot of the percentage of variance associated to the PCs of the effect matrices ordered by importance based on the outputs of \code{\link{lmpContributions}}.}
  \item{\code{\link{lmpContributions}}}{This reports the contribution of each effect to the total variance, but also the contribution of each PC to the total variance per effect. Moreover, these contributions are summarized in a barplot.}
  \item{\code{\link{lmpScorePlot}}}{Draws the score plots of each effect matrix provided in the \code{\link{lmpPcaEffects}} function output.}
  \item{\code{\link{lmpLoading1dPlot}} or \code{\link{lmpLoading2dPlot}}}{Plots the loadings as a line plot (1D) or in 2D as a scatterplot.}
  \item{\code{\link{lmpScoreScatterPlotM}}}{Plots the scores of all model effects simultaneously in a scatterplot matrix. By default, the first PC only is kept for each model effect.}
  \item{\code{\link{lmpEffectPlot}}}{Plots the ASCA scores by effect levels for a given model effect and for one PC at a time. This graph is especially appealing to interpret interactions or combined effects.}
}

\strong{Other useful functions to visualise and explore by PCA the multivariate data are:}

\describe{
  \item{\code{\link{plotDesign}}}{Provides a graphical representation of the experimental design. It allows to visualize factor levels and check the design balance.}
  \item{\code{\link{plotScatter}}}{Produces a plot describing the relationship between two columns of the outcomes matrix \eqn{\mathbf{Y}}. It allows to choose colors and symbols for the levels of the design factors. Ellipses, polygons or segments can be added to group different sets of points on the graph.}
  \item{\code{\link{plotScatterM}}}{Produces a scatter plot matrix between the selected columns of the outcomes matrix \eqn{\mathbf{Y}} choosing specific colors and symbols for up to four factors from the design on the upper and lower diagonals.}
  \item{\code{\link{plotMeans}}}{Draws, for a given response variable, a plot of the response means by levels of up to three categorical factors from the design. When the design is balanced, it allows to visualize main effects or interactions for the response of interest. For unbalanced designs, this plot must be used with caution.}
  \item{\code{\link{plotLine}}}{Generates the response profile of one or more observations i.e. plots of one or more rows of the outcomes matrix on the y-axis against the m response variables on the x-axis. Depending on the response type (spectra, gene expression...), point, line or segment plots can be used.}
  \item{\code{\link{pcaBySvd}}}{Operates a principal component analysis on the \eqn{\mathbf{Y}} outcome/response matrix by a singular value decomposition. Outputs are can be visulised with the functions \code{\link{pcaScorePlot}}, \code{\link{pcaLoading1dPlot}}, \code{\link{pcaLoading2dPlot}} and \code{\link{pcaScreePlot}}.}
  \item{\code{\link{pcaScorePlot}}}{Produces score plots from the \code{\link{pcaBySvd}} output.}
  \item{\code{\link{pcaLoading1dPlot}} or \code{\link{pcaLoading2dPlot}}}{Plots the PCA loadings as a line plot (1D) or in 2D as a scatterplot.}
  \item{\code{\link{pcaScreePlot}}}{Returns a bar plot of the percentage of variance explained by each Principal Component (PC) calculated by \code{\link{pcaBySvd}}.}
 }

}

\details{

\tabular{ll}{
Package: \tab limpca\cr
Type: \tab Package\cr
License: \tab GPL-2\cr
}

See the package vignettes (\code{vignette(package = "limpca")}) for detailed case studies.
}


\references{
Thiel, M., Benaiche, N., Martin, M., Franceschini, S., Van Oirbeek, R., Govaerts, B. (2023). limpca: An R package for the linear modeling of high-dimensional designed data based on ASCA/APCA family of methods. \emph{Journal of Chemometrics}. e3482. \url{https://doi.org/10.1002/cem.3482}

Martin, M. (2020). \emph{Uncovering informative content in metabolomics data: from pre-processing of 1H NMR spectra to biomarkers discovery in multifactorial designs.}  Prom.: Govaerts, B. PhD thesis. Institut de statistique, biostatistique et sciences actuarielles, UCLouvain, Belgium. \url{http://hdl.handle.net/2078.1/227671}

Thiel M., Feraud B. and Govaerts B. (2017). ASCA+ and APCA+: Extensions of ASCA and APCA
in the analysis of unbalanced multifactorial designs. \emph{Journal of Chemometrics}. 31:e2895. \url{https://doi.org/10.1002/cem.2895}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpEffectPlot.R
\name{lmpEffectPlot}
\alias{lmpEffectPlot}
\title{Effect plot}
\usage{
lmpEffectPlot(
  resASCA,
  effectName,
  axes = 1,
  x,
  z = NULL,
  w = NULL,
  hline = 0,
  ...
)
}
\arguments{
\item{resASCA}{A list corresponding to the ASCA output value of \code{\link{lmpPcaEffects}}.}

\item{effectName}{Name of the effect to be used to plot the scores.}

\item{axes}{A numerical vector with the Principal Components axes to be drawn.}

\item{x}{A character string giving the \code{design} factor whose levels will form the x axis.}

\item{z}{A character string giving the \code{design} factor whose levels will form the traces.}

\item{w}{A character string giving the \code{design} factor whose levels will be used for the facet.}

\item{hline}{If not \code{NULL}, draws one or several horizontal line(s) at values given in \code{hline}.}

\item{...}{Additional arguments to be passed to \code{\link{plotMeans}}.}
}
\value{
An effect plot (ggplot).
}
\description{
Plots the ASCA scores by effect levels for a given model effect and for one PC at a time.
This graph is especially appealing to interpret interactions or combined effects.
It is a wrapper of plotMeans.
}
\details{
\code{lmpEffectPlot} is a wrapper of \code{\link{plotMeans}}.
}
\examples{

data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resASCA <- lmpPcaEffects(
  resLmpEffectMatrices = resLmpEffectMatrices,
  method = "ASCA", combineEffects = list(c("Hippurate", "Time", "Hippurate:Time"))
)

# Effect plot for an interaction effect
lmpEffectPlot(resASCA, effectName = "Hippurate:Time", x = "Hippurate", z = "Time")
# Effect plot for a combined effect
lmpEffectPlot(resASCA, effectName = "Hippurate+Time+Hippurate:Time", x = "Hippurate", z = "Time")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotScatter.R
\name{plotScatter}
\alias{plotScatter}
\title{Scatter plot}
\usage{
plotScatter(
  Y = NULL,
  design = NULL,
  lmpDataList = NULL,
  xy,
  color = NULL,
  shape = NULL,
  points_labs = NULL,
  title = "Scatter plot",
  xlab = NULL,
  ylab = NULL,
  size = 2,
  size_lab = 3,
  drawShapes = c("none", "ellipse", "polygon", "segment"),
  typeEl = c("norm", "t", "euclid"),
  levelEl = 0.9,
  alphaPoly = 0.4,
  theme = theme_bw(),
  drawOrigin = FALSE
)
}
\arguments{
\item{Y}{A \eqn{n \times m} matrix with \eqn{n} observations and \eqn{m} variables. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{design}{A \eqn{n \times k} "freely encoded" experimental design data.frame. Can be \code{NULL} if \code{lmpDataList} is defined.}

\item{lmpDataList}{If not \code{NULL}, a list with outcomes, design and formula, as outputted by \code{\link{data2LmpDataList}}.}

\item{xy}{x- and y-axis values: a vector of length 2 with either the column name(s) of the \eqn{Y} matrix to plot (character) or the index position(s) (integer).}

\item{color}{If not \code{NULL}, a character string giving the column name of \code{design} to be used as color. Currently treated as a discrete variable.}

\item{shape}{If not \code{NULL}, a character string giving the column name of \code{design} to be used as shape. Currently treated as a discrete variable.}

\item{points_labs}{If not \code{NULL}, a character vector with point labels.}

\item{title}{Plot title.}

\item{xlab}{If not \code{NULL}, label for the x-axis.}

\item{ylab}{If not \code{NULL}, label for the y-axis.}

\item{size}{The points size, by default \code{2}.}

\item{size_lab}{The size of points labels, by default \code{3}.}

\item{drawShapes}{Multiple shapes can be drawn based on the \code{color}: \code{"none"} for no shape (default), \code{"ellipse"} (ellipses with \code{ggplot2::stat_ellipse()}), \code{"polygon"} (polygons with \code{ggplot2::geom_polygon()}) or \code{"segment"} (segment from the centroids with \code{ggplot2::geom_segment()}).}

\item{typeEl}{The type of ellipse, either \code{"norm"} (multivariate normal distribution, the default), \code{"t"} (multivariate t-distribution) or \code{"euclid"} (draws a circle with the radius equal to level, representing the euclidean distance from the center).}

\item{levelEl}{The confidence level at which to draw an ellipse, by default \code{0.9}.}

\item{alphaPoly}{The degree of transparency for polygons, by default \code{0.4}.}

\item{theme}{The \code{ggplot2} theme (default: \code{theme_bw()}), see \code{?ggtheme} for more info.}

\item{drawOrigin}{If \code{TRUE}, draws horizontal and vertical intercepts at (0,0).}
}
\value{
A \code{ggplot2} scatter plot.
}
\description{
Produces a plot describing the relationship between two columns of the outcomes matrix \eqn{Y}. Colors and symbols can be chosen for the levels of the design factors. Ellipses, polygons or segments can be added to group different sets of points on the graph.
}
\details{
Either \code{Y} or \code{lmpDataList} need to be defined. If both are given, the priority goes to \code{Y}.
The same rule applies for \code{design} or \code{lmpDataList}.
}
\examples{

data("UCH")

# Without the design info
plotScatter(Y = UCH$outcomes, xy = c(453, 369))

# equivalent to:
plotScatter(lmpDataList = UCH, xy = c(453, 369))

# With color and shape
plotScatter(
  lmpDataList = UCH,
  xy = c(453, 369), color = "Hippurate",
  shape = "Citrate"
)

# equivalent to:
plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), color = "Hippurate",
  shape = "Citrate"
)

# With color and shapes
plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), color = "Hippurate",
  drawShapes = "ellipse"
)

plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), color = "Hippurate",
  drawShapes = "polygon"
)

plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), color = "Hippurate",
  drawShapes = "segment"
)

# With customized shapes
library(ggplot2)
plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), shape = "Hippurate", size = 3
) +
  scale_discrete_identity(
    aesthetics = "shape",
    guide = "legend"
  )

plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), shape = "Hippurate"
) +
  scale_shape_discrete(solid = FALSE)

plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), shape = "Hippurate"
) +
  scale_shape_manual(values = c(15, 16, 17))

# With labels
plotScatter(
  Y = UCH$outcomes, design = UCH$design,
  xy = c(453, 369), points_labs = rownames(UCH$design)
)

}

\name{UCH}
\alias{UCH}
\docType{data}
\title{UCH: the Urine Citrate-Hippurate metabolomic dataset}

\description{This dataset comes from a 1H NMR analysis of urine of female rats with hippuric and citric acid were added to the samples in different known concentrations.}
\usage{data("UCH")}

\format{
A list of length 3: the experimental design (`design`), the outcomes for every observations (`outcomes`) and the formula considered to analyze the data (`formula`).
  \describe{
    \item{\code{design}}{A data.frame with the experimental design of 34 observations and 5 explanatory variables:
   \code{Hippurate}: concentration of hippuric acid;
   \code{Citrate}: concentration of citric acid;
   \code{Dilution}: dilution, here all the samples are diluted with a dilution rate of 50 \%;
   \code{Day}: for each medium, the preparation of the mixtures were performed in two series;
   \code{Time}: each mixture or experimental condition  was repeated twice.
  }
    \item{\code{outcomes}}{A numerical matrix with 34 observations and 600 response variables}
    \item{\code{formula}}{A character string with the suggested formula to analyze the data}
  }
}
\details{

The UCH vignette can be accessed with: (\code{print(vignette(topic = "UCH", package = "limpca"))}).

The database has been experimentally created in order to control the spectral locations of the biomarkers to find (i.e. \code{Hippurate} and \code{Citrate}). This property allows us to evaluate the performances of the data analysis of various statistical methods.
This urine experimental database is also designed in order to explore the influence on spectra of intra-sample 1H NMR replications (\code{Time}), and inter-day 1H NMR measurements (\code{Day}).

The model formula is:

\code{outcomes = Hippurate + Citrate + Time +
            Hippurate:Citrate + Time:Hippurate + Time:Citrate + Hippurate:Citrate:Time}
}
\source{
Martin, M. (2020). \emph{Uncovering informative content in metabolomics data: from pre-processing of 1H NMR spectra to biomarkers discovery in multifactorial designs.}  Prom.: Govaerts, B. PhD thesis. Institut de statistique, biostatistique et sciences actuarielles, UCLouvain, Belgium. \url{http://hdl.handle.net/2078.1/227671}
}

\examples{
data("UCH")
str(UCH)
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pcaScorePlot.R
\name{pcaScorePlot}
\alias{pcaScorePlot}
\title{Score plots}
\usage{
pcaScorePlot(
  resPcaBySvd,
  design = NULL,
  axes = c(1, 2),
  title = "PCA score plot",
  points_labs_rn = FALSE,
  ...
)
}
\arguments{
\item{resPcaBySvd}{A list corresponding to the output value of \code{\link{pcaBySvd}}.}

\item{design}{A \eqn{n \times k} "freely encoded" experimental design data.frame.}

\item{axes}{A numerical vector of length 2 with the Principal Components axes to be drawn.}

\item{title}{Plot title.}

\item{points_labs_rn}{Boolean indicating if the rownames of the scores matrix should be plotted.}

\item{...}{Additional arguments to be passed to \code{\link{plotScatter}}.}
}
\value{
A \code{ggplot2} PCA score plot.
}
\description{
Produces score plots from \code{\link{pcaBySvd}} output with the same graphical options as \code{\link{plotScatter}} as this is a wrapper of this function..
}
\details{
\code{pcaScorePlot} is a wrapper of \code{\link{plotScatter}}. See \code{?plotScatter} for more information on the additional arguments.
}
\examples{

data("UCH")

# design is explicitly defined
ResPCA <- pcaBySvd(Y = UCH$outcomes)

pcaScorePlot(
  resPcaBySvd = ResPCA, axes = c(1, 2),
  title = "PCA score plot UCH", design = UCH$design,
  color = "Hippurate", shape = "Citrate"
)

# design is recovered from lmpDataList through pcaBySvd()
ResPCA <- pcaBySvd(lmpDataList = UCH)

pcaScorePlot(
  resPcaBySvd = ResPCA, axes = c(1, 2),
  title = "PCA score plot UCH",
  color = "Hippurate", shape = "Citrate"
)

}

\name{trout}
\alias{trout}
\docType{data}
\title{trout: the Rainbow trouts transcriptomic dataset}

\description{
This dataset comes from the study of the modulation of immunity in rainbow trout (Oncorhynchus mykiss) by exposure to cadmium (Cd) combined with polyunsaturated fatty acids (PUFAs) enriched diets [Cornet et al., 2018].

The responses were quantified by measuring the modification of the expression of 15 immune-related genes (m = 15) by RT-qPCR (reverse transcription quantita- tive polymerase chain reaction). The experiment was carried out on 72 trouts and 3 factors were considered in the experimental design:
\describe{
    \item{\code{Day}}{Measurements on trouts were collected on days 28, 70 and 72}
    \item{\code{Treatment}}{Four polyunsaturated fatty acid diets: alpha-linolenic acid (ALA), linoleic acid (LA), eicosapentaenoic acid (EPA) and docosahexaenoic acid (DHA)}
    \item{\code{Exposure}}{Trouts were exposed (level = 2) or not (level = 0) to high cadmium concentrations.}
}
This gives a 3 × 4 × 2 factorial design.  Each of the 24 trials corresponds to a different aquarium. Three fish were analysed (3 replicates) for each condition, giving a total of 72 observations.

In the limpca vignette, some outliers are first detected and removed from the dataset. The data of each aquarium are then transformed in log10 and mean aggregated in order to avoid the use of an aquarium random factor in the statictical model. Data are then centered and scaled by column. The ASCA+/APCA+ analysis is then applyed on the transformed data.
}

\usage{data("trout")}
\format{
  A list of 3 : the experimental design, the outcomes for every observation and the formula considered to analyze the data.
  Caution ! Here, the data must first be aggregated before being analyzed with ASCA+ (see details in related vignette)
  \describe{
    \item{\code{outcomes}}{A dataset with 72 observations and 15 response variables}
    \item{\code{formula}}{The suggested formula to analyze the data}
    \item{\code{design}}{The experimental design of 72 observations and 4 explanatory variables}
  }
}
\details{

  The data must first be aggregated before being analyzed with limpca. This will remove the hierarchy in the design and allow to apply a classical fixed effect general linear model to the data. See more details in the trout vignette (\code{print(vignette(topic = "Trout", package = "limpca"))}).
}
\source{
  Cornet, V., Ouaach, A., Mandiki, S., Flamion, E., Ferain, A., Van Larebeke, M., Lemaire, B., Reyes Lopez F.,  Tort, L., Larondelle, Y. and Kestemont, P. (2018).
  Environmentally-realistic concentration of cadmium combined with polyunsaturated fatty acids enriched diets modulated non-specific immunity in rainbow trout.
  \emph{Aquatic Toxicology}, \bold{196}, 104--116. \url{https://doi.org/10.1016/j.aquatox.2018.01.012}

  Benaiche, N. (2022). \emph{Stabilisation of the R package LMWiRe – Linear Models for Wide Responses.}  Prom. : Govaerts, B. Master thesis. Institut de statistique, biostatistique et sciences actuarielles, UCLouvain, Belgium.
  \url{http://hdl.handle.net/2078.1/thesis:33996}
  }

\examples{
  data("trout")}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpScreePlot.R
\name{lmpScreePlot}
\alias{lmpScreePlot}
\title{Scree Plot}
\usage{
lmpScreePlot(
  resLmpContributions,
  effectNames = NULL,
  nPC = 5,
  theme = theme_bw()
)
}
\arguments{
\item{resLmpContributions}{A resLmpContributions list from the function \code{\link{lmpContributions}}.}

\item{effectNames}{Names of the effects to be plotted. if \code{NULL}, all the effects are plotted.}

\item{nPC}{An integer with the number of components to plot.}

\item{theme}{\code{ggplot} theme}
}
\value{
A scree plot (ggplot).
}
\description{
Provides a barplot of the percentage of variance associated to the PCs of the effect matrices ordered by importance based on the outputs of \code{\link{lmpContributions}}.
}
\examples{

data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resASCAE <- lmpPcaEffects(resLmpEffectMatrices, method = "ASCA-E")
resLmpContributions <- lmpContributions(resASCAE)
lmpScreePlot(resLmpContributions, effectNames = "Hippurate:Citrate", nPC = 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpContributions.R
\name{lmpContributions}
\alias{lmpContributions}
\title{Summary of the contributions of each effect}
\usage{
lmpContributions(resLmpPcaEffects, nPC = 5)
}
\arguments{
\item{resLmpPcaEffects}{A list corresponding to the output value of \code{\link{lmpPcaEffects}}.}

\item{nPC}{The number of Principal Components to display.}
}
\value{
A list of:
\describe{
\item{\code{totalContribTable}}{Table of the percentage of contribution of each effect to the total variance.}
\item{\code{effectTable}}{Table of the percentage of variance explained by each principal component in each model effect decomposition.}
\item{\code{contribTable}}{Table of the percentage of variance explained by each principal component of each effect reported to the percentage contribution of the given effect to the total variance.}
\item{\code{combinedEffectTable}}{Equivalent of the \emph{EffectTable} for combined effects.}
\item{\code{plotTotal}}{Plot of the ordered contributions of \emph{TotalContribTable}.}
\item{\code{plotContrib}}{Plot of the ordered contributions of \emph{ContribTable}.}
}
}
\description{
Reports the contribution of each effect to the total variance,
but also the contribution of each PC to the total variance per effect.
These contributions are also summarized in a barplot.
}
\examples{
data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resLmpPcaEffects <- lmpPcaEffects(resLmpEffectMatrices, method = "ASCA-E")

lmpContributions(resLmpPcaEffects)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpScoreScatterPlotM.R
\name{lmpScoreScatterPlotM}
\alias{lmpScoreScatterPlotM}
\title{Scatterplot matrix of effect matrices scores}
\usage{
lmpScoreScatterPlotM(
  resLmpPcaEffects,
  effectNames = NULL,
  PCdim = NULL,
  modelAbbrev = FALSE,
  ...
)
}
\arguments{
\item{resLmpPcaEffects}{A list corresponding to the output value of \code{\link{lmpPcaEffects}}.}

\item{effectNames}{A character vector with the name of the effects to plot.}

\item{PCdim}{A numeric vector with the same length than effectNames and indicating the number of component to plot.}

\item{modelAbbrev}{A logical whether to abbreviate the interaction terms or not.}

\item{...}{Additional arguments to be passed to \code{\link{plotScatterM}}.}
}
\value{
A matrix of graphs
}
\description{
Plots the scores of all model effects simultaneously in a scatterplot matrix.
By default, the first PC only is kept for each model effect and, as a wrapper of \code{\link{plotScatterM}}, the choice of symbols and colors to distinguish factor levels allows an enriched visualization of the factors’ effect on the responses.
}
\details{
\code{lmpScoreScatterPlotM} is a wrapper of \code{\link{plotScatterM}}.
}
\examples{

data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
ResLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)
resLmpPcaEffects <- lmpPcaEffects(ResLmpEffectMatrices, method = "ASCA-E")

lmpScoreScatterPlotM(resLmpPcaEffects,
  varname.colorup = "Citrate",
  varname.pchup = "Hippurate",
  varname.pchdown = "Day",
  varname.colordown = "Time"
)

# advanced setting
lmpScoreScatterPlotM(resLmpPcaEffects,
  modelAbbrev = FALSE,
  effectNames = c("Citrate", "Hippurate", "Hippurate:Citrate"),
  PCdim = c(2, 2, 2),
  varname.colorup = "Citrate",
  vec.colorup = c("red", "blue", "green"),
  varname.pchup = "Hippurate",
  vec.pchup = c(1, 2, 3),
  varname.pchdown = "Day",
  vec.pchdown = c(4, 5),
  varname.colordown = "Time",
  vec.colordown = c("brown", "grey")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmpScorePlot.R
\name{lmpScorePlot}
\alias{lmpScorePlot}
\title{Score plots of effect matrices}
\usage{
lmpScorePlot(resLmpPcaEffects, effectNames = NULL, axes = c(1, 2), ...)
}
\arguments{
\item{resLmpPcaEffects}{A list corresponding to the output value of \code{\link{lmpPcaEffects}}.}

\item{effectNames}{Names of the effects to be plotted. If \code{NULL}, all the effects are plotted.}

\item{axes}{A numerical vector with the 2 Principal Components axes to be drawn.}

\item{...}{Additional arguments to be passed to \code{\link{plotScatter}}.}
}
\value{
A list of score plots (ggplot).
}
\description{
Draws the score plots of each (augmented) effect matrix provided in \code{\link{lmpPcaEffects}}. As a wrapper of the \code{\link{plotScatter}} function, it allows to visualize the scores of the effect matrices for two components at a time with all the available options in  \code{\link{plotScatter}}.
}
\details{
\code{lmpScorePlot} is a wrapper of \code{\link{plotScatter}}.
}
\examples{

data("UCH")
resLmpModelMatrix <- lmpModelMatrix(UCH)
resLmpEffectMatrices <- lmpEffectMatrices(resLmpModelMatrix)

# PCA decomposition of effect matrices (ASCA)
resASCA <- lmpPcaEffects(resLmpEffectMatrices)
# Score plot of Hippurate effect matrix
lmpScorePlot(resASCA,
  effectNames = "Hippurate",
  color = "Hippurate", shape = "Hippurate"
)

# PCA decomposition of augmented effect matrices (APCA)
resASCA <- lmpPcaEffects(resLmpEffectMatrices, method = "APCA")
# Score plot of Hippurate augmented effect matrix
lmpScorePlot(resASCA,
  effectNames = "Hippurate",
  color = "Hippurate", shape = "Hippurate", drawShapes = "ellipse"
)
}
